@@ -6,14 +6,13 @@
 
 #include <fcntl.h>
 #include "fd.h"
-#include "term.h"
 #include "func.h"
 #include "kanji.h"
 
 #define	MAXTIMESTR	8
 
 typedef struct _attrib_t {
-	u_short mode;
+	u_int mode;
 #ifdef	HAVEFLAGS
 	u_long flags;
 #endif
@@ -30,12 +29,13 @@ extern int subwindow;
 extern int win_x;
 extern int win_y;
 extern int lcmdline;
+extern int tradlayout;
 extern int sizeinfo;
 extern int mark;
 #ifdef	HAVEFLAGS
 extern u_long flaglist[];
 #endif
-#if	!MSDOS && !defined (_NOEXTRACOPY)
+#if	!defined (_USEDOSCOPY) && !defined (_NOEXTRACOPY)
 extern int inheritcopy;
 #endif
 
@@ -76,7 +76,7 @@ int copypolicy = 0;
 int removepolicy = 0;
 char *destpath = NULL;
 
-static u_short attrmode = 0;
+static u_int attrmode = 0;
 #ifdef	HAVEFLAGS
 static u_long attrflags = 0;
 #endif
@@ -112,7 +112,7 @@ char *path, *org;
 		return(0);
 	}
 
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 	if (dospath(path, NULL) != dospath(org, NULL)) path = NULL;
 	else
 #endif
@@ -128,7 +128,7 @@ char *path, *org;
 static int NEAR islowerdir(path, org)
 char *path, *org;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char orgpath[MAXPATHLEN];
 #endif
 	char *cp, *top, *cwd, tmp[MAXPATHLEN];
@@ -136,12 +136,12 @@ char *path, *org;
 
 	cwd = getwd2();
 	if (!org) org = cwd;
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	else org = nodospath(orgpath, org);
 #endif
 	strcpy(tmp, path);
 	top = tmp;
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 	if (dospath(path, NULL) != dospath(org, NULL)) {
 		free(cwd);
 		return(0);
@@ -188,7 +188,7 @@ char *mes, *arg;
 		dir[0] = '.';
 		dir[1] = '\0';
 	}
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 	else if (_dospath(dir) && !dir[2]) {
 		dir = realloc2(dir, 4);
 		dir[2] = '.';
@@ -278,10 +278,11 @@ struct stat *stp1, *stp2;
 		if (!n || n == 2) {
 			locate(0, L_CMDLINE);
 			putterm(l_clear);
-			putch2('[');
 			cp = SAMEF_K;
-			kanjiputs2(dest, n_column - (int)strlen(cp) - 1, -1);
+			i = strlen2(cp) - (sizeof("%.*s") - 1);
+			cp = asprintf3(cp, n_column - i, dest);
 			kanjiputs(cp);
+			free(cp);
 		}
 		if (!n) {
 			str[0] = UPDAT_K;
@@ -336,12 +337,10 @@ struct stat *stp1, *stp2;
 /*NOTREACHED*/
 				break;
 			case 2:
-				if ((cp = strrdelim(dest, 1))) cp++;
-				else cp = dest;
 				lcmdline = L_INFO;
 				if (!(tmp = inputstr(NEWNM_K, 1,
 					-1, NULL, -1))) return(-1);
-				strcpy(cp, tmp);
+				strcpy(getbasename(dest), tmp);
 				free(tmp);
 				if (Xlstat(dest, stp2) < 0) {
 					stp2 -> st_mode = S_IWRITE;
@@ -387,6 +386,9 @@ struct stat *stp1, *stp2;
 				break;
 		}
 	}
+
+/*NOTREACHED*/
+	return(0);
 }
 
 static int NEAR checkrmv(path, mode)
@@ -394,7 +396,7 @@ char *path;
 int mode;
 {
 	char *cp, *str[4];
-	int val[4];
+	int len, val[4];
 #if	MSDOS
 
 	if (Xaccess(path, mode) >= 0) return(0);
@@ -438,22 +440,27 @@ int mode;
 		int duperrno;
 
 		duperrno = errno;
+# ifdef	NOUID
+		mode = logical_access(stp -> st_mode)
+# else
 		mode = logical_access(stp -> st_mode,
 			stp -> st_uid, stp -> st_gid);
+# endif
 		if (!(mode & F_ISWRI)) {
 			errno = duperrno;
 			warning(-1, path);
 			return(-1);
 		}
 	}
-#endif
+#endif	/* !MSDOS */
 	if (removepolicy > 0) return(removepolicy - 2);
 	locate(0, L_CMDLINE);
 	putterm(l_clear);
-	putch2('[');
 	cp = DELPM_K;
-	kanjiputs2(path, n_column - (int)strlen(cp) - 1, -1);
+	len = strlen2(cp) - (sizeof("%.*s") - 1);
+	cp = asprintf3(cp, n_column - len, path);
 	kanjiputs(cp);
+	free(cp);
 	str[0] = ANYES_K;
 	str[1] = ANNO_K;
 	str[2] = ANALL_K;
@@ -532,11 +539,11 @@ char *path;
 static int touchdir(path)
 char *path;
 {
-#if	MSDOS || !defined (_NOEXTRACOPY)
+#if	defined (_USEDOSCOPY) || !defined (_NOEXTRACOPY)
 	struct stat st;
 	char dest[MAXPATHLEN];
 
-# if	!MSDOS
+# if	!defined (_USEDOSCOPY)
 	if (!inheritcopy) return(0);
 # endif
 	if (getdestpath(path, dest, &st) < 0) return(-2);
@@ -548,7 +555,7 @@ char *path;
 	st.st_flags = 0xffffffff;
 # endif
 	if (touchfile(dest, &st) < 0) return(-1);
-#endif	/* MSDOS || !_NOEXTRACOPY */
+#endif	/* _USEDOSCOPY || !_NOEXTRACOPY */
 	return(0);
 }
 
@@ -590,18 +597,11 @@ char *path;
 int findfile(path)
 char *path;
 {
-	char *cp;
-
-	if ((cp = strrdelim(path, 1))) cp++;
-	else cp = path;
-
-	if (regexp_exec(findregexp, cp, 1)) {
+	if (regexp_exec(findregexp, getbasename(path), 1)) {
 		if (path[0] == '.' && path[1] == _SC_) path += 2;
 		locate(0, L_CMDLINE);
 		putterm(l_clear);
-		putch2('[');
-		kanjiputs2(path, n_column - 2, -1);
-		putch2(']');
+		cprintf2("[%.*k]", n_column - 2, path);
 		if (yesno(FOUND_K)) {
 			destpath = strdup2(path);
 			return(-2);
@@ -613,12 +613,7 @@ char *path;
 int finddir(path)
 char *path;
 {
-	char *cp;
-
-	if ((cp = strrdelim(path, 1))) cp++;
-	else cp = path;
-
-	if (regexp_exec(findregexp, cp, 1)) {
+	if (regexp_exec(findregexp, getbasename(path), 1)) {
 		if (yesno(FOUND_K)) {
 			destpath = strdup2(path);
 			return(-2);
@@ -627,8 +622,8 @@ char *path;
 	return(0);
 }
 
-static VOID NEAR showattr(listp, attr, y)
-namelist *listp;
+static VOID NEAR showattr(namep, attr, y)
+namelist *namep;
 attrib_t *attr;
 int y;
 {
@@ -636,7 +631,7 @@ int y;
 	char buf[WMODE + 1];
 	int x1, x2, w;
 
-	tm = localtime(&(listp -> st_mtim));
+	tm = localtime(&(namep -> st_mtim));
 	if (isbestomit()) {
 		x1 = n_column / 2 - 16;
 		x2 = n_column / 2 - 3;
@@ -654,9 +649,7 @@ int y;
 	locate(0, ++y);
 	putterm(l_clear);
 	locate(x1, y);
-	putch2('[');
-	kanjiputs2(listp -> name, w, 0);
-	putch2(']');
+	cprintf2("[%-*.*k]", w, w, namep -> name);
 	locate(x2 + 3, y);
 	kanjiputs(TOLD_K);
 	locate(x2 + 13, y);
@@ -667,11 +660,11 @@ int y;
 	locate(x1, y);
 	kanjiputs(TMODE_K);
 	locate(x2, y);
-	putmode(buf, listp -> st_mode);
-	cputs2(&(buf[1]));
+	putmode(buf, namep -> st_mode, 1);
+	cputs2(buf);
 	locate(x2 + 10, y);
-	putmode(buf, attr -> mode);
-	cputs2(&(buf[1]));
+	putmode(buf, attr -> mode, 1);
+	cputs2(buf);
 
 #ifdef	HAVEFLAGS
 	locate(0, ++y);
@@ -679,7 +672,7 @@ int y;
 	locate(x1, y);
 	kanjiputs(TFLAG_K);
 	locate(x2, y);
-	putflags(buf, listp -> st_flags);
+	putflags(buf, namep -> st_flags);
 	cputs2(buf);
 	locate(x2 + 10, y);
 	putflags(buf, attr -> flags);
@@ -710,14 +703,14 @@ int y;
 	putterm(l_clear);
 }
 
-int inputattr(listp, flag)
-namelist *listp;
+int inputattr(namep, flag)
+namelist *namep;
 int flag;
 {
 	struct tm *tm;
 	attrib_t attr;
 	char buf[WMODE + 1];
-	u_short tmp;
+	u_int tmp;
 	int i, ch, x, y, xx, yy, ymin, ymax, dupwin_x, dupwin_y;
 
 	dupwin_x = win_x;
@@ -730,16 +723,16 @@ int flag;
 	if (yy <= L_TITLE) yy = L_TITLE + 1;
 	xx = n_column / 2 + ((isbestomit()) ? 7 : 10);
 
-	attr.mode = listp -> st_mode;
+	attr.mode = namep -> st_mode;
 #ifdef	HAVEFLAGS
-	attr.flags = listp -> st_flags;
+	attr.flags = namep -> st_flags;
 #endif
-	tm = localtime(&(listp -> st_mtim));
+	tm = localtime(&(namep -> st_mtim));
 	snprintf2(attr.timestr[0], sizeof(attr.timestr[0]), "%02d-%02d-%02d",
 		tm -> tm_year % 100, tm -> tm_mon + 1, tm -> tm_mday);
 	snprintf2(attr.timestr[1], sizeof(attr.timestr[1]), "%02d:%02d:%02d",
 		tm -> tm_hour, tm -> tm_min, tm -> tm_sec);
-	showattr(listp, &attr, yy);
+	showattr(namep, &attr, yy);
 	y = ymin = (flag & 1) ? 0 : WMODELINE;
 	ymax = (flag & 2) ? WMODELINE + 1 : 0;
 	x = 0;
@@ -841,7 +834,7 @@ int flag;
 				while (yy + WMODELINE + 5 > n_line - 1) yy--;
 				if (yy <= L_TITLE) yy = L_TITLE + 1;
 				xx = n_column / 2 + ((isbestomit()) ? 7 : 10);
-				showattr(listp, &attr, yy);
+				showattr(namep, &attr, yy);
 				break;
 			case ' ':
 #ifdef	HAVEFLAGS
@@ -874,8 +867,8 @@ int flag;
 				attr.mode ^= tmp;
 #endif
 				locate(xx, yy + y + 2);
-				putmode(buf, attr.mode);
-				cputs2(&(buf[1]));
+				putmode(buf, attr.mode, 1);
+				cputs2(buf);
 				break;
 			default:
 				break;
@@ -915,11 +908,11 @@ int flag;
 #endif
 	attrtime = (flag & 2) ? timelocal2(tm) : (time_t)-1;
 	if (flag == 3) {
-		if (attrmode == listp -> st_mode) attrmode = 0xffff;
+		if (attrmode == namep -> st_mode) attrmode = 0xffff;
 #ifdef	HAVEFLAGS
-		if (attrflags == listp -> st_flags) attrflags = 0xffffffff;
+		if (attrflags == namep -> st_flags) attrflags = 0xffffffff;
 #endif
-		if (attrtime == listp -> st_mtim) attrtime = (time_t)-1;
+		if (attrtime == namep -> st_mtim) attrtime = (time_t)-1;
 	}
 	return(1);
 }
@@ -934,7 +927,7 @@ char *path;
 	st.st_flags = attrflags;
 #endif
 	st.st_mode = attrmode;
-#if	!MSDOS
+#ifndef	NOUID
 	st.st_gid = (gid_t)-1;
 #endif
 	return(touchfile(path, &st));
@@ -944,7 +937,7 @@ int applyfile(func, endmes)
 int (*func)__P_((char *));
 char *endmes;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char path[MAXPATHLEN];
 #endif
 	int i, ret, old, dupfilepos;
@@ -1005,7 +998,7 @@ int *maxp, depth;
 			continue;
 		dirlist = b_realloc(dirlist, *maxp, char *);
 		dirlist[(*maxp)++] = strdup2(dir);
-		if (!(d = depth));
+		if (!(d = depth)) /*EMPTY*/;
 		else if (d > 1) d--;
 		else continue;
 		dirlist = getdirtree(dir, dirlist, maxp, d);
@@ -1036,10 +1029,8 @@ int verbose;
 	if (verbose) {
 		locate(0, L_CMDLINE);
 		putterm(l_clear);
-		putch2('[');
 		cp = (dir[0] == '.' && dir[1] == _SC_) ? dir + 2 : dir;
-		kanjiputs2(cp, n_column - 2, -1);
-		putch2(']');
+		cprintf2("[%.*k]", n_column - 2, cp);
 		tflush();
 	}
 #ifdef	FAKEUNINIT
@@ -1085,9 +1076,7 @@ int verbose;
 		if (verbose) {
 			locate(0, L_CMDLINE);
 			putterm(l_clear);
-			putch2('[');
-			kanjiputs2(cp, n_column - 2, -1);
-			putch2(']');
+			cprintf2("[%.*k]", n_column - 2, cp);
 			tflush();
 		}
 	}
@@ -1141,7 +1130,7 @@ char *endmes;
 		realpath2(dir, path, 0);
 		dir = path;
 	}
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	else dir = nodospath(path, dir);
 #endif
 	return(_applydir(dir, funcf, funcd1, funcd2, order, endmes, verbose));
@@ -1232,7 +1221,7 @@ int tr;
 	else if (islowerdir(destpath, filelist[filepos].name))
 		warning(EINVAL, filelist[filepos].name);
 	else {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 		char path[MAXPATHLEN];
 #endif
 
