@@ -5,9 +5,7 @@
  */
 
 #include "fd.h"
-#include "term.h"
 #include "func.h"
-#include "kctype.h"
 #include "kanji.h"
 
 #ifndef	_NOARCHIVE
@@ -18,9 +16,6 @@
 
 #ifndef	_NODOSDRIVE
 extern int shutdrv __P_((int));
-# if	MSDOS
-extern int checkdrive __P_((int));
-# endif
 #endif
 #if	MSDOS
 extern char *unixgetcurdir __P_((char *, int));
@@ -28,6 +23,9 @@ extern char *unixgetcurdir __P_((char *, int));
 
 extern int mark;
 extern int stackdepth;
+#ifndef	_NOTRADLAYOUT
+extern int tradlayout;
+#endif
 extern int sizeinfo;
 extern off_t marksize;
 extern off_t totalsize;
@@ -62,7 +60,7 @@ static char *ign_lha[] = {
 	"Listing of archive : *",
 	"  Name          Original *",
 	"--------------*",
-# ifdef	FAKEMETA
+# if	defined (FAKEMETA) || !defined (BSPATHDELIM)
 	"* files * ???.?% ??-??-?? ??:??:??",
 # else
 	"* files * ???.?%% ??-??-?? ??:??:??",
@@ -80,13 +78,14 @@ static char *form_tar[] = {
 	"%a %u/%g %s %m %d %t %y %*f",		/* traditional */
 	"%a %u/%g %s %y-%m-%d %t %*f",		/* GNU >=1.12 */
 	"%a %u/%g %s %m %d %y %t %*f",		/* kmtar */
-#else
+#else	/* !MSDOS */
 	"%a %u/%g %s %m %d %t %y %*f",		/* SVR4 */
 	"%a %u/%g %s %y-%m-%d %t %*f",		/* GNU >=1.12 */
 	"%a %l %u %g %s %m %d %{yt} %*f",	/* pax */
 	"%10a %u/%g %s %m %d %t %y %*f",	/* tar (UXP/DS) */
 	"%9a %u/%g %s %m %d %t %y %*f",		/* traditional */
-#endif
+	"%a %u/%g %m %d %t %y %*f",		/* IRIX */
+#endif	/* !MSDOS */
 	NULL
 };
 #define	PM_LHA	form_lha, ign_lha, 0, 0
@@ -171,7 +170,7 @@ static VOID NEAR poparchdupl __P_((VOID_A));
 static int NEAR readattr __P_((namelist *, char *));
 #if	FD >= 2
 static char *NEAR checkspace __P_((char *, int *));
-# if	MSDOS || defined (_NOKANJIFCONV)
+# ifdef	_NOKANJIFCONV
 #define	readfname	strndup2
 # else
 static char *NEAR readfname __P_((char *, int));
@@ -212,7 +211,7 @@ launchtable launchlist[MAXLAUNCHTABLE] = {
 	{"*.tar.bz2",	"bzip2 -cd %S|tar tvf -",	PM_TAR, 0},
 	{"*.taz",	"gzip -cd %S|tar tvf -",	PM_TAR, 0},
 	{"*.tgz",	"gzip -cd %S|tar tvf -",	PM_TAR, 0},
-#else
+#else	/* !MSDOS */
 	{"*.lzh",	"lha l",		PM_LHA, 0},
 	{"*.tar",	"tar tvf",		PM_TAR, 0},
 	{"*.tar.Z",	"zcat %C|tar tvf -",	PM_TAR, 0},
@@ -221,7 +220,7 @@ launchtable launchlist[MAXLAUNCHTABLE] = {
 	{"*.taZ",	"zcat %C|tar tvf -",	PM_TAR, 0},
 	{"*.taz",	"gzip -cd %C|tar tvf -",	PM_TAR, 0},
 	{"*.tgz",	"gzip -cd %C|tar tvf -",	PM_TAR, 0},
-#endif
+#endif	/* !MSDOS */
 	{NULL,		NULL,			PM_NULL, 0}
 };
 archivetable archivelist[MAXARCHIVETABLE] = {
@@ -238,7 +237,7 @@ archivetable archivelist[MAXARCHIVETABLE] = {
 					"gzip -cd %S|tar xf - %TA", 0},
 	{"*.tgz",	"tar cf - %T|gzip -c > %C",
 					"gzip -cd %S|tar xf - %TA", 0},
-#else
+#else	/* !MSDOS */
 	{"*.lzh",	"lha aq %C %TA",	"lha xq %C %TA", 0},
 	{"*.tar",	"tar cf %C %T",		"tar xf %C %TA", 0},
 	{"*.tar.Z",	"tar cf - %T|compress -c > %C",
@@ -253,7 +252,7 @@ archivetable archivelist[MAXARCHIVETABLE] = {
 					"gzip -cd %C|tar xf - %TA", 0},
 	{"*.tgz",	"tar cf - %T|gzip -c > %C",
 					"gzip -cd %C|tar xf - %TA", 0},
-#endif
+#endif	/* !MSDOS */
 	{NULL,		NULL,			NULL, 0}
 };
 char archivedir[MAXPATHLEN];
@@ -263,7 +262,7 @@ char **browsevar = NULL;
 
 #if	FD >= 2
 static char *autoformat[] = {
-#if	MSDOS
+# if	MSDOS
 	"%*f\n%s %x %x %y-%m-%d %t %a",		/* LHa (-v) */
 	"%a %u/%g %s %m %d %t %y %*f",		/* tar (traditional) */
 	"%a %u/%g %s %y-%m-%d %t %*f",		/* tar (GNU >=1.12) */
@@ -273,7 +272,7 @@ static char *autoformat[] = {
 	" %s %x %x %x %y-%m-%d %t %*f",		/* pkunzip */
 	" %s %x %x %d %m %y %t %*f",		/* zoo */
 	"%1x %12f %s %x %x %y-%m-%d %t %a",	/* LHa (-l) */
-#else
+# else	/* !MSDOS */
 	"%a %u/%g %s %m %d %t %y %*f",		/* tar (SVR4) */
 	"%a %u/%g %s %y-%m-%d %t %*f",		/* tar (GNU >=1.12) */
 	"%a %l %u %g %s %m %d %{yt} %*f",	/* ls or pax */
@@ -283,7 +282,8 @@ static char *autoformat[] = {
 	"%10a %u/%g %s %m %d %t %y %*f",	/* tar (UXP/DS) */
 	"%9a %u/%g %s %m %d %t %y %*f",		/* tar (traditional) */
 	"%9a %u/%g %s %x %m %d %{yt} %*f",	/* LHa */
-#endif
+	"%a %u/%g %m %d %t %y %*f",		/* tar (IRIX) */
+# endif	/* !MSDOS */
 };
 #define	AUTOFORMATSIZ	((int)(sizeof(autoformat) / sizeof(char *)))
 #endif	/* FD >= 2 */
@@ -390,7 +390,7 @@ static VOID NEAR poparchdupl(VOID_A)
 	if (arcflist) {
 		for (i = 0; i < maxarcf; i++) {
 			free(arcflist[i].name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 			if (arcflist[i].linkname) free(arcflist[i].linkname);
 #endif
 		}
@@ -459,7 +459,7 @@ namelist *tmp;
 char *buf;
 {
 	int i, c, len;
-	u_short mode;
+	u_int mode;
 
 	if (!*buf) return(0);
 	len = strlen(buf);
@@ -576,21 +576,19 @@ int *scorep;
 	return(strndup2(s, len));
 }
 
-# if	!MSDOS && !defined (_NOKANJIFCONV)
+# ifndef	_NOKANJIFCONV
 static char *NEAR readfname(s, len)
 char *s;
 int len;
 {
-	char buf[MAXPATHLEN];
+	char *cp, *tmp;
 
-	s = strndup2(s, len);
-	if (kanjiconv2(buf, s, MAXPATHLEN - 1, fnamekcode, DEFCODE) != s) {
-		free(s);
-		s = strdup2(buf);
-	}
-	return(s);
+	cp = strndup2(s, len);
+	tmp = newkanjiconv(cp, fnamekcode, DEFCODE, L_FNAME);
+	if (tmp != cp) free(cp);
+	return(tmp);
 }
-# endif	/* !MSDOS && !_NOKANJIFCONV */
+# endif	/* !_NOKANJIFCONV */
 
 static int NEAR readfileent(tmp, line, form, skip)
 namelist *tmp;
@@ -598,7 +596,7 @@ char *line, *form;
 int skip;
 {
 	struct tm tm;
-#if	!MSDOS
+#ifndef	NOUID
 	uidtable *up;
 	gidtable *gp;
 #endif
@@ -611,7 +609,7 @@ int skip;
 	tmp -> name = NULL;
 	tmp -> st_mode = 0644;
 	tmp -> st_nlink = 1;
-#if	!MSDOS
+#ifndef	NOUID
 	tmp -> st_uid = (uid_t)-1;
 	tmp -> st_gid = (gid_t)-1;
 	tmp -> linkname = NULL;
@@ -665,7 +663,7 @@ int skip;
 			form = strchr2(s, '}');
 			if (!form || form <= s) {
 				if (tmp -> name) free(tmp -> name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 				if (tmp -> linkname) free(tmp -> linkname);
 #endif
 				return(-1);
@@ -702,7 +700,7 @@ int skip;
 				hit++;
 				break;
 			case 'u':
-#if	!MSDOS
+#ifndef	NOUID
 				if ((cp = evalnumeric(buf, &n, -1)) && !*cp)
 					tmp -> st_uid = n;
 				else if ((up = finduid(0, buf)))
@@ -711,7 +709,7 @@ int skip;
 				hit++;
 				break;
 			case 'g':
-#if	!MSDOS
+#ifndef	NOUID
 				if ((cp = evalnumeric(buf, &n, -1)) && !*cp)
 					tmp -> st_gid = n;
 				else if ((gp = findgid(0, buf)))
@@ -791,8 +789,17 @@ int skip;
 			case 'f':
 				for (i = 0; i < len; i++) {
 #if	MSDOS
+# ifdef	BSPATHDELIM
 					if (rawbuf[i] == '/') rawbuf[i] = _SC_;
-#endif
+# else
+					if (rawbuf[i] == '\\')
+						rawbuf[i] = _SC_;
+					if (iskanji1(rawbuf, i)) {
+						i++;
+						continue;
+					}
+# endif
+#endif	/* MSDOS */
 					if (!iswhitespace(rawbuf[i])) continue;
 					cp = &(rawbuf[i]);
 					for (cp++; iswhitespace(*cp); cp++);
@@ -800,7 +807,7 @@ int skip;
 						if (!ch) i = len;
 						break;
 					}
-#if	!MSDOS
+#ifndef	NOSYMLINK
 					if (cp[0] == '-' && cp[1] == '>')
 						break;
 #endif
@@ -815,7 +822,7 @@ int skip;
 				tmp -> name = readfname(rawbuf, i);
 				hit++;
 				err = 0;
-#if	!MSDOS
+#ifndef	NOSYMLINK
 				while (iswhitespace(*cp)) cp++;
 				if (ch && cp >= &(line[len])) break;
 				if (cp[0] != '-' || cp[1] != '>') break;
@@ -839,7 +846,7 @@ int skip;
 				free(buf);
 				free(rawbuf);
 				if (tmp -> name) free(tmp -> name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 				if (tmp -> linkname) free(tmp -> linkname);
 #endif
 				return(-1);
@@ -857,7 +864,7 @@ int skip;
 
 	if (score >= MAXSCORE || !(tmp -> name) || !*(tmp -> name)) {
 		if (tmp -> name) free(tmp -> name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 		if (tmp -> linkname) free(tmp -> linkname);
 #endif
 		return(MAXSCORE);
@@ -883,7 +890,7 @@ int skip;
 
 	tmp -> st_mtim = timelocal2(&tm);
 	tmp -> flags |=
-#if	MSDOS
+#ifdef	NOUID
 		logical_access(tmp -> st_mode);
 #else
 		logical_access(tmp -> st_mode, tmp -> st_uid, tmp -> st_gid);
@@ -981,7 +988,7 @@ launchtable *list;
 int max;
 {
 	struct tm tm;
-#if	!MSDOS
+#ifndef	NOUID
 	uidtable *up;
 	gidtable *gp;
 #endif
@@ -997,7 +1004,7 @@ int max;
 	tmp -> tmpflags = F_STAT;
 	tmp -> st_mode = 0;
 	tmp -> st_nlink = 1;
-#if	!MSDOS
+#ifndef	NOSYMLINK
 	tmp -> linkname = NULL;
 #endif
 	getfield(buf, line, skip, list, F_NAME);
@@ -1006,8 +1013,15 @@ int max;
 		return(-1);
 	}
 #if	MSDOS
+# ifdef	BSPATHDELIM
 	for (i = 0; buf[i]; i++) if (buf[i] == '/') buf[i] = _SC_;
-#endif
+# else
+	for (i = 0; buf[i]; i++) {
+		if (buf[i] == '\\') buf[i] = _SC_;
+		if (iskanji1(buf, i)) i++;
+	}
+# endif
+#endif	/* MSDOS */
 	if (isdelim(buf, i = (int)strlen(buf) - 1)) {
 		if (i > 0) buf[i] = '\0';
 		tmp -> st_mode |= S_IFDIR;
@@ -1020,7 +1034,7 @@ int max;
 	if ((tmp -> st_mode & S_IFMT) == S_IFDIR) tmp -> flags |= F_ISDIR;
 	else if ((tmp -> st_mode & S_IFMT) == S_IFLNK) tmp -> flags |= F_ISLNK;
 
-#if	!MSDOS
+#ifndef	NOUID
 	getfield(buf, line, skip, list, F_UID);
 	if ((cp = evalnumeric(buf, &n, -1)) && !*cp) tmp -> st_uid = n;
 	else tmp -> st_uid = ((up = finduid(0, buf))) ? up -> uid : (uid_t)-1;
@@ -1080,7 +1094,7 @@ int max;
 
 	tmp -> st_mtim = timelocal2(&tm);
 	tmp -> flags |=
-#if	MSDOS
+#ifdef	NOUID
 		logical_access(tmp -> st_mode);
 #else
 		logical_access(tmp -> st_mode, tmp -> st_uid, tmp -> st_gid);
@@ -1124,27 +1138,58 @@ char *file, *dir;
 		strcpy(strcpy2(strcatdelim2(arch, fullpath, file), ":"), dir);
 	}
 
+#ifndef	_NOTRADLAYOUT
+	if (istradlayout()) {
+		locate(0, TL_PATH);
+		putterm(l_clear);
+
+		locate(TC_PATH, TL_PATH);
+		putterm(t_standout);
+#ifndef	_NOBROWSE
+		if (browselist) {
+			cputs2(TS_BROWSE);
+			len = TD_BROWSE;
+		}
+		else
+#endif
+		{
+			cputs2(TS_ARCH);
+			len = TD_ARCH;
+		}
+		putterm(end_standout);
+		cprintf2("%-*.*k", len, len, arch);
+		free(arch);
+
+		locate(TC_MARK, TL_PATH);
+		cprintf2("%<*d", TD_MARK, mark);
+		putterm(t_standout);
+		cputs2(TS_MARK);
+		putterm(end_standout);
+		cprintf2("%<'*qd", TD_SIZE, marksize);
+
+		tflush();
+		return;
+	}
+#endif	/* !_NOTRADLAYOUT */
+
 	locate(0, L_PATH);
 	putterm(l_clear);
-	len = 0;
-	if (!isleftshift()) {
-		putch2(' ');
-		len++;
-	}
+
+	locate(C_PATH, L_PATH);
 	putterm(t_standout);
 #ifndef	_NOBROWSE
 	if (browselist) {
-		cputs2("Browse:");
-		len += 7;
+		cputs2(S_BROWSE);
+		len = D_BROWSE;
 	}
 	else
 #endif
 	{
-		cputs2("Arch:");
-		len += 5;
+		cputs2(S_ARCH);
+		len = D_ARCH;
 	}
 	putterm(end_standout);
-	kanjiputs2(arch, n_column - len, 0);
+	cprintf2("%-*.*k", len, len, arch);
 	free(arch);
 
 	tflush();
@@ -1157,7 +1202,7 @@ char *s1, *s2;
 
 	for (i = j = 0; s2[j]; i++, j++) {
 		if (s1[i] == s2[j]) {
-#if	MSDOS
+#ifdef	BSPATHDELIM
 			if (iskanji1(s1, i) && (s1[++i] != s2[++j] || !s2[j]))
 				return(s1[i] - s2[j]);
 #endif
@@ -1177,7 +1222,7 @@ char *s1, *s2;
 
 	for (i = j = 0; s1[i]; i++, j++) {
 		if (s1[i] == s2[j]) {
-#if	MSDOS
+#ifdef	BSPATHDELIM
 			if (iskanji1(s1, i)) {
 				if (s1[++i] != s2[++j]) return(0);
 				if (s1[i]) continue;
@@ -1270,7 +1315,7 @@ char **argv;
 static int NEAR matchlist(s, argv)
 char *s, **argv;
 {
-# if	!MSDOS
+# ifndef	PATHNOCASE
 	int duppathignorecase;
 # endif
 	reg_t *re;
@@ -1278,7 +1323,7 @@ char *s, **argv;
 	int i, len, ret;
 
 	if (!argv) return(0);
-# if	!MSDOS
+# ifndef	PATHNOCASE
 	duppathignorecase = pathignorecase;
 	pathignorecase = 0;
 # endif
@@ -1301,17 +1346,17 @@ char *s, **argv;
 		free(s2);
 		regexp_free(re);
 	}
-# if	!MSDOS
+# ifndef	PATHNOCASE
 	pathignorecase = duppathignorecase;
 # endif
 	return(ret);
 }
 #endif	/* FD >= 2 */
 
-static int NEAR parsearchive(fp, list, flist, linenop)
+static int NEAR parsearchive(fp, list, namep, linenop)
 FILE *fp;
 launchtable *list;
-namelist *flist;
+namelist *namep;
 int *linenop;
 {
 #if	FD >= 2
@@ -1380,9 +1425,9 @@ int *linenop;
 		if (list -> field[i] < 255 && (int)(list -> field[i]) > max)
 			max = (int)(list -> field[i]);
 #endif	/* FD < 2 */
-	flist -> name = NULL;
-#if	!MSDOS
-	flist -> linkname = NULL;
+	namep -> name = NULL;
+#ifndef	NOSYMLINK
+	namep -> linkname = NULL;
 #endif
 
 	needline = 0;
@@ -1416,13 +1461,18 @@ int *linenop;
 			lvar[nline] = NULL;
 			if (nline < needline) {
 #if	FD >= 2
-				if (needline > 1 && na < AUTOFORMATSIZ)
+				if (needline > 1 && na < AUTOFORMATSIZ) {
+					if (!formlist[nf]) continue;
+					free(formlist[nf]);
+					for (i = nf; formlist[i]; i++)
+						formlist[i] = formlist[i + 1];
 					continue;
-				if (flist -> name) free(flist -> name);
-				flist -> name = NULL;
-# if	!MSDOS
-				if (flist -> linkname) free(flist -> linkname);
-				flist -> linkname = NULL;
+				}
+				if (namep -> name) free(namep -> name);
+				namep -> name = NULL;
+# ifndef	NOSYMLINK
+				if (namep -> linkname) free(namep -> linkname);
+				namep -> linkname = NULL;
 # endif
 				if (form0) free(form0);
 				free(scorelist);
@@ -1477,34 +1527,35 @@ int *linenop;
 
 		if (score >= MAXSCORE);
 		else if (!i) {
-			if (flist -> name) free(flist -> name);
-# if	!MSDOS
-			if (flist -> linkname) free(flist -> linkname);
+			if (namep -> name) free(namep -> name);
+# ifndef	NOSYMLINK
+			if (namep -> linkname) free(namep -> linkname);
 # endif
-			memcpy((char *)flist, (char *)&tmp, sizeof(tmp));
+			memcpy((char *)namep, (char *)&tmp, sizeof(tmp));
 		}
 		else {
 			if (tmp.name) free(tmp.name);
-# if	!MSDOS
+# ifndef	NOSYMLINK
 			if (tmp.linkname) free(tmp.linkname);
 # endif
 		}
 
 		if (!score) break;
 #else	/* FD < 2 */
-		score = readfileent(flist, cp, list, max);
+		score = readfileent(namep, cp, list, max);
 		free(cp);
 		break;
+/*NOTREACHED*/
 #endif	/* FD < 2 */
 	}
 
 #if	FD >= 2
 	if (score < 0) {
-		if (flist -> name) free(flist -> name);
-		flist -> name = NULL;
-# if	!MSDOS
-		if (flist -> linkname) free(flist -> linkname);
-		flist -> linkname = NULL;
+		if (namep -> name) free(namep -> name);
+		namep -> name = NULL;
+# ifndef	NOSYMLINK
+		if (namep -> linkname) free(namep -> linkname);
+		namep -> linkname = NULL;
 # endif
 	}
 #endif	/* FD >= 2 */
@@ -1527,16 +1578,15 @@ int *linenop;
 
 static VOID NEAR unpackerror(VOID_A)
 {
-	if (isttyiomode) {
-		locate(0, n_line - 1);
-		cputs2("\r\n");
-		tflush();
-		hideclock = 1;
-		warning(0, UNPNG_K);
+	if (!isttyiomode) {
+		kanjifputs(UNPNG_K, stderr);
+		fputnl(stderr);
 	}
 	else {
-		fputs(UNPNG_K, stderr);
-		fputc('\n', stderr);
+		locate(0, n_line - 1);
+		cputnl();
+		hideclock = 1;
+		warning(0, UNPNG_K);
 	}
 }
 
@@ -1562,7 +1612,7 @@ int argset;
 	maxfile = 0;
 	addlist();
 	filelist[0].name = strdup2("..");
-#if	!MSDOS
+#ifndef	NOSYMLINK
 	filelist[0].linkname = NULL;
 #endif
 	filelist[0].flags = F_ISDIR;
@@ -1584,7 +1634,7 @@ int argset;
 			parsearchive(NULL, NULL, NULL, NULL);
 			for (i = 0; i < maxfile; i++) {
 				free(filelist[i].name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 				if (filelist[i].linkname)
 					free(filelist[i].linkname);
 #endif
@@ -1604,7 +1654,7 @@ int argset;
 			filelist[maxfile].st_size = 0;
 			filelist[maxfile].flags |= F_ISDIR;
 			filelist[maxfile].name = dir;
-#if	!MSDOS
+#ifndef	NOSYMLINK
 			filelist[maxfile].linkname = NULL;
 #endif
 			filelist[maxfile].ent = no;
@@ -1612,7 +1662,7 @@ int argset;
 		}
 		if (!dir) {
 			free(tmp.name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 			if (tmp.linkname) free(tmp.linkname);
 #endif
 			continue;
@@ -1630,7 +1680,7 @@ int argset;
 	for (i = maxfile - 1; i > 0; i--) {
 		if (filelist[i].ent <= no) break;
 		free(filelist[i].name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 		if (filelist[i].linkname) free(filelist[i].linkname);
 #endif
 	}
@@ -1640,7 +1690,7 @@ int argset;
 		maxfile = 0;
 		free(filelist[0].name);
 		filelist[0].name = NOFIL_K;
-#if	!MSDOS
+#ifndef	NOSYMLINK
 		if (filelist[0].linkname) {
 			free(filelist[0].linkname);
 			filelist[0].linkname = NULL;
@@ -1655,7 +1705,7 @@ int argset;
 		if (i > 0) unpackerror();
 		for (i = 0; i < maxfile; i++) {
 			free(filelist[i].name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 			if (filelist[i].linkname) free(filelist[i].linkname);
 #endif
 		}
@@ -1682,7 +1732,7 @@ char *arcre;
 			memcpy((char *)&(filelist[0]),
 				(char *)&(arcflist[i]), sizeof(namelist));
 			filelist[0].name = arcflist[0].name;
-#if	!MSDOS
+#ifndef	NOSYMLINK
 			filelist[0].linkname = NULL;
 #endif
 			continue;
@@ -1702,7 +1752,7 @@ char *arcre;
 		memcpy((char *)&(filelist[maxfile]),
 			(char *)&(arcflist[i]), sizeof(namelist));
 		filelist[maxfile].name = cp;
-#if	!MSDOS
+#ifndef	NOSYMLINK
 		filelist[maxfile].linkname = arcflist[i].linkname;
 #endif
 		if (isfile(&(filelist[maxfile])))
@@ -1828,7 +1878,7 @@ char *path;
 			}
 			cp = file + (int)strlen(file) - 1;
 			while (cp > file && *cp == _SC_) cp--;
-#if	MSDOS
+#ifdef	BSPATHDELIM
 			if (onkanji1(file, cp - file)) cp++;
 #endif
 			cp = strrdelim2(file, cp);
@@ -1855,7 +1905,7 @@ char ***argvp;
 	char *cp, *tmp, *new, *file, dir[MAXPATHLEN], duparcdir[MAXPATHLEN];
 	int i, len;
 
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 	if (_dospath(path)) return(argc);
 #endif
 
@@ -1922,7 +1972,7 @@ int argset;
 	drive = 0;
 #endif
 
-	if (argset);
+	if (argset) /*EMPTY*/;
 	else if (archivefile) {
 		if (!(tmpdir = tmpunpack(1))) {
 			putterm(t_bell);
@@ -1939,7 +1989,7 @@ int argset;
 	if (!isarchbr(list)) {
 		execusercomm(list -> comm, filelist[filepos].name,
 			1, argset, 1);
-		if (argset);
+		if (argset) /*EMPTY*/;
 #ifndef	_NODOSDRIVE
 		else if (drive) removetmp(tmpdir, filelist[filepos].name);
 #endif
@@ -1948,7 +1998,7 @@ int argset;
 	}
 
 	pusharchdupl();
-	if (argset);
+	if (argset) /*EMPTY*/;
 #ifndef	_NODOSDRIVE
 	else if (drive) {
 		archduplp -> v_fullpath = strdup2(fullpath);
@@ -2028,13 +2078,13 @@ int launcher(VOID_A)
 #endif	/* !_NOBROWSE */
 
 	for (i = 0; i < maxlaunch; i++) {
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (launchlist[i].flags & LF_IGNORECASE) pathignorecase++;
 #endif
 		re = regexp_init(launchlist[i].ext, -1);
 		n = regexp_exec(re, filelist[filepos].name, 0);
 		regexp_free(re);
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (launchlist[i].flags & LF_IGNORECASE) pathignorecase--;
 #endif
 		if (n) break;
@@ -2055,14 +2105,14 @@ char *path, *file, *full, *tmpdir;
 	char *cp;
 
 	cp = file;
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 	if (_dospath(cp)) cp += 2;
 #endif
 
 #ifndef	_NODOSDRIVE
 	if (tmpdir) strcpy(path, tmpdir);
 	else
-#endif	/* !_NODOSDRIVE */
+#endif
 	if (*cp == _SC_) {
 		strcpy(path, file);
 		return(path);
@@ -2137,13 +2187,13 @@ char *arc;
 
 	for (i = 0; i < maxarchive; i++) {
 		if (!archivelist[i].p_comm) continue;
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (archivelist[i].flags & AF_IGNORECASE) pathignorecase++;
 #endif
 		re = regexp_init(archivelist[i].ext, -1);
 		n = regexp_exec(re, arc, 0);
 		regexp_free(re);
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (archivelist[i].flags & AF_IGNORECASE) pathignorecase--;
 #endif
 		if (n) break;
@@ -2245,13 +2295,13 @@ int tr, ignorelist;
 #endif
 	for (i = 0; i < maxarchive; i++) {
 		if (!archivelist[i].u_comm) continue;
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (archivelist[i].flags & AF_IGNORECASE) pathignorecase++;
 #endif
 		re = regexp_init(archivelist[i].ext, -1);
 		n = regexp_exec(re, arc, 0);
 		regexp_free(re);
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (archivelist[i].flags & AF_IGNORECASE) pathignorecase--;
 #endif
 		if (n) break;
@@ -2280,7 +2330,7 @@ int tr, ignorelist;
 		if (!*dir) strcpy(path, ".");
 		else {
 			cp = strcpy2(path, dir);
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 			if (_dospath(dir) && !dir[2]) strcpy(cp, ".");
 #endif
 		}
@@ -2314,7 +2364,7 @@ int tr, ignorelist;
 	dupmaxfile = maxfile;
 	dupfilepos = filepos;
 	alist[0].name = arc;
-#if	!MSDOS
+#ifndef	NOSYMLINK
 	alist[0].linkname = NULL;
 #endif
 	alist[0].st_mode = 0700;
@@ -2365,7 +2415,7 @@ int tr, ignorelist;
 char *tmpunpack(single)
 int single;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char tmp[MAXPATHLEN];
 #endif
 
@@ -2396,7 +2446,7 @@ int single;
 	while (*subdir == _SC_) subdir++;
 
 	if (ret < 0) putterm(t_bell);
-	else if (!ret);
+	else if (!ret) /*EMPTY*/;
 	else if (_chdir2(path) < 0) {
 		hideclock = 1;
 		warning(-1, path);
@@ -2497,13 +2547,13 @@ int maxf, n;
 
 	for (i = 0; i < maxlaunch; i++) {
 		if (!isarchbr(&(launchlist[i]))) continue;
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (launchlist[i].flags & LF_IGNORECASE) pathignorecase++;
 #endif
 		re = regexp_init(launchlist[i].ext, -1);
 		c = regexp_exec(re, file, 0);
 		regexp_free(re);
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (launchlist[i].flags & LF_IGNORECASE) pathignorecase--;
 #endif
 		if (c) break;
@@ -2572,7 +2622,7 @@ int maxf, n;
 		}
 		if (cp) match = no;
 		free(tmp.name);
-#if	!MSDOS
+#ifndef	NOSYMLINK
 		if (tmp.linkname) free(tmp.linkname);
 #endif
 	}
