@@ -5,20 +5,14 @@
  */
 
 #include "fd.h"
-#include "term.h"
 #include "func.h"
 #include "funcno.h"
-#include "kctype.h"
 #include "kanji.h"
 
 #ifndef	_NOORIGSHELL
 #include "system.h"
 #endif
 
-#ifndef	_NODOSDRIVE
-extern int flushdrv __P_((int, VOID_T (*)__P_((VOID_A))));
-#endif
-
 extern int curcolumns;
 extern int mark;
 extern off_t marksize;
@@ -37,6 +31,7 @@ extern int win_y;
 extern char *destpath;
 extern char *histfile;
 extern int savehist;
+extern int tradlayout;
 extern int sizeinfo;
 extern char fullpath[];
 #ifndef	_NOORIGSHELL
@@ -142,6 +137,9 @@ reg_t *findregexp = NULL;
 #ifndef	_NOWRITEFS
 int writefs = 0;
 #endif
+#if	FD >= 2
+int loopcursor = 0;
+#endif
 int internal_status = -2;
 bindtable bindlist[MAXBINDTABLE] = {
 	{K_UP,		CUR_UP,		255},
@@ -294,75 +292,146 @@ char *name;
 static int cur_up(arg)
 char *arg;
 {
-	int n;
+	int n, old;
 
-	if (filepos <= 0) return(0);
+	old = filepos;
 	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
-	if ((filepos -= n) < 0) filepos = 0;
-	return(2);
+	filepos -= n;
+#if	FD >= 2
+	if (loopcursor) {
+		int p;
+
+		p = FILEPERPAGE;
+		while (filepos < 0 || filepos / p < old / p)
+			filepos += p;
+		if (filepos >= maxfile) filepos = maxfile - 1;
+	}
+	else
+#endif
+	if (filepos < 0) filepos = 0;
+	return((filepos != old) ? 2 : 0);
 }
 
 static int cur_down(arg)
 char *arg;
 {
-	int n;
+	int n, old;
 
-	if (filepos >= maxfile - 1) return(0);
+	old = filepos;
 	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
-	if ((filepos += n) > maxfile - 1) filepos = maxfile - 1;
-	return(2);
+	filepos += n;
+#if	FD >= 2
+	if (loopcursor) {
+		int p;
+
+		p = FILEPERPAGE;
+		if (filepos >= maxfile) filepos = (filepos / p) * p;
+		while (filepos / p > old / p) {
+			filepos -= p;
+			if (filepos < 0) break;
+		}
+		if (filepos < 0) filepos = 0;
+	}
+	else
+#endif
+	if (filepos >= maxfile) filepos = maxfile - 1;
+	return((filepos != old) ? 2 : 0);
 }
 
 static int cur_right(arg)
 char *arg;
 {
-	int n;
+	int n, r, p, old;
 
+	old = filepos;
 	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
-	if (filepos + FILEPERROW * n < maxfile) filepos += FILEPERROW * n;
-	else if (filepos / FILEPERPAGE == (maxfile - 1) / FILEPERPAGE)
-		return(0);
+	r = FILEPERROW;
+	p = FILEPERPAGE;
+	filepos += r * n;
+
+#if	FD >= 2
+	if (loopcursor) {
+		if (filepos >= maxfile)
+			filepos = (filepos / p) * p + (old % r);
+		while (filepos / p > old / p) {
+			filepos -= p;
+			if (filepos < 0) break;
+		}
+		if (filepos < 0) filepos = 0;
+	}
+	else
+#endif
+	if (filepos < maxfile) /*EMPTY*/;
+	else if (old / p == (maxfile - 1) / p) filepos = old;
 	else {
-		filepos = ((maxfile - 1) / FILEPERROW) * FILEPERROW
-			+ filepos % FILEPERROW;
+		filepos = ((maxfile - 1) / r) * r + (old % r);
 		if (filepos >= maxfile) {
-			filepos -= FILEPERROW;
-			if (filepos / FILEPERPAGE
-			< (maxfile - 1) / FILEPERPAGE)
+			filepos -= r;
+			if (filepos / p < (maxfile - 1) / p)
 				filepos = maxfile - 1;
 		}
 	}
-	return(2);
+	return((filepos != old) ? 2 : 0);
 }
 
 static int cur_left(arg)
 char *arg;
 {
-	int n;
+	int n, r, old;
 
-	if (filepos < FILEPERROW) return(0);
+	old = filepos;
 	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
-	if (filepos - FILEPERROW * n >= 0) filepos -= FILEPERROW * n;
-	else filepos = filepos % FILEPERROW;
-	return(2);
+	r = FILEPERROW;
+	filepos -= r * n;
+
+#if	FD >= 2
+	if (loopcursor) {
+		int p;
+
+		p = FILEPERPAGE;
+		while (filepos < 0 || filepos / p < old / p)
+			filepos += p;
+		if (filepos >= maxfile) {
+			filepos = (maxfile - 1) / r * r + (old % r);
+			if (filepos >= maxfile) filepos -= r;
+		}
+	}
+	else
+#endif
+	if (filepos >= 0) /*EMPTY*/;
+	else filepos = old % r;
+	return((filepos != old) ? 2 : 0);
 }
 
 static int roll_up(arg)
 char *arg;
 {
-	int i;
+	int n, p, old;
 
-	i = (filepos / FILEPERPAGE + 1) * FILEPERPAGE;
-	if (i >= maxfile) return(warning_bell(arg));
-	filepos = i;
+	old = filepos;
+	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	p = FILEPERPAGE;
+	filepos = (filepos / p + n) * p;
+	if (filepos >= maxfile) {
+		filepos = old;
+		return(warning_bell(arg));
+	}
 	return(2);
 }
 
 static int roll_down(arg)
 char *arg;
 {
-	if (filepos < FILEPERPAGE) return(warning_bell(arg));
-	filepos = (filepos / FILEPERPAGE - 1) * FILEPERPAGE;
+	int n, p, old;
+
+	old = filepos;
+	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	p = FILEPERPAGE;
+	filepos = (filepos / p - n) * p;
+	if (filepos < 0) {
+		filepos = old;
+		return(warning_bell(arg));
+	}
 	return(2);
 }
 
@@ -399,7 +468,7 @@ char *arg;
 {
 	int i, m;
 
-#if	!MSDOS
+#ifndef	NOSYMLINK
 	if (islink(&(filelist[filepos]))) {
 # ifndef	_NODOSDRIVE
 		char path[MAXPATHLEN];
@@ -422,7 +491,7 @@ char *arg;
 		i = 1 - (strlen3(tmp) + 4);
 	}
 	else
-#endif	/* !MSDOS */
+#endif	/* !NOSYMLINK */
 	{
 		i = calcwidth();
 		if (isdisptyp(dispmode)) {
@@ -471,15 +540,21 @@ char *arg;
 
 static VOID NEAR markcount(VOID_A)
 {
-	char buf[14 + 1];
-	int x;
+#ifndef	_NOTRADLAYOUT
+	if (istradlayout()) {
+		locate(TC_MARK, TL_PATH);
+		cprintf2("%<*d", TD_MARK, mark);
+		locate(TC_MARK + TD_MARK + TW_MARK, TL_PATH);
+		cprintf2("%<'*qd", TD_SIZE, marksize);
 
-	x = (isleftshift()) ? 1 : 0;
-	locate(C_MARK + 5 - x, L_STATUS);
-	cputs2(ascnumeric(buf, (off_t)mark, 4, 4));
+		return;
+	}
+#endif
+	locate(C_MARK + W_MARK, L_STATUS);
+	cprintf2("%<*d", D_MARK, mark);
 	if (sizeinfo) {
-		locate(C_SIZE + 5 - x, L_SIZE);
-		cputs2(ascnumeric(buf, marksize, 3, 14));
+		locate(C_SIZE + W_SIZE, L_SIZE);
+		cprintf2("%<'*qd", D_SIZE, marksize);
 	}
 }
 
@@ -606,7 +681,7 @@ static int in_dir(arg)
 char *arg;
 {
 #ifndef	_NOARCHIVE
-	if (archivefile);
+	if (archivefile) /*EMPTY*/;
 	else
 #endif
 	if (!isdir(&(filelist[filepos]))
@@ -721,7 +796,7 @@ char *arg;
 	}
 	else
 #endif
-	if (chdir3(path) < 0) {
+	if (chdir3(path, 0) < 0) {
 		warning(-1, path);
 		free(path);
 		return(1);
@@ -753,9 +828,7 @@ char *arg;
 	char *path;
 
 	path = strdup2(_SS_);
-	if (chdir2(path) < 0) error(path);
-	if (findpattern) free(findpattern);
-	findpattern = NULL;
+	if (chdir3(path, 1) < 0) error(path);
 	replacefname(path);
 	return(4);
 }
@@ -766,7 +839,7 @@ char *file;
 {
 	FILE *fp;
 	char *buf, *prompt;
-	int i;
+	int i, len;
 
 	if (!(fp = Xfopen(file, "r"))) {
 		warning(-1, file);
@@ -776,8 +849,8 @@ char *file;
 	tflush();
 	buf = NEXT_K;
 	i = strlen3(file);
-	if (i + strlen(buf) > n_lastcolumn)
-		i = n_lastcolumn - (int)strlen(buf);
+	len = strlen2(buf);
+	if (i + len > n_lastcolumn) i = n_lastcolumn - len;
 	prompt = malloc2(i * KANAWID + strlen(buf) + 1);
 	strncpy3(prompt, file, &i, 0);
 	strcat(prompt, buf);
@@ -786,7 +859,7 @@ char *file;
 	while ((buf = fgets2(fp, 0))) {
 		locate(0, i);
 		putterm(l_clear);
-		kanjiputs2(buf, n_column, -1);
+		cprintf2("%.*k", n_column, buf);
 		free(buf);
 		if (++i >= n_line - 1) {
 			i = 0;
@@ -834,23 +907,12 @@ static int NEAR execshell(VOID_A)
 # endif
 	}
 #ifndef	_NOORIGSHELL
-	else {
-		char *cp;
-
-		if ((cp = strrdelim(sh, 1))) cp++;
-		else cp = sh;
-# if	MSDOS
-		if (*cp == 'r' || *cp == 'R') cp++;
-# else
-		if (*cp == 'r') cp++;
-# endif
-		if (!strpathcmp(cp, FDSHELL)) {
-			if (!fdmode) {
-				warning(0, RECUR_K);
-				return(-1);
-			}
-			sh = NULL;
+	else if (!strpathcmp(getshellname(sh, NULL, NULL), FDSHELL)) {
+		if (!fdmode) {
+			warning(0, RECUR_K);
+			return(-1);
 		}
+		sh = NULL;
 	}
 #endif
 
@@ -858,7 +920,7 @@ static int NEAR execshell(VOID_A)
 	mode = termmode(0);
 	nl = stdiomode();
 	kanjifputs(SHEXT_K, stderr);
-	fputc('\n', stderr);
+	fputnl(stderr);
 #ifdef	_NOORIGSHELL
 	ret = system(sh);
 #else
@@ -917,7 +979,7 @@ char *arg;
 	if (archivefile) removetmp(dir, NULL);
 	else
 #endif
-	;
+	/*EMPTY*/;
 	return(2);
 }
 
@@ -1141,7 +1203,7 @@ char *arg;
 static int rename_file(arg)
 char *arg;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char path[MAXPATHLEN];
 #endif
 	char *file;
@@ -1183,6 +1245,9 @@ char *arg;
 static int delete_file(arg)
 char *arg;
 {
+	char *cp;
+	int len;
+
 	removepolicy = 0;
 	if (mark > 0) {
 		if (!yesno(DELMK_K)) return(1);
@@ -1190,7 +1255,10 @@ char *arg;
 	}
 	else if (isdir(&(filelist[filepos]))) return(warning_bell(arg));
 	else {
-		if (!yesno(DELFL_K, filelist[filepos].name)) return(1);
+		cp = DELFL_K;
+		len = strlen2(cp) - (sizeof("%.*s") - 1);
+		if (!yesno(cp, n_lastcolumn - len, filelist[filepos].name))
+			return(1);
 		filepos = applyfile(rmvfile, NULL);
 	}
 	if (filepos >= maxfile && (filepos -= 2) < 0) filepos = 0;
@@ -1200,11 +1268,16 @@ char *arg;
 static int delete_dir(arg)
 char *arg;
 {
+	char *cp;
+	int len;
+
 	if (!isdir(&(filelist[filepos])) || isdotdir(filelist[filepos].name))
 		return(warning_bell(arg));
-	if (!yesno(DELDR_K, filelist[filepos].name)) return(1);
+	cp = DELDR_K;
+	len = strlen2(cp) - (sizeof("%.*s") - 1);
+	if (!yesno(cp, n_lastcolumn - len, filelist[filepos].name)) return(1);
 	removepolicy = 0;
-#if	!MSDOS
+#ifndef	NOSYMLINK
 	if (islink(&(filelist[filepos]))) {
 # ifndef	_NODOSDRIVE
 		char path[MAXPATHLEN];
@@ -1264,7 +1337,7 @@ char *arg;
 	if (!(cp = strrdelim(destpath, 0))) cp = destpath;
 	else {
 		*(cp++) = '\0';
-		chdir2(destpath);
+		chdir3(destpath, 1);
 	}
 
 	replacefname(strdup2(cp));
@@ -1305,14 +1378,14 @@ char *arg;
 #if	!defined (_NOARCHIVE) || !defined (_NODOSDRIVE)
 	char *dir;
 #endif
-	char *com;
-#if	!MSDOS || defined (DISMISS_CURPATH)
+#ifndef	CWDINPATH
 	char *tmp;
 #endif
-	int ret, len;
 #ifndef	_NODOSDRIVE
 	int drive;
 #endif
+	char *com;
+	int ret, len;
 
 #if	!defined (_NOARCHIVE) && !defined (_NOBROWSE)
 	if (browselist) return(warning_bell(arg));
@@ -1324,7 +1397,7 @@ char *arg;
 #ifndef	_NODOSDRIVE
 	drive = 0;
 #endif
-#if	MSDOS && !defined (DISMISS_CURPATH)
+#ifdef	CWDINPATH
 	len = (!isdir(&(filelist[filepos])) && isexec(&(filelist[filepos])))
 		? strlen(filelist[filepos].name) + 1 : 0;
 	ttyiomode(1);
@@ -1415,7 +1488,7 @@ char *arg;
 static int unpack_file(arg)
 char *arg;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char path[MAXPATHLEN];
 #endif
 	int ret;
@@ -1432,7 +1505,7 @@ char *arg;
 static int unpack_tree(arg)
 char *arg;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char path[MAXPATHLEN];
 #endif
 	int ret;
@@ -1491,7 +1564,7 @@ char *arg;
 		if (selectstr(&flag, 2, 35, str, val) == K_ESC) return(1);
 	}
 	else {
-#if	!MSDOS
+#ifndef	NOSYMLINK
 		if (islink(&(filelist[filepos]))) {
 			warning(0, ILLNK_K);
 			return(1);
@@ -1519,7 +1592,7 @@ char *arg;
 	char *path;
 
 	if (!(path = tree(0, NULL))) return(2);
-	if (chdir2(path) < 0) {
+	if (chdir3(path, 1) < 0) {
 		warning(-1, path);
 		free(path);
 		return(2);
@@ -1690,7 +1763,7 @@ char *arg;
 	if (++win >= windows) win = 0;
 	duplwin(oldwin);
 	movewin(oldwin);
-	if (chdir2(fullpath) < 0) lostcwd(fullpath);
+	if (chdir3(fullpath, 1) < 0) lostcwd(fullpath);
 	return(4);
 }
 #endif	/* !_NOSPLITWIN */
