@@ -7,7 +7,6 @@
 #include <fcntl.h>
 #include "fd.h"
 #include "func.h"
-#include "kctype.h"
 #include "kanji.h"
 
 #ifndef	ENOSPC
@@ -95,7 +94,7 @@ static VOID NEAR restorefile __P_((char *, char *, int));
 #endif	/* !_NOWRITEFS */
 
 char *deftmpdir = NULL;
-#if	!MSDOS && !defined (_NOEXTRACOPY)
+#if	!defined (_USEDOSCOPY) && !defined (_NOEXTRACOPY)
 int inheritcopy = 0;
 #endif
 int tmpumask = TMPUMASK;
@@ -105,7 +104,7 @@ static int dosdrv = -1;
 #endif
 
 
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 char *nodospath(path, file)
 char *path, *file;
 {
@@ -115,9 +114,9 @@ char *path, *file;
 	strcpy(&(path[2]), file);
 	return(path);
 }
-#endif
+#endif	/* _USEDOSEMU */
 
-#if	MSDOS
+#ifdef	NOUID
 int logical_access(mode)
 u_int mode;
 #else
@@ -127,122 +126,69 @@ uid_t uid;
 gid_t gid;
 #endif
 {
-#if	!MSDOS
-	struct group *gp;
-	uidtable *up;
-	uid_t euid;
-	int i;
-#endif
 	int dir;
 
 	dir = ((mode & S_IFMT) == S_IFDIR);
-#if	MSDOS
+#ifdef	NOUID
 	mode >>= 6;
-#else
-	euid = geteuid();
-	if (uid == euid) mode >>= 6;
+#else	/* !NOUID */
+	if (uid == geteuid()) mode >>= 6;
 	else if (gid == getegid()) mode >>= 3;
-	else if ((up = finduid(euid, NULL))) {
-# ifdef	DEBUG
-		_mtrace_file = "setgrent(start)";
-		setgrent();
-		if (_mtrace_file) _mtrace_file = NULL;
-		else {
-			_mtrace_file = "setgrent(end)";
-			malloc(0);	/* dummy alloc */
-		}
-		for (;;) {
-			_mtrace_file = "getgrent(start)";
-			gp = getgrent();
-			if (_mtrace_file) _mtrace_file = NULL;
-			else {
-				_mtrace_file = "getgrent(end)";
-				malloc(0);	/* dummy alloc */
-			}
-			if (!gp) break;
-			if (gid != gp -> gr_gid) continue;
-			for (i = 0; gp -> gr_mem[i]; i++) {
-				if (!strcmp(up -> name, gp -> gr_mem[i])) {
-					mode >>= 3;
-					break;
-				}
-			}
-			break;
-		}
-		_mtrace_file = "endgrent(start)";
-		endgrent();
-		if (_mtrace_file) _mtrace_file = NULL;
-		else {
-			_mtrace_file = "endgrent(end)";
-			malloc(0);	/* dummy alloc */
-		}
-# else	/* !DEBUG */
-		setgrent();
-		while ((gp = getgrent())) {
-			if (gid != gp -> gr_gid) continue;
-			for (i = 0; gp -> gr_mem[i]; i++) {
-				if (!strcmp(up -> name, gp -> gr_mem[i])) {
-					mode >>= 3;
-					break;
-				}
-			}
-			break;
-		}
-		endgrent();
-# endif	/* !DEBUG */
-	}
-#endif
+	else if (isgroupmember(gid)) mode >>= 3;
+#endif	/* !NOUID */
 	if (dir && !(mode & F_ISEXE)) mode &= ~(F_ISRED | F_ISWRI);
 	return(mode & 007);
 }
 
-int getstatus(list)
-namelist *list;
+int getstatus(namep)
+namelist *namep;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char path[MAXPATHLEN];
 #endif
 	struct stat st, lst;
 	char *cp;
 
-	cp = nodospath(path, list -> name);
+	cp = nodospath(path, namep -> name);
 	if (Xlstat(cp, &lst) < 0 || stat2(cp, &st) < 0) return(-1);
-	list -> flags = 0;
-	if ((st.st_mode & S_IFMT) == S_IFDIR) list -> flags |= F_ISDIR;
-	if ((lst.st_mode & S_IFMT) == S_IFLNK) list -> flags |= F_ISLNK;
+	namep -> flags = 0;
+	if ((st.st_mode & S_IFMT) == S_IFDIR) namep -> flags |= F_ISDIR;
+	if ((lst.st_mode & S_IFMT) == S_IFLNK) namep -> flags |= F_ISLNK;
 
 	if (isdisplnk(dispmode))
 		memcpy((char *)&lst, (char *)&st, sizeof(struct stat));
 #if	!MSDOS
 	if ((lst.st_mode & S_IFMT) == S_IFCHR
-	|| (lst.st_mode & S_IFMT) == S_IFBLK) list -> flags |= F_ISDEV;
+	|| (lst.st_mode & S_IFMT) == S_IFBLK) namep -> flags |= F_ISDEV;
 #endif
 
-	list -> st_mode = lst.st_mode;
-	list -> st_nlink = lst.st_nlink;
+	namep -> st_mode = lst.st_mode;
+	namep -> st_nlink = lst.st_nlink;
+#ifndef	NOUID
+	namep -> st_uid = lst.st_uid;
+	namep -> st_gid = lst.st_gid;
+#endif
 #if	MSDOS
-	list -> st_size = lst.st_size;
+	namep -> st_size = lst.st_size;
 #else
-	list -> st_uid = lst.st_uid;
-	list -> st_gid = lst.st_gid;
-	list -> st_size = isdev(list) ? (off_t)(lst.st_rdev) : lst.st_size;
+	namep -> st_size = isdev(namep) ? (off_t)(lst.st_rdev) : lst.st_size;
 #endif
 #ifdef	HAVEFLAGS
-	list -> st_flags = lst.st_flags;
+	namep -> st_flags = lst.st_flags;
 #endif
-	list -> st_mtim = lst.st_mtime;
-	list -> flags |=
-#if	MSDOS
+	namep -> st_mtim = lst.st_mtime;
+	namep -> flags |=
+#ifdef	NOUID
 		logical_access((u_int)(st.st_mode));
 #else
 		logical_access((u_int)(st.st_mode), st.st_uid, st.st_gid);
 #endif
 
-	if (isfile(list) && list -> st_size) {
-		totalsize += getblock(list -> st_size);
-		if (ismark(list)) marksize += getblock(list -> st_size);
+	if (isfile(namep) && namep -> st_size) {
+		totalsize += getblock(namep -> st_size);
+		if (ismark(namep)) marksize += getblock(namep -> st_size);
 	}
-	list -> tmpflags |= F_STAT;
+	namep -> tmpflags |= F_STAT;
 	return(0);
 }
 
@@ -250,50 +196,50 @@ int cmplist(vp1, vp2)
 CONST VOID_P vp1;
 CONST VOID_P vp2;
 {
-	namelist *list1, *list2;
+	namelist *namep1, *namep2;
 	char *cp1, *cp2;
 	int tmp;
 
-	list1 = (namelist *)vp1;
-	list2 = (namelist *)vp2;
-	if (!sorton) return(list1 -> ent - list2 -> ent);
+	namep1 = (namelist *)vp1;
+	namep2 = (namelist *)vp2;
+	if (!sorton) return(namep1 -> ent - namep2 -> ent);
 
 #ifndef	_NOPRECEDE
-	if (!havestat(list1) && getstatus(list1) < 0) return(1);
-	if (!havestat(list2) && getstatus(list2) < 0) return(-1);
+	if (!havestat(namep1) && getstatus(namep1) < 0) return(1);
+	if (!havestat(namep2) && getstatus(namep2) < 0) return(-1);
 #endif
 
-	if (!isdir(list1) && isdir(list2)) return(1);
-	if (isdir(list1) && !isdir(list2)) return(-1);
+	if (!isdir(namep1) && isdir(namep2)) return(1);
+	if (isdir(namep1) && !isdir(namep2)) return(-1);
 
-	if (list1 -> name[0] == '.' && list1 -> name[1] == '\0') return(-1);
-	if (list2 -> name[0] == '.' && list2 -> name[1] == '\0') return(1);
-	if (list1 -> name[0] == '.' && list1 -> name[1] == '.'
-	&& list1 -> name[2] == '\0') return(-1);
-	if (list2 -> name[0] == '.' && list2 -> name[1] == '.'
-	&& list2 -> name[2] == '\0') return(1);
+	if (namep1 -> name[0] == '.' && namep1 -> name[1] == '\0') return(-1);
+	if (namep2 -> name[0] == '.' && namep2 -> name[1] == '\0') return(1);
+	if (namep1 -> name[0] == '.' && namep1 -> name[1] == '.'
+	&& namep1 -> name[2] == '\0') return(-1);
+	if (namep2 -> name[0] == '.' && namep2 -> name[1] == '.'
+	&& namep2 -> name[2] == '\0') return(1);
 
 	switch (sorton & 7) {
 		case 5:
-			tmp = (int)strlen(list1 -> name)
-				- (int)strlen(list2 -> name);
+			tmp = (int)strlen(namep1 -> name)
+				- (int)strlen(namep2 -> name);
 			if (tmp) break;
 /*FALLTHRU*/
 		case 1:
-			tmp = strpathcmp2(list1 -> name, list2 -> name);
+			tmp = strpathcmp2(namep1 -> name, namep2 -> name);
 			break;
 		case 2:
-			if (isdir(list1)) {
-				tmp = strpathcmp2(list1 -> name,
-					list2 -> name);
+			if (isdir(namep1)) {
+				tmp = strpathcmp2(namep1 -> name,
+					namep2 -> name);
 				break;
 			}
-			cp1 = list1 -> name + strlen(list1 -> name);
-			cp2 = list2 -> name + strlen(list2 -> name);
+			cp1 = namep1 -> name + strlen(namep1 -> name);
+			cp2 = namep2 -> name + strlen(namep2 -> name);
 			for (;;) {
-				while (cp1 > list1 -> name)
+				while (cp1 > namep1 -> name)
 					if (*(--cp1) == '.') break;
-				while (cp2 > list2 -> name)
+				while (cp2 > namep2 -> name)
 					if (*(--cp2) == '.') break;
 				if (*cp2 != '.') {
 					tmp = (*cp1 == '.');
@@ -308,18 +254,18 @@ CONST VOID_P vp2;
 			}
 			break;
 		case 3:
-			if (isdir(list1))
-				tmp = strpathcmp2(list1 -> name,
-					list2 -> name);
-			else if (list1 -> st_size == list2 -> st_size)
+			if (isdir(namep1))
+				tmp = strpathcmp2(namep1 -> name,
+					namep2 -> name);
+			else if (namep1 -> st_size == namep2 -> st_size)
 				tmp = 0;
-			else tmp = (list1 -> st_size > list2 -> st_size)
+			else tmp = (namep1 -> st_size > namep2 -> st_size)
 				? 1 : -1;
 			break;
 		case 4:
-			if (list1 -> st_mtim == list2 -> st_mtim)
+			if (namep1 -> st_mtim == namep2 -> st_mtim)
 				tmp = 0;
-			else tmp = (list1 -> st_mtim > list2 -> st_mtim)
+			else tmp = (namep1 -> st_mtim > namep2 -> st_mtim)
 				? 1 : -1;
 			break;
 		default:
@@ -328,7 +274,7 @@ CONST VOID_P vp2;
 	}
 
 	if (sorton > 7) tmp = -tmp;
-	if (!tmp) tmp = list1 -> ent - list2 -> ent;
+	if (!tmp) tmp = namep1 -> ent - namep2 -> ent;
 
 	return(tmp);
 }
@@ -456,7 +402,7 @@ char *dir;
 	char *cp;
 
 	cp = dir;
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 	if (_dospath(dir)) cp += 2;
 #endif
 	if (!isdotdir(cp)) {
@@ -466,8 +412,7 @@ char *dir;
 #if	MSDOS
 				if (errno != EEXIST) return(-1);
 				if (cp[0] != _SC_ || cp[1]) {
-					if ((cp = strrdelim(dir, 1))) cp++;
-					else cp = dir;
+					cp = getbasename(dir);
 					if (cp[0] != '.' || cp[1]) return(-1);
 				}
 				st.st_mode = S_IFDIR;
@@ -553,16 +498,17 @@ struct stat *stp;
 	}
 #if	MSDOS
 	else if (!(st.st_mode & S_IWRITE)) Xchmod(path, st.st_mode);
-#else
+#endif
+#ifndef	NOUID
 	if (stp -> st_gid != (gid_t)-1) {
 # ifndef	_NODOSDRIVE
-		if (dospath(path, NULL));
+		if (dospath(path, NULL)) /*EMPTY*/;
 		else
 # endif
 		if (chown(path, stp -> st_uid, stp -> st_gid) < 0)
-			chown(path, -1, stp -> st_gid);
+			chown(path, (uid_t)-1, stp -> st_gid);
 	}
-#endif
+#endif	/* !NOUID */
 	if (ret < 0) errno = duperrno;
 	return(ret);
 }
@@ -730,7 +676,7 @@ struct stat *stp1, *stp2;
 #ifdef	HAVEFLAGS
 	stp1 -> st_flags = 0xffffffff;
 #endif
-#if	MSDOS
+#ifdef	_USEDOSCOPY
 	if (touchfile(dest, stp1) < 0) return(-1);
 #else
 # ifndef	_NOEXTRACOPY
@@ -754,7 +700,7 @@ struct stat *stp1, *stp2;
 #endif
 
 	for (;;) {
-		if (cpfile(src, dest, stp1, stp2) >= 0) return(0);
+		if (cpfile(src, dest, stp1, stp2) >= 0) break;
 		if (errno && errno != ENOSPC) return(-1);
 		for (;;) {
 			if (!yesno(NOSPC_K)) {
@@ -771,6 +717,8 @@ struct stat *stp1, *stp2;
 			break;
 		}
 	}
+
+	return(0);
 }
 
 int safemvfile(src, dest, stp1, stp2)
@@ -811,7 +759,7 @@ char *buf;
 int len;
 {
 	static char seq[] = {
-#if	MSDOS
+#ifdef	PATHNOCASE
 		'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
 		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
 		'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
@@ -948,7 +896,7 @@ char *file;
 	char *cp, path[MAXPATHLEN];
 	int fd, len, duperrno;
 
-	*path = '\0';
+	path[0] = '\0';
 	if (mktmpdir(path) < 0) return(-1);
 	cp = strcatdelim(path);
 	len = sizeof(path) - 1 - (cp - path);
@@ -995,11 +943,11 @@ char *path;
 VOID removetmp(dir, file)
 char *dir, *file;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char path[MAXPATHLEN];
 #endif
 
-	if (!dir || !*dir);
+	if (!dir || !*dir) /*EMPTY*/;
 	else if (_chdir2(dir) < 0) {
 		warning(-1, dir);
 		*dir = '\0';
@@ -1019,7 +967,7 @@ char *dir, *file;
 		lastdrv = dosdrv;
 		dosdrv = -1;
 	}
-#endif
+#endif	/* !_NODOSDRIVE */
 
 	if (_chdir2(fullpath) < 0) lostcwd(fullpath);
 	if (dir) {
@@ -1171,8 +1119,7 @@ int dos, boundary, dirsize, ofs;
 		lfn = dot = 0;
 		for (i = len = 0; s[i]; i++, len++) {
 			if (s[i] == '.') {
-				if (dot) lfn = 1;
-				else if (i > DOSBODYLEN) lfn = 1;
+				if (dot || !i || i > DOSBODYLEN) lfn = 1;
 				dot = i + 1;
 			}
 			else if (issjis1(s[i]) && issjis2(s[i + 1])) {
@@ -1181,7 +1128,7 @@ int dos, boundary, dirsize, ofs;
 			}
 			else if (!lfn && strchr(LFNONLY, s[i])) lfn = 1;
 		}
-		if (lfn);
+		if (lfn) /*EMPTY*/;
 		else if (dot) {
 			if (i - dot > DOSEXTLEN) lfn = 1;
 		}
@@ -1211,12 +1158,12 @@ int size, dos, boundary, dirsize, ofs;
 static int NEAR saferename(from, to)
 char *from, *to;
 {
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char fpath[MAXPATHLEN], tpath[MAXPATHLEN];
 #endif
 
 	if (!strpathcmp(from, to)) return(0);
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	from = nodospath(fpath, from);
 	to = nodospath(tpath, to);
 #endif
@@ -1242,7 +1189,7 @@ char *tmpdir, *old;
 
 	for (;;) {
 		genrandname(fname, len);
-#if	!MSDOS
+#ifndef	PATHNOCASE
 		if (dos) {
 			int i;
 
@@ -1255,13 +1202,13 @@ char *tmpdir, *old;
 			if (!isexist(path)) {
 				if (old) {
 #if	!MSDOS
-					if (isexist(fname));
+					if (isexist(fname)) /*EMPTY*/;
 					else
 #endif
 					if (saferename(old, fname) >= 0)
 						return(fname);
 #if	MSDOS
-					else if (errno == EACCES);
+					else if (errno == EACCES) /*EMPTY*/;
 #endif
 					else break;
 				}
@@ -1326,7 +1273,7 @@ off_t bsiz;
 
 	return(tmp);
 }
-#endif
+#endif	/* !MSDOS */
 
 static VOID NEAR restorefile(dir, path, fnamp)
 char *dir, *path;
@@ -1357,12 +1304,12 @@ int fs;
 #if	!MSDOS
 	off_t persec, totalent, dirblocksize;
 	u_char *entnum;
-	char **tmpfile;
-	int n, tmpno, block, ptr, totalptr;
+	char **tmpfiles;
+	int n, tmpno, block, ptr, totalptr, headbyte;
 #endif
 	DIR *dirp;
 	struct dirent *dp;
-	int dos, headbyte, boundary, dirsize, namofs;
+	int dos, boundary, dirsize, namofs;
 	int i, top, size, len, fnamp, ent;
 	char *cp, *tmpdir, **fnamelist, path[MAXPATHLEN];
 
@@ -1389,24 +1336,39 @@ int fs;
 			dirsize = 4;	/* short + short */
 			namofs = 3;
 			break;
-#endif
+# ifndef	_NODOSDRIVE
+		case 7:	/* Windows95 File System on DOSDRIVE */
+			dos = 3;
+			headbyte = -1;
+			boundary = LFNENTSIZ;
+			dirsize = DOSDIRENT;
+			namofs = 0;
+			break;
+# endif
+#endif	/* !MSDOS */
 		case 4:	/* MS-DOS File System */
 			dos = 1;
+#if	!MSDOS
 			headbyte = -1;
+#endif
 			boundary = 1;
 			dirsize = DOSDIRENT;
 			namofs = 0;
 			break;
 		case 5:	/* Windows95 File System */
 			dos = 2;
+#if	!MSDOS
 			headbyte = -1;
+#endif
 			boundary = LFNENTSIZ;
 			dirsize = DOSDIRENT;
 			namofs = 0;
 			break;
 		default:
 			dos = 0;
+#if	!MSDOS
 			headbyte = 0;
+#endif
 			boundary = 4;
 			dirsize = 8;	/* long + short + short */
 			namofs = 0;
@@ -1421,7 +1383,7 @@ int fs;
 			if (top < 0) top = i;
 			cp = convput(path, filelist[i].name, 1, 0, NULL, NULL);
 		}
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 		if (_dospath(cp)) cp += 2;
 #endif
 		fnamelist[i] = strdup2(cp);
@@ -1461,8 +1423,10 @@ int fs;
 #if	MSDOS
 			if (!(dp -> d_alias[0])) len = DOSBODYLEN;
 #else
-			if (dos == 2 && dp -> d_reclen == DOSDIRENT)
+# ifndef	_NODOSDRIVE
+			if (dos == 3 && dp -> d_reclen == DOSDIRENT)
 				len = DOSBODYLEN;
+# endif
 #endif
 			ent = i;
 		}
@@ -1511,8 +1475,8 @@ int fs;
 	ptr = 3;
 	totalptr = 0;
 	if (entnum) totalptr = entnum[block];
-	tmpfile = NULL;
-#endif
+	tmpfiles = NULL;
+#endif	/* !MSDOS */
 
 	for (i = 0; i < maxfile; i++) {
 		if (isdotdir(fnamelist[i]) || i == top) continue;
@@ -1536,8 +1500,8 @@ int fs;
 		if (ent < size) {
 			n = getnamlen(ent, dos, boundary, dirsize, namofs);
 			if (n > 0) {
-				tmpfile = b_realloc(tmpfile, tmpno, char *);
-				tmpfile[tmpno++] =
+				tmpfiles = b_realloc(tmpfiles, tmpno, char *);
+				tmpfiles[tmpno++] =
 					maketmpfile(n, dos, tmpdir, NULL);
 			}
 			ptr = 0;
@@ -1557,12 +1521,12 @@ int fs;
 	}
 #if	!MSDOS
 	if (entnum) free(entnum);
-	if (tmpfile) {
-		for (i = 0; i < tmpno; i++) if (tmpfile[i]) {
-			if (Xunlink(tmpfile[i]) < 0) warning(-1, tmpfile[i]);
-			free(tmpfile[i]);
+	if (tmpfiles) {
+		for (i = 0; i < tmpno; i++) if (tmpfiles[i]) {
+			if (Xunlink(tmpfiles[i]) < 0) warning(-1, tmpfiles[i]);
+			free(tmpfiles[i]);
 		}
-		free(tmpfile);
+		free(tmpfiles);
 	}
 #endif
 
