@@ -14,7 +14,7 @@ extern int errno;
 
 #if	MSDOS
 #include "unixemu.h"
-#else
+#else	/* !MSDOS */
 #include <sys/time.h>
 #include <sys/param.h>
 #include <sys/file.h>
@@ -29,7 +29,7 @@ extern int errno;
 # ifdef	USEUTIME
 # include <utime.h>
 # endif
-#endif
+#endif	/* !MSDOS */
 
 #ifdef	USETIMEH
 #include <time.h>
@@ -88,7 +88,7 @@ extern VOID saveorigenviron __P_((VOID_A));
 extern int loadruncom __P_((char *, int));
 
 /* dosemu.c or unixemu.c */
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 extern int _dospath __P_((char *));
 extern int dospath __P_((char *, char *));
 #endif
@@ -101,11 +101,6 @@ extern int dospath3 __P_((char *));
 #define	dospath3(path)	dospath(path, NULL)
 # endif
 #endif
-#if	!MSDOS && !defined (_NOKANJIFCONV)
-extern int getkcode __P_((char *));
-#endif
-extern char *convget __P_((char *, char *, int));
-extern char *convput __P_((char *, char *, int, int, char *, int *));
 extern DIR *Xopendir __P_((char *));
 extern int Xclosedir __P_((DIR *));
 extern struct dirent *Xreaddir __P_((DIR *));
@@ -205,11 +200,9 @@ extern int strncpy3 __P_((char *, char *, int *, int));
 extern int strlen2 __P_((char *));
 extern int strlen3 __P_((char *));
 extern int atoi2 __P_((char *));
-extern int fprintf2 __P_((FILE *, CONST char *, ...));
-extern int snprintf2 __P_((char *, int, CONST char *, ...));
+extern char *asprintf3 __P_((CONST char *, ...));
 extern VOID perror2 __P_((char *));
-extern int putenv2 __P_((char *));
-extern int setenv2 __P_((char *, char *));
+extern int setenv2 __P_((char *, char *, int));
 extern char *getenv2 __P_((char *));
 #ifdef	USESIGACTION
 extern sigcst_t signal2 __P_((int, sigcst_t));
@@ -225,13 +218,13 @@ extern time_t timelocal2 __P_((struct tm *));
 extern char *fgets2 __P_((FILE *, int));
 
 /* file.c */
-#if	MSDOS || defined (_NODOSDRIVE)
-#define	nodospath(p, f)	(f)
-#else
+#ifdef	_USEDOSEMU
 extern char *nodospath __P_((char *, char *));
+#else
+#define	nodospath(p, f)	(f)
 #endif
 #define	fnodospath(p, i)	nodospath(p, filelist[i].name)
-#if	MSDOS
+#ifdef	NOUID
 extern int logical_access __P_((u_int));
 #else
 extern int logical_access __P_((u_int, uid_t, gid_t));
@@ -287,7 +280,6 @@ extern int forcemovefile __P_((char *));
 /* parse.c */
 extern char *skipspace __P_((char *));
 extern char *evalnumeric __P_((char *, long *, int));
-extern char *ascnumeric __P_((char *, off_t, int, int));
 #ifdef	_NOORIGSHELL
 extern char *strtkchr __P_((char *, int, int));
 extern int getargs __P_((char *, char ***));
@@ -301,7 +293,7 @@ extern char *evalpaths __P_((char *, int));
 #else
 extern char *killmeta __P_((char *));
 #endif
-#if	!MSDOS && defined (_NOORIGSHELL)
+#ifdef	_NOORIGSHELL
 extern VOID adjustpath __P_((VOID_A));
 #endif
 extern char *includepath __P_((char *, char *));
@@ -309,38 +301,47 @@ extern char *includepath __P_((char *, char *));
 extern char *getrange __P_((char *, u_char *, u_char *, u_char *));
 #endif
 extern int evalprompt __P_((char **, char *));
-#if	FD >= 2
-extern char **duplvar __P_((char **, int));
-#endif
 #ifndef	_NOARCHIVE
 extern char *getext __P_((char *, u_char *));
 extern int extcmp __P_((char *, int, char *, int, int));
 #endif
 extern int getkeycode __P_((char *, int));
 extern char *getkeysym __P_((int, int));
-extern char *decodestr __P_((char *, int *, int));
-#if	!MSDOS && !defined (_NOKEYMAP)
+extern char *decodestr __P_((char *, u_char *, int));
+#ifndef	_NOKEYMAP
 extern char *encodestr __P_((char *, int));
 #endif
 
 /* builtin.c */
 #ifndef	_NOARCHIVE
-extern VOID printlaunchcomm __P_((int, int, FILE *));
-extern VOID printarchcomm __P_((int, FILE *));
+extern VOID freelaunch __P_((launchtable *));
+extern int searchlaunch __P_((launchtable *, int, launchtable *));
+extern int parselaunch __P_((int, char *[], launchtable *));
+extern VOID printlaunchcomm __P_((launchtable *, int, int, int, FILE *));
+extern VOID freearch __P_((archivetable *));
+extern int searcharch __P_((archivetable *, int, archivetable *));
+extern int parsearch __P_((int, char *[], archivetable *));
+extern VOID printarchcomm __P_((archivetable *, int, int, FILE *));
 # ifndef	_NOBROWSE
 extern VOID freebrowse __P_((launchtable *));
 # endif
 #endif
+extern int ismacro __P_((int));
 extern int freemacro __P_((int));
-extern VOID printmacro __P_((int, FILE *));
-#if	!MSDOS && !defined (_NODOSDRIVE)
-extern int searchdrv __P_((devinfo *, int, char *, int, int, int, int));
+extern int searchkeybind __P_((bindtable *, bindtable *));
+extern int parsekeybind __P_((int, char *[], bindtable *));
+extern char *gethelp __P_((bindtable *));
+extern VOID printmacro __P_((bindtable *, int, int, FILE *));
+#ifdef	_USEDOSEMU
+extern int searchdrv __P_((devinfo *, devinfo *, int));
 extern int deletedrv __P_((int));
-extern int insertdrv __P_((int, int, char *, int, int, int));
-extern VOID printsetdrv __P_((int, int, FILE *));
+extern int insertdrv __P_((int, devinfo *));
+extern VOID printsetdrv __P_((devinfo [], int, int, int, FILE *));
+extern int parsesetdrv __P_((int, char *[], devinfo *));
 #endif
-#if	!MSDOS && !defined (_NOKEYMAP)
-extern VOID printkeymap __P_((int, char *, int, FILE *));
+#ifndef	_NOKEYMAP
+extern int parsekeymap __P_((int, char *[], keyseq_t *));
+extern VOID printkeymap __P_((keyseq_t *, int, FILE *));
 #endif
 #ifdef	_NOORIGSHELL
 extern VOID printalias __P_((int, FILE *));
@@ -362,6 +363,7 @@ extern VOID freedefine __P_((VOID_A));
 #endif
 
 #define	BL_SET		"set"
+#define	BL_UNSET	"unset"
 #define	BL_PENV		"printenv"
 #define	BL_LAUNCH	"launch"
 #define	BL_ARCH		"arch"
@@ -392,10 +394,10 @@ extern VOID freedefine __P_((VOID_A));
 #define	BL_DOT		"."
 
 /* shell.c */
+extern char *restorearg __P_((char *));
 extern char *evalcommand __P_((char *, char *, macrostat *, int));
 extern int replaceargs __P_((int *, char ***, char **, int));
 extern int replacearg __P_((char **));
-extern int argfputs __P_((char *, FILE *));
 extern VOID demacroarg __P_((char **));
 extern char *inputshellstr __P_((char *, int, char *));
 extern char *inputshellloop __P_((int, char *));
@@ -423,36 +425,37 @@ extern int completeuserfunc __P_((char *, int, int, char ***));
 
 /* kanji.c */
 extern int onkanji1 __P_((char *, int));
-#if	(!MSDOS && !defined (_NOKANJICONV)) \
+#if	!defined (_NOKANJICONV) \
 || (!defined (_NOENGMES) && !defined (_NOJPNMES))
 extern int getlang __P_((char *, int));
 #endif
-#if	!MSDOS && (!defined (_NOKANJICONV) \
-|| (!defined (_NODOSDRIVE) && defined (CODEEUC)))
+#if	!defined (_NOKANJICONV) \
+|| (defined (_USEDOSEMU) && defined (CODEEUC))
 extern int sjis2ujis __P_((char *, u_char *, int));
 extern int ujis2sjis __P_((char *, u_char *, int));
 #endif
-#if	(!MSDOS && defined (FD) && (FD >= 2) && !defined (_NOKANJICONV)) \
-|| !defined (_NODOSDRIVE)
-extern VOID readunitable __P_((VOID_A));
+#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+extern VOID readunitable __P_((int));
 extern VOID discardunitable __P_((VOID_A));
 extern u_int unifysjis __P_((u_int, int));
 extern u_int cnvunicode __P_((u_int, int));
 #endif
-#if	!MSDOS && !defined (_NOKANJICONV)
+#ifndef	_NOKANJICONV
 extern int kanjiconv __P_((char *, char *, int, int, int, int));
 # ifndef	_NOKANJIFCONV
 extern char *kanjiconv2 __P_((char *, char *, int, int, int));
 # endif
 #endif
-extern int kanjiputs __P_((char *));
-extern int kanjifputs __P_((char *, FILE *));
-extern int kanjiprintf __P_((CONST char *, ...));
-extern int kanjiputs2 __P_((char *, int, int));
+#ifndef	_NOKANJIFCONV
+extern int getkcode __P_((char *));
+#endif
+extern char *convget __P_((char *, char *, int));
+extern char *convput __P_((char *, char *, int, int, char *, int *));
 
 /* input.c */
 extern int intrkey __P_((VOID_A));
 extern int Xgetkey __P_((int, int));
+extern int kanjiputs2 __P_((char *, int, int));
 extern char *inputstr __P_((char *, int, int, char *, int));
 extern int yesno __P_((CONST char *, ...));
 extern VOID warning __P_((int, char *));
@@ -508,14 +511,14 @@ extern VOID freestrarray __P_((char **, int));
 extern char **copystrarray __P_((char **, char **, int *, int));
 extern bindtable *copybind __P_((bindtable *, bindtable *));
 # ifndef	_NOARCHIVE
-extern VOID freelaunch __P_((launchtable *, int));
+extern VOID freelaunchlist __P_((launchtable *, int));
 extern launchtable *copylaunch __P_((launchtable *, launchtable *,
 	int *, int));
-extern VOID freearch __P_((archivetable *, int));
+extern VOID freearchlist __P_((archivetable *, int));
 extern archivetable *copyarch __P_((archivetable *, archivetable *,
 	int *, int));
 # endif
-# if	!MSDOS && !defined (_NODOSDRIVE)
+# ifdef	_USEDOSEMU
 extern VOID freedosdrive __P_((devinfo *));
 extern devinfo *copydosdrive __P_((devinfo *, devinfo *));
 # endif
@@ -527,7 +530,7 @@ extern int customize __P_((VOID_A));
 
 /* browse.c */
 extern VOID helpbar __P_((VOID_A));
-extern int putmode __P_((char *, u_int));
+extern int putmode __P_((char *, u_int, int));
 #ifdef	HAVEFLAGS
 extern int putflags __P_((char *, u_long));
 #endif
