@@ -6,9 +6,7 @@
 
 #include <signal.h>
 #include "fd.h"
-#include "term.h"
 #include "func.h"
-#include "kctype.h"
 #include "kanji.h"
 
 #ifndef	_NOORIGSHELL
@@ -19,6 +17,7 @@ extern char **history[];
 extern short histsize[];
 extern int curcolumns;
 extern int minfilename;
+extern int tradlayout;
 extern int sizeinfo;
 extern int hideclock;
 #ifdef	SIGALRM
@@ -40,6 +39,11 @@ extern char *promptstr2;
 #define	iseol(n, p, l)	(within(n, p, l) && !(((n) + (p)) % (l)))
 #define	LEFTMARGIN	0
 #define	RIGHTMARGIN	2
+#ifdef	SIGALRM
+#define	sigalrm(sig)	((!noalrm && (sig)) ? SIGALRM : 0)
+#else
+#define	sigalrm(sig)	0
+#endif
 
 static char *NEAR trquote __P_((char *, int, int *));
 static int NEAR vlen __P_((char *, int));
@@ -61,6 +65,7 @@ static VOID NEAR movecursor __P_((char *, char *, int));
 #endif
 static VOID NEAR Xlocate __P_((int, int));
 static VOID NEAR setcursor __P_((char *, int, int, int, int, int));
+static int NEAR putstr __P_((char *, char *, int, int, int, int, int));
 static VOID NEAR ringbell __P_((VOID_A));
 static VOID NEAR clearline __P_((VOID_A));
 static VOID NEAR newline __P_((int));
@@ -115,27 +120,27 @@ int dumbshell = 0;
 #endif
 
 #ifndef	_NOEDITMODE
-static int emulatekey[] = {
+static CONST int emulatekey[] = {
 	K_UP, K_DOWN, K_RIGHT, K_LEFT,
 	K_IC, K_DC, K_IL, K_DL,
 	K_HOME, K_END, K_BEG, K_EOL,
 	K_PPAGE, K_NPAGE, K_ENTER, K_ESC
 };
-static char emacskey[] = {
+static CONST char emacskey[] = {
 	K_CTRL('P'), K_CTRL('N'), K_CTRL('F'), K_CTRL('B'),
 	K_ESC, K_CTRL('D'), K_CTRL('Q'), K_CTRL('K'),
 	K_ESC, K_ESC, K_CTRL('A'), K_CTRL('E'),
 	K_CTRL('V'), K_CTRL('Y'), K_CTRL('O'), K_CTRL('G')
 };
 #define	EMACSKEYSIZ	((int)(sizeof(emacskey) / sizeof(char)))
-static char vikey[] = {
+static CONST char vikey[] = {
 	'k', 'j', 'l', 'h',
 	K_ESC, 'x', K_ESC, 'D',
 	'g', 'G', '0', '$',
 	K_CTRL('B'), K_CTRL('F'), 'o', K_ESC
 };
 #define	VIKEYSIZ	((int)(sizeof(vikey) / sizeof(char)))
-static char wordstarkey[] = {
+static CONST char wordstarkey[] = {
 	K_CTRL('E'), K_CTRL('X'), K_CTRL('D'), K_CTRL('S'),
 	K_CTRL('V'), K_CTRL('G'), K_CTRL(']'), K_CTRL('Y'),
 	K_CTRL('W'), K_CTRL('Z'), K_CTRL('A'), K_CTRL('F'),
@@ -160,7 +165,7 @@ int intrkey(VOID_A)
 {
 	int c;
 
-	if (kbhit2(0) && ((c = getkey2(0)) == cc_intr || c == K_ESC)) {
+	if (kbhit2(0L) && ((c = getkey2(0)) == cc_intr || c == K_ESC)) {
 		warning(0, INTR_K);
 		return(1);
 	}
@@ -185,18 +190,14 @@ int sig, eof;
 		return('\0');
 	}
 
-#ifdef	SIGALRM
-	sig = (noalrm || !sig) ? 0 : SIGALRM;
-#else
-	sig = 0;
-#endif
+	sig = sigalrm(sig);
 	ch = getkey2(sig);
 	if (eof && (ch != cc_eof || prev == ch)) eof = 0;
 	prev = ch;
 	if (eof) return(-1);
 
 #ifndef	_NOEDITMODE
-	if (!editmode) return(ch);
+	if (!editmode) /*EMPTY*/;
 	else if (!strcmp(editmode, "emacs")) {
 		for (i = 0; i < EMACSKEYSIZ; i++) {
 			if (emacskey[i] == K_ESC) continue;
@@ -294,39 +295,36 @@ char *s;
 int len, *widthp;
 {
 	char *cp;
-	int c, i, j, v;
+	int i, j, v;
 
 	cp = malloc2(len * 4 + 1);
 	v = 0;
 	for (i = j = 0; i < len; i++) {
+		if (i + 1 < len && iskanji1(s, i)) {
+			cp[j++] = s[i++];
+			cp[j++] = s[i];
+			v += 2;
+		}
 #ifdef	CODEEUC
-		if (i + 1 < len && isekana(s, i)) {
+		else if (i + 1 < len && isekana(s, i)) {
 			cp[j++] = s[i++];
 			cp[j++] = s[i];
 			v++;
 		}
 #else
-		if (iskna(s[i])) {
+		else if (isskana(s, i)) {
 			cp[j++] = s[i];
 			v++;
 		}
 #endif
-		else if (i + 1 < len && iskanji1(s, i)) {
-			cp[j++] = s[i++];
-			cp[j++] = s[i];
-			v += 2;
-		}
-		else if (i + 1 < len && s[i] == QUOTE) {
+		else if (iscntrl2(s[i])) {
 			cp[j++] = '^';
-			cp[j++] = (s[++i] + '@') & 0x7f;
+			cp[j++] = (s[i] + '@') & 0x7f;
 			v += 2;
 		}
 		else if (ismsb(s[i])) {
-			c = s[i] & 0xff;
-			cp[j++] = '\\';
-			cp[j++] = (c / (8 * 8)) + '0';
-			cp[j++] = ((c % (8 * 8)) / 8) + '0';
-			cp[j++] = (c % 8) + '0';
+			j += snprintf2(&(cp[j]), len * 4 + 1 - j,
+				"\\%03o", s[i] & 0xff);
 			v += 4;
 		}
 		else {
@@ -348,16 +346,19 @@ int cx;
 
 	v = r = 0;
 	while (r < cx) {
+		if (r + 1 < cx && iskanji1(s, r)) rw = vw = 2;
 #ifdef	CODEEUC
-		if (r + 1 < cx && isekana(s, r)) {
+		else if (r + 1 < cx && isekana(s, r)) {
 			rw = 2;
 			vw = 1;
 		}
 #else
-		if (iskna(s[r])) rw = vw = 1;
+		else if (isskana(s, r)) rw = vw = 1;
 #endif
-		else if (r + 1 < cx && (iskanji1(s, r) || s[r] == QUOTE))
-			rw = vw = 2;
+		else if (iscntrl2(s[r])) {
+			rw = 1;
+			vw = 2;
+		}
 		else if (ismsb(s[r])) {
 			rw = 1;
 			vw = 4;
@@ -378,16 +379,19 @@ int cx2;
 
 	v = r = 0;
 	while (v < cx2) {
+		if (v + 1 < cx2 && iskanji1(s, r)) rw = vw = 2;
 #ifdef	CODEEUC
-		if (isekana(s, r)) {
+		else if (isekana(s, r)) {
 			rw = 2;
 			vw = 1;
 		}
 #else
-		if (iskna(s[r])) rw = vw = 1;
+		else if (isskana(s, r)) rw = vw = 1;
 #endif
-		else if (v + 1 < cx2 && (iskanji1(s, r) || s[r] == QUOTE))
-			rw = vw = 2;
+		else if (v + 1 < cx2 && iscntrl2(s[r])) {
+			rw = 1;
+			vw = 2;
+		}
 		else if (v + 3 < cx2 && ismsb(s[r])) {
 			rw = 1;
 			vw = 4;
@@ -408,15 +412,19 @@ int cx2;
 
 	v = r = 0;
 	while (v < cx2) {
+		if (iskanji1(s, r)) rw = vw = 2;
 #ifdef	CODEEUC
-		if (isekana(s, r)) {
+		else if (isekana(s, r)) {
 			rw = 2;
 			vw = 1;
 		}
 #else
-		if (iskna(s[r])) rw = vw = 1;
+		else if (isskana(s, r)) rw = vw = 1;
 #endif
-		else if (iskanji1(s, r) || s[r] == QUOTE) rw = vw = 2;
+		else if (iscntrl2(s[r])) {
+			rw = 1;
+			vw = 2;
+		}
 		else if (ismsb(s[r])) {
 			rw = 1;
 			vw = 4;
@@ -430,6 +438,27 @@ int cx2;
 	return(iskanji1(s, r) ? r + 1 : 0);
 }
 
+int kanjiputs2(s, len, ptr)
+char *s;
+int len, ptr;
+{
+	char *buf;
+	int n;
+
+	if (len < 0) return(0);
+	n = len * KANAWID;
+	buf = malloc2(n + 1);
+	strncpy3(buf, s, &len, ptr);
+#ifdef	_NOKANJICONV
+	cputs2(buf);
+	n = strlen2(buf);
+#else
+	n = kanjiputs(buf);
+#endif
+	free(buf);
+	return(n);
+}
+
 static VOID NEAR putcursor(c, n)
 int c, n;
 {
@@ -493,7 +522,7 @@ static VOID NEAR forwline(x)
 int x;
 {
 	if (win_x < n_column) {
-		cputs2("\r\n");
+		cputnl();
 		win_y++;
 	}
 	else {
@@ -632,6 +661,20 @@ int cx, cx2, len, plen, linemax;
 	if (f) rightcursor();
 }
 
+static int NEAR putstr(cp, s, cx, cx2, len, plen, linemax)
+char *cp, *s;
+int cx, cx2, len, plen, linemax;
+{
+	while (*cp) {
+		cx2++;
+		putcursor(*cp, 1);
+		if (iseol(cx2, plen, linemax))
+			setcursor(s, cx, cx2, len, plen, linemax);
+		cp++;
+	}
+	return(cx2);
+}
+
 static VOID NEAR ringbell(VOID_A)
 {
 #ifndef	_NOORIGSHELL
@@ -674,16 +717,19 @@ int *cxp, cx2, len, plen, linemax;
 {
 	int rw, vw;
 
+	if (*cxp + 1 < len && iskanji1(s, *cxp)) rw = vw = 2;
 #ifdef	CODEEUC
-	if (*cxp + 1 < len && isekana(s, *cxp)) {
+	else if (*cxp + 1 < len && isekana(s, *cxp)) {
 		rw = 2;
 		vw = 1;
 	}
 #else
-	if (iskna(s[*cxp])) rw = vw = 1;
+	else if (isskana(s, *cxp)) rw = vw = 1;
 #endif
-	else if (*cxp + 1 < len && (iskanji1(s, *cxp) || s[*cxp] == QUOTE))
-		rw = vw = 2;
+	else if (iscntrl2(s[*cxp])) {
+		rw = 1;
+		vw = 2;
+	}
 	else if (ismsb(s[*cxp])) {
 		rw = 1;
 		vw = 4;
@@ -714,16 +760,19 @@ int *cxp, cx2, len, plen, linemax;
 {
 	int rw, vw, ocx2;
 
+	if (*cxp >= 2 && onkanji1(s, *cxp - 2)) rw = vw = 2;
 #ifdef	CODEEUC
-	if (*cxp >= 2 && isekana(s, *cxp - 2)) {
+	else if (*cxp >= 2 && isekana(s, *cxp - 2)) {
 		rw = 2;
 		vw = 1;
 	}
 #else
-	if (iskna(s[*cxp - 1])) rw = vw = 1;
+	else if (isskana(s, *cxp - 1)) rw = vw = 1;
 #endif
-	else if (*cxp >= 2 && onkanji1(s, *cxp - 2)) rw = vw = 2;
-	else if (*cxp >= 2 && s[*cxp - 2] == QUOTE) rw = vw = 2;
+	else if (iscntrl2(s[*cxp - 1])) {
+		rw = 1;
+		vw = 2;
+	}
 	else if (ismsb(s[*cxp - 1])) {
 		rw = 1;
 		vw = 4;
@@ -773,7 +822,7 @@ int cx, len, plen, linemax, ins;
 		}
 	}
 	else
-#endif /* !_NOORIGSHELL */
+#endif	/* !_NOORIGSHELL */
 #if	!MSDOS
 	if (*c_insert) {
 # ifndef	_NOORIGSHELL
@@ -970,7 +1019,7 @@ int cx, len, plen, linemax, del;
 		dumbputs(dupl, i + cx2, l, linemax - (i + j), del);
 	}
 	else
-#endif /* !_NOORIGSHELL */
+#endif	/* !_NOORIGSHELL */
 #if	!MSDOS
 	if (*c_delete) {
 		j = linemax - (cx2 + plen) % linemax;
@@ -1130,7 +1179,7 @@ int *cxp, *lenp, plen, linemax;
 		forwcursor(xpos);
 		i = plen;
 		x = cx2 - linemax;
-		y = (linemax + 1) / 2; 
+		y = (linemax + 1) / 2;
 		for (lastofs2 = 0; lastofs2 <= i + x; lastofs2 += y) i = 1;
 		if (!lastofs2) dispprompt(NULL, 0);
 		else {
@@ -1237,29 +1286,32 @@ int linemax, rins, vins;
 	}
 
 	if (cx >= *lenp) rw = vw = 0;
+	else if (cx + 1 < *lenp && iskanji1(*sp, cx)) rw = vw = 2;
 #ifdef	CODEEUC
 	else if (cx + 1 < *lenp && isekana(*sp, cx)) {
 		rw = 2;
 		vw = 1;
 	}
 #else
-	else if (iskna((*sp)[cx])) rw = vw = 1;
+	else if (isskana(*sp, cx)) rw = vw = 1;
 #endif
-	else if (cx + 1 < *lenp && (iskanji1(*sp, cx) || (*sp)[cx] == QUOTE))
-		rw = vw = 2;
+	else if (iscntrl2((*sp)[cx])) {
+		rw = 1;
+		vw = 2;
+	}
 	else if (ismsb((*sp)[cx])) {
 		rw = 1;
 		vw = 4;
 	}
 	else rw = vw = 1;
 
-	if (!vw);
+	if (!vw) /*EMPTY*/;
 	else if (vins > vw)
 		insertchar(*sp, cx, *lenp, plen, linemax, vins - vw);
 	else if (vins < vw)
 		deletechar(*sp, cx, *lenp, plen, linemax, vw - vins);
 
-	if (!rw);
+	if (!rw) /*EMPTY*/;
 	else if (rins > rw) {
 		*sp = c_realloc(*sp, *lenp + rins - rw, sizep);
 		insshift(*sp, cx, *lenp, rins - rw);
@@ -1287,11 +1339,10 @@ int ins, quote;
 	if (len2 + vlen(strins, ins) > max) ins = rlen(strins, max - len2);
 
 	if (ins > 0) {
-#ifdef	CODEEUC
-		if (isekana(strins, ins - 1)) ins--;
-		else
-#endif
 		if (onkanji1(strins, ins - 1)) ins--;
+#ifdef	CODEEUC
+		else if (isekana(strins, ins - 1)) ins--;
+#endif
 	}
 	if (ins <= 0) return(0);
 
@@ -1302,24 +1353,15 @@ int ins, quote;
 	for (i = 0; i < ins; i++) (*sp)[cx + i] = ' ';
 	len += ins;
 	for (i = j = 0; i < ins; i++, j++) {
-		if (isctl(strins[i])) {
-			insertchar(*sp, cx, len, plen, linemax, 1);
-			*sp = c_realloc(*sp, len + 1, sizep);
-			insshift(*sp, cx + j, len, 1);
-			dupl[j] = '^';
-			(*sp)[cx + j] = QUOTE;
-			j++;
-			dupl[j] = (strins[i] + '@') & 0x7f;
-			(*sp)[cx + j] = strins[i];
-		}
 #ifdef	CODEEUC
-		else if (isekana(strins, i)) {
+		if (isekana(strins, i)) {
 			dupl[j] = (*sp)[cx + j] = strins[i];
 			j++;
 			dupl[j] = (*sp)[cx + j] = strins[++i];
 		}
+		else
 #endif
-		else if (iskanji1(strins, i)) {
+		if (iskanji1(strins, i)) {
 			dupl[j] = (*sp)[cx + j] = strins[i];
 			j++;
 			dupl[j] = (*sp)[cx + j] = strins[++i];
@@ -1435,7 +1477,7 @@ char **argv;
 			selectlist[i].name = strdup2(argv[i]);
 			selectlist[i].flags = (F_ISRED | F_ISWRI);
 			selectlist[i].tmpflags = F_STAT;
-			len = strlen3(argv[i]);
+			len = strlen2(argv[i]);
 			if (maxlen < len) maxlen = len;
 		}
 		dupsorton = sorton;
@@ -1447,11 +1489,12 @@ char **argv;
 			int j, n;
 
 			n = 1;
-			cputs2("\r\n");
+			cputnl();
 			if (argc < LIMITSELECTWARN) i = 'Y';
 			else {
 				cprintf2("There are %d possibilities.", argc);
-				cputs2("  Do you really\r\n");
+				cputs2("  Do you really");
+				cputnl();
 				cputs2("wish to see them all? (y or n)");
 				tflush();
 				for (;;) {
@@ -1460,16 +1503,16 @@ char **argv;
 					ringbell();
 					tflush();
 				}
-				cputs2("\r\n");
+				cputnl();
 				n += 2;
 			}
 
-			if (i == 'N');
+			if (i == 'N') /*EMPTY*/;
 			else if (n_column < maxlen + 2) {
 				n += argc;
 				for (i = 0; i < argc; i++) {
 					kanjiputs(selectlist[i].name);
-					cputs2("\r\n");
+					cputnl();
 				}
 			}
 			else {
@@ -1478,9 +1521,10 @@ char **argv;
 				n += len;
 				for (i = 0; i < len; i++) {
 					for (j = i; j < argc; j += len)
-						kanjiputs2(selectlist[j].name,
-							maxlen + 2, 0);
-					cputs2("\r\n");
+						cprintf2("%-*.*k",
+							maxlen + 2, maxlen + 2,
+							selectlist[j].name);
+					cputnl();
 				}
 			}
 
@@ -1545,7 +1589,7 @@ int linemax, comline, cont;
 # ifndef	_NOORIGSHELL
 	int vartop;
 # endif
-	char *cp1, *cp2, **argv;
+	char *cp, *tmp, **argv;
 	int i, l, pc, ins, top, fix, argc, quote, quoted, hasmeta;
 
 	if (selectlist && cont > 0) {
@@ -1582,7 +1626,7 @@ int linemax, comline, cont;
 			if ((*sp)[i] == '`') top = i + 1;
 		}
 # endif
-		else if (pc != PC_NORMAL);
+		else if (pc != PC_NORMAL) /*EMPTY*/;
 		else if ((*sp)[i] == '=' || strchr(CMDLINE_DELIM, (*sp)[i]))
 			top = i + 1;
 	}
@@ -1599,27 +1643,26 @@ int linemax, comline, cont;
 			vartop = 0;
 		else {
 			for (i = vartop; i < cx; i++)
-				if (!isidentchar((*sp)[i])
-				&& !isdigit((*sp)[i])) break;
+				if (!isidentchar2((*sp)[i])) break;
 			if (i < cx) vartop = 0;
 			else top = vartop;
 		}
 	}
 # endif	/* !_NOORIGSHELL */
 
-	cp1 = strndup2(&((*sp)[top]), cx - top);
+	cp = strndup2(&((*sp)[top]), cx - top);
 # ifndef	_NOORIGSHELL
 	if (vartop);
 	else
 # endif
-	cp1 = evalpath(cp1, 1);
+	cp = evalpath(cp, 1);
 	hasmeta = 0;
-	for (i = 0; cp1[i]; i++) {
-		if (strchr(METACHAR, cp1[i])) {
+	for (i = 0; cp[i]; i++) {
+		if (strchr(METACHAR, cp[i])) {
 			hasmeta = 1;
 			break;
 		}
-		if (iskanji1(cp1, i)) i++;
+		if (iskanji1(cp, i)) i++;
 	}
 
 	if (selectlist && cont < 0) {
@@ -1630,50 +1673,48 @@ int linemax, comline, cont;
 # ifndef	_NOORIGSHELL
 	else if (vartop) {
 		argv = NULL;
-		l = strlen(cp1);
-		argc = completeshellvar(cp1, l, 0, &argv);
+		l = strlen(cp);
+		argc = completeshellvar(cp, l, 0, &argv);
 	}
 # endif
 	else {
 		argv = NULL;
 		argc = 0;
-		l = strlen(cp1);
-		if (comline && !strdelim(cp1, 1)) {
+		l = strlen(cp);
+		if (comline && !strdelim(cp, 1)) {
 # ifdef	_NOORIGSHELL
-			argc = completeuserfunc(cp1, l, argc, &argv);
-			argc = completealias(cp1, l, argc, &argv);
-			argc = completebuiltin(cp1, l, argc, &argv);
-			argc = completeinternal(cp1, l, argc, &argv);
+			argc = completeuserfunc(cp, l, argc, &argv);
+			argc = completealias(cp, l, argc, &argv);
+			argc = completebuiltin(cp, l, argc, &argv);
+			argc = completeinternal(cp, l, argc, &argv);
 # else
-			argc = completeshellcomm(cp1, l, argc, &argv);
+			argc = completeshellcomm(cp, l, argc, &argv);
 # endif
 		}
 # ifndef	_NOARCHIVE
-		if (archivefile && !comline && *cp1 != _SC_)
-			argc = completearch(cp1, l, argc, &argv);
+		if (archivefile && !comline && *cp != _SC_)
+			argc = completearch(cp, l, argc, &argv);
 		else
 # endif
-		argc = completepath(cp1, l, argc, &argv, comline);
+		argc = completepath(cp, l, argc, &argv, comline);
 		if (!argc && comline)
-			argc = completepath(cp1, l, argc, &argv, 0);
+			argc = completepath(cp, l, argc, &argv, 0);
 	}
 
-	if ((cp2 = strrdelim(cp1, 1))) cp2++;
-	else cp2 = cp1;
-	ins = strlen(cp2);
-	free(cp1);
+	ins = strlen(getbasename(cp));
+	free(cp);
 	if (!argc) {
 		ringbell();
 		if (argv) free(argv);
 		return(0);
 	}
 
-	cp1 = findcommon(argc, argv);
+	cp = findcommon(argc, argv);
 	fix = '\0';
-	if (argc == 1 && cp1)
-		fix = ((cp2 = strrdelim(cp1, 0)) && !cp2[1]) ? _SC_ : ' ';
+	if (argc == 1 && cp)
+		fix = ((tmp = strrdelim(cp, 0)) && !tmp[1]) ? _SC_ : ' ';
 
-	if (!cp1 || ((ins = (int)strlen(cp1) - ins) <= 0 && fix != ' ')) {
+	if (!cp || ((ins = (int)strlen(cp) - ins) <= 0 && fix != ' ')) {
 		if (cont <= 0) {
 			ringbell();
 			l = 0;
@@ -1687,23 +1728,23 @@ int linemax, comline, cont;
 		}
 		for (i = 0; i < argc; i++) free(argv[i]);
 		free(argv);
-		if (cp1) free(cp1);
+		if (cp) free(cp);
 		return(l);
 	}
 	for (i = 0; i < argc; i++) free(argv[i]);
 	free(argv);
 
 	l = 0;
-	if (!hasmeta) for (i = 0; cp1[i]; i++) {
-		if (strchr(METACHAR, cp1[i])) {
+	if (!hasmeta) for (i = 0; cp[i]; i++) {
+		if (strchr(METACHAR, cp[i])) {
 			hasmeta = 1;
 			break;
 		}
-		if (iskanji1(cp1, i)) i++;
+		if (iskanji1(cp, i)) i++;
 	}
 
 	if (hasmeta) {
-		char *tmp, *home;
+		char *home;
 		int hlen;
 
 		if (quote || quoted > top || (*sp)[top] != '~') {
@@ -1718,7 +1759,7 @@ int linemax, comline, cont;
 			i = ++tmp - &((*sp)[top]);
 		}
 
-		if (quote);
+		if (quote) /*EMPTY*/;
 		else if (quoted > top) {
 			quote = (*sp)[quoted];
 			setcursor(*sp, quoted, -1, len, plen, linemax);
@@ -1773,9 +1814,9 @@ int linemax, comline, cont;
 		else hasmeta = 0;
 	}
 
-	cp2 = cp1 + (int)strlen(cp1) - ins;
+	tmp = cp + (int)strlen(cp) - ins;
 	if (fix == _SC_) ins--;
-	i = insertstr(sp, cx, len, plen, sizep, linemax, cp2, ins, quote);
+	i = insertstr(sp, cx, len, plen, sizep, linemax, tmp, ins, quote);
 	len += i;
 	l += i;
 	if (fix && !overflow(vlen(*sp, len), plen, linemax)) {
@@ -1798,7 +1839,7 @@ int linemax, comline, cont;
 		}
 	}
 
-	free(cp1);
+	free(cp);
 	return(l);
 }
 #endif	/* !_NOCOMPLETE */
@@ -1830,7 +1871,7 @@ char **tmp;
 	if (dumbmode || cx2 < linemax)
 #endif
 	{
-		if (h < 0 || !history[h] || *histnop >= histsize[h]
+		if (h < 0 || !history[h] || *histnop >= (int)histsize[h]
 		|| !history[h][*histnop]) {
 			ringbell();
 			return(cx2);
@@ -1841,7 +1882,6 @@ char **tmp;
 		}
 		for (i = j = 0; history[h][*histnop][i]; i++) {
 			*sp = c_realloc(*sp, j, sizep);
-			if (isctl(history[h][*histnop][i])) (*sp)[j++] = QUOTE;
 			(*sp)[j++] = history[h][*histnop][i];
 		}
 		(*sp)[j] = '\0';
@@ -1925,8 +1965,6 @@ char **tmp;
 		if (--(*histnop) > 0) {
 			for (i = j = 0; history[h][*histnop - 1][i]; i++) {
 				*sp = c_realloc(*sp, j, sizep);
-				if (isctl(history[h][*histnop - 1][i]))
-					(*sp)[j++] = QUOTE;
 				(*sp)[j++] = history[h][*histnop - 1][i];
 			}
 			(*sp)[j] = '\0';
@@ -1990,21 +2028,29 @@ int cx, len, plen, linemax;
 		return(len);
 	}
 
-#ifdef	CODEEUC
-	if (isekana(s, cx)) {
-		rw = 2;
-		vw = 1;
-	}
-#else
-	if (iskna(s[cx])) rw = vw = 1;
-#endif
-	else if (iskanji1(s, cx) || s[cx] == QUOTE) {
+	if (iskanji1(s, cx)) {
 		if (cx + 1 >= len) {
 			ringbell();
 			return(len);
 		}
 		rw = vw = 2;
 	}
+#ifdef	CODEEUC
+	else if (isekana(s, cx)) {
+		if (cx + 1 >= len) {
+			ringbell();
+			return(len);
+		}
+		rw = 2;
+		vw = 1;
+	}
+#else
+	else if (isskana(s, cx)) rw = vw = 1;
+#endif
+	else if (iscntrl2(s[cx])) {
+		rw = 1;
+		vw = 2;
+	}
 	else if (ismsb(s[cx])) {
 		rw = 1;
 		vw = 4;
@@ -2110,18 +2156,18 @@ int *cxp, cx2, *lenp, plen;
 ALLOC_T *sizep;
 int linemax, ch;
 {
-#if	!MSDOS && !defined (_NOKANJICONV)
+#ifndef	_NOKANJICONV
 	char tmpkanji[3];
 #endif
 	int rw, vw, ch2;
 
-#if	!MSDOS && !defined (_NOKANJICONV)
-	if (inputkcode == EUC && ch == 0x8e) {
+#ifndef	_NOKANJICONV
+	if (inputkcode == EUC && isekana2(ch)) {
 		rw = KANAWID;
 		vw = 1;
-		ch2 = (kbhit2(WAITMETA * 1000L)) ? getkey2(0) : '\0';
-		if (!iskna(ch2)
-		|| preparestr(sp, *cxp, lenp,
+		ch = C_EKANA;
+		ch2 = (ch & 0xff);
+		if (preparestr(sp, *cxp, lenp,
 		plen, sizep, linemax, rw, vw) < 0) {
 			ringbell();
 			keyflush();
@@ -2132,19 +2178,19 @@ int linemax, ch;
 		tmpkanji[2] = '\0';
 		kanjiconv(&((*sp)[*cxp]), tmpkanji, 2,
 			inputkcode, DEFCODE, L_INPUT);
-		kanjiputs2(&((*sp)[*cxp]), 1, -1);
+		cprintf2("%.*k", 1, &((*sp)[*cxp]));
 		win_x += 1;
 		*cxp += KANAWID;
 	}
 	else
-#else
+#else	/* _NOKANJICONV */
 # ifdef	CODEEUC
-	if (ch == 0x8e) {
+	if (isekana2(ch)) {
 		rw = KANAWID;
 		vw = 1;
-		ch2 = (kbhit2(WAITMETA * 1000L)) ? getkey2(0) : '\0';
-		if (!iskna(ch2)
-		|| preparestr(sp, *cxp, lenp,
+		ch = C_EKANA;
+		ch2 = (ch & 0xff);
+		if (preparestr(sp, *cxp, lenp,
 		plen, sizep, linemax, rw, vw) < 0) {
 			ringbell();
 			keyflush();
@@ -2157,7 +2203,7 @@ int linemax, ch;
 	}
 	else
 # endif
-#endif
+#endif	/* _NOKANJICONV */
 	if (isinkanji1(ch)) {
 		rw = vw = 2;
 		ch2 = (kbhit2(WAITMETA * 1000L)) ? getkey2(0) : '\0';
@@ -2169,7 +2215,7 @@ int linemax, ch;
 			return(cx2);
 		}
 
-#if	MSDOS || defined (_NOKANJICONV)
+#ifdef	_NOKANJICONV
 		(*sp)[*cxp] = ch;
 		(*sp)[*cxp + 1] = ch2;
 #else
@@ -2179,13 +2225,13 @@ int linemax, ch;
 		kanjiconv(&((*sp)[*cxp]), tmpkanji, 2,
 			inputkcode, DEFCODE, L_INPUT);
 #endif
-		kanjiputs2(&((*sp)[*cxp]), 2, -1);
+		cprintf2("%.*k", 2, &((*sp)[*cxp]));
 		win_x += 2;
 		*cxp += 2;
 	}
 	else {
 		rw = vw = 1;
-		if (ch >= K_MIN || isctl(ch) || ismsb(ch)
+		if (ch >= K_MIN || iscntrl2(ch) || ismsb(ch)
 		|| preparestr(sp, *cxp, lenp,
 		plen, sizep, linemax, rw, vw) < 0) {
 			ringbell();
@@ -2213,10 +2259,9 @@ ALLOC_T size;
 int linemax, def, comline, h;
 {
 #if	!MSDOS
-	char *cp;
-	int l;
+	keyseq_t key;
 #endif
-	char *tmphist;
+	char *tmphist, buf[5];
 	int i, ch, ch2, cx, cx2, ocx2, len, hist, quote, sig;
 
 	subwindow = 1;
@@ -2267,7 +2312,7 @@ int linemax, def, comline, h;
 #endif
 		}
 		else {
-			i = ch = getkey2(sig);
+			i = ch = getkey2(sigalrm(sig));
 			quote = 0;
 #if	MSDOS
 			switch (i) {
@@ -2281,47 +2326,48 @@ int linemax, def, comline, h;
 					break;
 			}
 #else
-			if ((cp = getkeyseq(i, &l)) && l == 1) i = *cp;
+			key.code = i;
+			if (getkeyseq(&key) >= 0 && key.len == 1)
+				i = *(key.str);
+#endif
+#ifndef	_NOKANJICONV
+			if (inputkcode == EUC && isekana2(i)) /*EMPTY*/;
+			else
+#else
+# ifdef	CODEEUC
+			if (isekana2(i)) /*EMPTY*/;
+			else
+# endif
 #endif
 			if (i >= K_MIN) {
 				ringbell();
 				continue;
 			}
-			else if (isctl(i)) {
+			else if (iscntrl2(i)) {
 				keyflush();
 				ch = '\0';
 				if (!i) continue;
 				if (preparestr(sp, cx, &len,
-				plen, &size, linemax, 2, 2) < 0) {
+				plen, &size, linemax, 1, 2) < 0) {
 					ringbell();
 					continue;
 				}
-				(*sp)[cx++] = QUOTE;
-				cx2++;
-				putcursor('^', 1);
-				if (iseol(cx2, plen, linemax))
-					setcursor(*sp, cx, cx2,
-						len, plen, linemax);
 				(*sp)[cx++] = i;
-				cx2++;
-				putcursor((i + '@') & 0x7f, 1);
-				if (iseol(cx2, plen, linemax))
-					setcursor(*sp, cx, cx2,
-						len, plen, linemax);
+				snprintf2(buf, sizeof(buf), "^%c",
+					(i + '@') & 0x7f);
+				cx2 = putstr(buf, *sp, cx, cx2,
+					len, plen, linemax);
 				continue;
 			}
-#if	!MSDOS && !defined (_NOKANJICONV)
-			else if (inputkcode == EUC
-			&& i == 0x8e && kbhit2(WAITMETA * 1000L));
-			else if (inputkcode != EUC && iskna(i));
+#ifndef	_NOKANJICONV
+			else if (inputkcode != EUC && iskana2(i)) /*EMPTY*/;
 #else
-# ifdef	CODEEUC
-			else if (i == 0x8e && kbhit2(WAITMETA * 1000L));
-# else
-			else if (iskna(i));
+# ifndef	CODEEUC
+			else if (iskana2(i)) /*EMPTY*/;
 # endif
 #endif
-			else if (isinkanji1(i) && kbhit2(WAITMETA * 1000L));
+			else if (isinkanji1(i) && kbhit2(WAITMETA * 1000L))
+				/*EMPTY*/;
 			else if (ismsb(i)) {
 				keyflush();
 				ch = '\0';
@@ -2331,26 +2377,9 @@ int linemax, def, comline, h;
 					continue;
 				}
 				(*sp)[cx++] = i;
-				cx2++;
-				putcursor('\\', 1);
-				if (iseol(cx2, plen, linemax))
-					setcursor(*sp, cx, cx2,
-						len, plen, linemax);
-				cx2++;
-				putcursor((i / (8 * 8)) + '0', 1);
-				if (iseol(cx2, plen, linemax))
-					setcursor(*sp, cx, cx2,
-						len, plen, linemax);
-				cx2++;
-				putcursor(((i % (8 * 8)) / 8) + '0', 1);
-				if (iseol(cx2, plen, linemax))
-					setcursor(*sp, cx, cx2,
-						len, plen, linemax);
-				cx2++;
-				putcursor((i % 8) + '0', 1);
-				if (iseol(cx2, plen, linemax))
-					setcursor(*sp, cx, cx2,
-						len, plen, linemax);
+				snprintf2(buf, sizeof(buf), "\\%03o", i);
+				cx2 = putstr(buf, *sp, cx, cx2,
+					len, plen, linemax);
 				continue;
 			}
 		}
@@ -2588,7 +2617,7 @@ int set;
 		{
 			putch2(' ');
 			putterm(t_standout);
-			len = kanjiputs(prompt) + 1;
+			len = 1 + kanjiputs(prompt);
 			putterm(end_standout);
 		}
 	}
@@ -2619,7 +2648,7 @@ int delsp, ptr;
 char *def;
 int h;
 {
-	char *dupl, *input;
+	char *input;
 	ALLOC_T size;
 	int i, j, len, ch, comline, dupwin_x, dupwin_y;
 
@@ -2669,17 +2698,14 @@ int h;
 		}
 		for (i = 0; def[i]; i++, j++) {
 			input = c_realloc(input, j + 2, &size);
-			if (isctl(def[i])) {
-				input[j++] = QUOTE;
-				if (ptr >= j) ptr++;
-			}
 #ifdef	CODEEUC
-			else if (isekana(def, i)) {
+			if (isekana(def, i)) {
 				input[j++] = def[i++];
 				if (ptr >= j) ptr++;
 			}
+			else
 #endif
-			else if (iskanji1(def, i)) input[j++] = def[i++];
+			if (iskanji1(def, i)) input[j++] = def[i++];
 			else if (!prompt && strchr(DQ_METACHAR, def[i])) {
 #ifdef	FAKEMETA
 				input[j++] = def[i];
@@ -2724,8 +2750,7 @@ int h;
 				cputs2("^C");
 				ch = K_ESC;
 			}
-			cputs2("\r\n");
-			tflush();
+			cputnl();
 		}
 	}
 	else {
@@ -2751,33 +2776,26 @@ int h;
 		for (len--; len > 0 && input[len - 1] == ' '; len--);
 		input[len] = '\0';
 	}
-	dupl = malloc2(len + 1);
-	for (i = j = 0; input[i]; i++, j++) {
-		if (input[i] == QUOTE) i++;
-		dupl[j] = input[i];
-	}
-	dupl[j] = '\0';
-	free(input);
-	return(dupl);
+	return(input);
 }
 
 static char *NEAR truncstr(s)
 char *s;
 {
 	int len;
-	char *cp1, *cp2;
+	char *cp, *tmp;
 
-	if ((len = (int)strlen2(s) + YESNOSIZE - n_lastcolumn) <= 0
-	|| !(cp1 = strchr2(s, '[')) || !(cp2 = strchr2(cp1, ']'))) return(s);
+	if ((len = strlen2(s) + YESNOSIZE - n_lastcolumn) <= 0
+	|| !(cp = strchr2(s, '[')) || !(tmp = strchr2(cp, ']'))) return(s);
 
-	cp1++;
-	len = cp2 - cp1 - len;
+	cp++;
+	len = tmp - cp - len;
 	if (len <= 0) len = 0;
+	else if (onkanji1(cp, len - 1)) len--;
 #ifdef	CODEEUC
-	else if (isekana(cp1, len - 1)) len--;
+	else if (isekana(cp, len - 1)) len--;
 #endif
-	else if (onkanji1(cp1, len - 1)) len--;
-	strcpy(&(cp1[len]), cp2);
+	strcpy(&(cp[len]), tmp);
 	return(s);
 }
 
@@ -2819,7 +2837,7 @@ int yesno(CONST char *fmt, ...)
 #else
 /*VARARGS1*/
 int yesno(fmt, va_alist)
-char *fmt;
+CONST char *fmt;
 va_dcl
 #endif
 {
@@ -2856,7 +2874,7 @@ va_dcl
 	win_y = ypos;
 
 	truncstr(buf);
-	len = strlen3(buf);
+	len = strlen2(buf);
 	ret = yesnomes(buf);
 	if (ret < len) len = ret;
 	if (win_x >= n_column) win_x = n_column - 1;
@@ -2913,13 +2931,13 @@ va_dcl
 	free(buf);
 
 #ifndef	_NOORIGSHELL
-	if (dumbmode) cputs2("\r\n");
+	if (dumbmode) cputnl();
 	else
 #endif
 	if (lcmdline) {
 		Xlocate(x, 0);
 		putch2((ret) ? 'Y' : 'N');
-		cputs2("\r\n");
+		cputnl();
 	}
 	else {
 		Xlocate(0, 0);
@@ -2955,8 +2973,8 @@ char *s;
 	tmp = NULL;
 	if (!s) s = err;
 	else if (no) {
-		len = n_lastcolumn - (int)strlen(err) - 3;
-		tmp = (char *)malloc2(strlen2(s) + strlen(err) + 3);
+		len = n_lastcolumn - strlen2(err) - 3;
+		tmp = malloc2(n_lastcolumn * KANAWID + 1);
 		strncpy3(tmp, s, &len, -1);
 		strcat(tmp, ": ");
 		strcat(tmp, err);
@@ -3009,7 +3027,7 @@ int multi;
 	int i, len;
 
 	for (i = len = 0; i < max; i++) if (str[i]) {
-		len += strlen(str[i]) + 1;
+		len += strlen2(str[i]) + 1;
 		if (multi) len++;
 	}
 	return(len);
@@ -3029,8 +3047,8 @@ int xx[], multi;
 	}
 	len = selectcnt(max, tmpstr, multi);
 
-	if (x + len < n_lastcolumn);
-	else if ((x = n_lastcolumn - 1 - len) >= 0);
+	if (x + len < n_lastcolumn) /*EMPTY*/;
+	else if ((x = n_lastcolumn - 1 - len) >= 0) /*EMPTY*/;
 	else {
 		x = maxlen = 0;
 		str = (char **)malloc2(max * sizeof(char *));
@@ -3039,7 +3057,7 @@ int xx[], multi;
 				str[i] = NULL;
 				continue;
 			}
-			if (*cp >= 'A' && *cp <= 'Z' && cp[1] == ':')
+			if (isupper2(*cp) && cp[1] == ':')
 				for (cp += 2; *cp == ' '; cp++);
 			len = strlen3(cp);
 			if (len > maxlen) maxlen = len;
@@ -3065,7 +3083,7 @@ int xx[], multi;
 			xx[i + 1] = xx[i];
 			continue;
 		}
-		xx[i + 1] = xx[i] + strlen3(tmpstr[i]) + 1;
+		xx[i + 1] = xx[i] + strlen2(tmpstr[i]) + 1;
 		if (multi) (xx[i + 1])++;
 	}
 
@@ -3116,8 +3134,7 @@ int val[];
 
 	for (i = 0; i < max; i++) {
 		tmpstr[i] = NULL;
-		initial[i] = (str[i] && *(str[i]) >= 'A' && *(str[i]) <= 'Z')
-			? *str[i] : -1;
+		initial[i] = (str[i] && isupper2(*(str[i]))) ? *str[i] : -1;
 	}
 	tmpx = selectadj(max, x, str, tmpstr, xx, (num) ? 0 : 1);
 
@@ -3161,8 +3178,8 @@ int val[];
 				putch2((val[new]) ? '*' : ' ');
 				break;
 			default:
+				if (!isalpha2(ch)) break;
 				ch = toupper2(ch);
-				if (ch < 'A' || ch > 'Z') break;
 				for (i = 0; i < max; i++)
 					if (ch == initial[i]) break;
 				if (i >= max) break;
@@ -3205,7 +3222,7 @@ int val[];
 			if (!tmpstr[i]) continue;
 			locate(tmpx + xx[i] + 1, L_MESLINE);
 			if (i == new) kanjiputs(tmpstr[i]);
-			else cprintf2("%*s", strlen3(tmpstr[i]), " ");
+			else cprintf2("%*s", strlen2(tmpstr[i]), "");
 		}
 	}
 	else {
@@ -3217,8 +3234,9 @@ int val[];
 		for (i = 0; i < max; i++) {
 			if (!tmpstr[i]) continue;
 			locate(tmpx + xx[i] + 1, L_MESLINE);
+			putch2(' ');
 			if (val[i]) kanjiputs(tmpstr[i]);
-			else cprintf2("%*s", strlen3(tmpstr[i]), " ");
+			else cprintf2("%*s", strlen2(tmpstr[i]), "");
 		}
 	}
 	locate(win_x, win_y);
