@@ -7,16 +7,13 @@
 #include <ctype.h>
 #include <signal.h>
 #include "fd.h"
-#include "term.h"
 #include "func.h"
 #include "kanji.h"
 #include "version.h"
 
 #if	MSDOS
 #include <process.h>
-# ifdef	DJGPP
-extern char *adjustfname __P_((char *));
-# else
+# ifndef	DJGPP
 # include <dos.h>
 #  ifdef	__TURBOC__
 extern unsigned _stklen = 0x5800;
@@ -33,7 +30,16 @@ extern unsigned _stklen = 0x5800;
 #include "system.h"
 #endif
 
-#if	!MSDOS && !defined (_NOORIGSHELL) && !defined (NOJOB)
+#if	MSDOS
+# ifndef	BSPATHDELIM
+extern char *adjustpname __P_((char *));
+# endif
+# if	defined (DJGPP) || !defined (BSPATHDELIM)
+extern char *adjustfname __P_((char *));
+# endif
+#endif	/* MSDOS */
+
+#if	!defined (_NOORIGSHELL) && !defined (NOJOB)
 extern VOID killjob __P_((VOID_A));
 #endif
 
@@ -49,9 +55,9 @@ extern archivetable archivelist[];
 extern int maxarchive;
 #endif
 extern char fullpath[];
-extern char **history[];
 extern char *histfile;
 extern char *helpindex[];
+extern int tradlayout;
 extern int sizeinfo;
 extern int subwindow;
 extern int win_x;
@@ -90,7 +96,7 @@ extern char *unitblpath;
 #endif
 
 #if	MSDOS && !defined (PROTECTED_MODE)
-static harderr_t far criticalerror __P_((u_short, u_short, u_short far *));
+static harderr_t far criticalerror __P_((u_int, u_int, u_short far *));
 #endif
 static VOID NEAR signalexit __P_((int));
 #ifdef	SIGALRM
@@ -180,7 +186,7 @@ int fd_restricted = 0;
 #ifndef	_NOCUSTOMIZE
 char **orighelpindex = NULL;
 bindtable *origbindlist = NULL;
-# if	!MSDOS && !defined (_NOKEYMAP)
+# ifndef	_NOKEYMAP
 keyseq_t *origkeymaplist = NULL;
 # endif
 # ifndef	_NOARCHIVE
@@ -189,12 +195,14 @@ int origmaxlaunch = 0;
 archivetable *origarchivelist = NULL;
 int origmaxarchive = 0;
 # endif
-# if	!MSDOS && !defined (_NODOSDRIVE)
+# ifdef	_USEDOSEMU
 devinfo *origfdtype = NULL;
 # endif
 #endif	/* !_NOCUSTOMIZE */
 int inruncom = 0;
+#ifndef	_NOORIGSHELL
 int fdmode = 0;
+#endif
 
 static int timersec = 0;
 #ifdef	SIGWINCH
@@ -206,7 +214,7 @@ static int winched = 0;
 #if	MSDOS && !defined (PROTECTED_MODE)
 /*ARGSUSED*/
 static harderr_t far criticalerror(deverr, errval, devhdr)
-u_short deverr, errval;
+u_int deverr, errval;
 u_short far *devhdr;
 {
 	if ((deverr & 0x8800) == 0x0800) _hardresume(_HARDERR_FAIL);
@@ -233,13 +241,13 @@ char *s;
 	if (errno) perror2(s);
 	else {
 		fputs(s, stderr);
-		fputc('\n', stderr);
+		fputnl(stderr);
 	}
 	inittty(1);
 	keyflush();
 	prepareexitfd();
 #ifndef	_NOORIGSHELL
-# if	!MSDOS && !defined (NOJOB)
+# ifndef	NOJOB
 	killjob();
 # endif
 	prepareexit(0);
@@ -248,8 +256,11 @@ char *s;
 # if	!MSDOS
 	freeterment();
 # endif
-	if (ttyout && ttyout != stderr) fclose(ttyout);
-	else if (ttyio >= 0) close(ttyio);
+	if (ttyout && ttyout != stderr) {
+		if (fileno(ttyout) == ttyio) ttyio = -1;
+		fclose(ttyout);
+	}
+	if (ttyio >= 0) close(ttyio);
 	muntrace();
 #endif
 
@@ -267,7 +278,7 @@ int sig;
 	endterm();
 	inittty(1);
 	signal2(sig, SIG_DFL);
-	kill(getpid(), sig);
+	VOID_C kill(getpid(), sig);
 }
 
 #ifdef	SIGALRM
@@ -457,9 +468,8 @@ int xmax, ymax;
 			cputs2(cp);
 			putterm(end_standout);
 		}
-		cputs2("\r\n");
 		putterm(t_bell);
-		tflush();
+		cputnl();
 		keyflush();
 		if (kbhit2(1000000L) && getkey2(0) == K_ESC) {
 			errno = 0;
@@ -508,7 +518,7 @@ static int printtime(VOID_A)
 		timersec = CLOCKUPDATE;
 	}
 	if (showsecond || timersec == CLOCKUPDATE) {
-#ifdef	DEBUG
+# ifdef	DEBUG
 		_mtrace_file = "localtime(start)";
 		tm = localtime(&now);
 		if (_mtrace_file) _mtrace_file = NULL;
@@ -516,9 +526,9 @@ static int printtime(VOID_A)
 			_mtrace_file = "localtime(end)";
 			malloc(0);	/* dummy malloc */
 		}
-#else
+# else
 		tm = localtime(&now);
-#endif
+# endif
 		today[0] = tm -> tm_year;
 		today[1] = tm -> tm_mon;
 		today[2] = tm -> tm_mday;
@@ -538,7 +548,7 @@ static int printtime(VOID_A)
 		}
 	}
 	timersec--;
-#ifdef	SIGWINCH
+# ifdef	SIGWINCH
 	if (!winchok) winchok++;
 	else if (winched) {
 		signal2(SIGWINCH, SIG_IGN);
@@ -548,12 +558,12 @@ static int printtime(VOID_A)
 		if (subwindow) ungetch2(K_CTRL('L'));
 		signal2(SIGWINCH, (sigcst_t)wintr);
 	}
-#endif
+# endif
 	signal2(SIGALRM, (sigcst_t)printtime);
 	errno = duperrno;
 	return(0);
 }
-#endif
+#endif	/* SIGALRM */
 
 int sigvecset(set)
 int set;
@@ -562,7 +572,7 @@ int set;
 	int old;
 
 	old = status;
-	if (set == old);
+	if (set == old) /*EMPTY*/;
 	else if (set) {
 #ifdef	SIGALRM
 		signal2(SIGALRM, (sigcst_t)printtime);
@@ -647,7 +657,7 @@ VOID saveorigenviron(VOID_A)
 {
 	orighelpindex = copystrarray(NULL, helpindex, NULL, 10);
 	origbindlist = copybind(NULL, bindlist);
-# if	!MSDOS && !defined (_NOKEYMAP)
+# ifndef	_NOKEYMAP
 	origkeymaplist = copykeyseq(NULL);
 # endif
 # ifndef	_NOARCHIVE
@@ -656,7 +666,7 @@ VOID saveorigenviron(VOID_A)
 	origarchivelist = copyarch(NULL, archivelist,
 		&origmaxarchive, maxarchive);
 # endif
-# if	!MSDOS && !defined (_NODOSDRIVE)
+# ifdef	_USEDOSEMU
 	origfdtype = copydosdrive(NULL, fdtype);
 # endif
 }
@@ -675,10 +685,11 @@ char *line;
 	else i = execpseudoshell(cp, 0, 1);
 	if (i) {
 		putterm(l_clear);
-		cprintf2("%s, line %d: %s\r\n", file, n, ILFNC_K);
+		cprintf2("%s, line %d: %s", file, n, ILFNC_K);
+		cputnl();
 		putterm(l_clear);
-		cprintf2("\t%s\r\n", line);
-		tflush();
+		cprintf2("\t%s", line);
+		cputnl();
 	}
 	free(command);
 	return((i) ? -1 : 0);
@@ -714,13 +725,14 @@ int exist;
 	if (!tmp)
 # endif	/* !MSDOS */
 	tmp = evalpath(strdup2(file), 1);
-	fp = fopen(tmp, "r");
+	fp = Xfopen(tmp, "r");
 	if (!fp) {
 		if (!exist) {
 			free(tmp);
 			return(0);
 		}
-		cprintf2("%s: Not found\r\n", tmp);
+		cprintf2("%s: Not found", tmp);
+		cputnl();
 		free(tmp);
 		return(-1);
 	}
@@ -823,7 +835,7 @@ char *argv[];
 			return(i + 1);
 		tmp = strdup2(&(argv[i][1]));
 		if ((cp = strchr(tmp, '='))) *(cp++) = '\0';
-		setenv2(tmp, cp);
+		setenv2(tmp, cp, 0);
 		free(tmp);
 	}
 	return(i);
@@ -836,7 +848,7 @@ char *s, *envp[];
 
 	len = strlen(s);
 	for (i = 0; envp[i]; i++)
-		if (!strnpathcmp(envp[i], s, len) && envp[i][len] == '=')
+		if (!strnenvcmp(envp[i], s, len) && envp[i][len] == '=')
 			return(&(envp[i][++len]));
 	return(NULL);
 }
@@ -844,15 +856,17 @@ char *s, *envp[];
 static VOID NEAR setexecname(argv)
 char *argv;
 {
-#if	MSDOS
+#if	MSDOS || defined (CYGWIN)
 	char *cp;
 #endif
 
-	if ((progname = strrdelim(argv, 1))) progname++;
-	else progname = argv;
-#if	MSDOS
-	if ((cp = strchr(progname, '.'))) *cp = '\0';
+	progname = getbasename(argv);
+#if	MSDOS || defined (CYGWIN)
+	if ((cp = strchr(progname, '.')) && cp > progname)
+		progname = strndup2(progname, cp - progname);
+	else
 #endif
+	progname = strdup2(progname);
 }
 
 static VOID NEAR setexecpath(argv, envp)
@@ -860,7 +874,8 @@ char *argv, *envp[];
 {
 	char *cp, buf[MAXPATHLEN];
 
-	origpath = getwd2();
+	if (!Xgetwd(buf)) error(NOCWD_K);
+	origpath = strdup2(buf);
 	if ((cp = searchenv("PWD", envp))) {
 		*fullpath = '\0';
 		realpath2(cp, fullpath, 0);
@@ -876,7 +891,7 @@ char *argv, *envp[];
 	cp = argv;
 #else
 	if (strdelim(argv, 0)) cp = argv;
-	else if ((cp = searchenv("PATH", envp))) cp = searchpath(argv, cp);
+	else if ((cp = searchenv("PATH", envp))) cp = searchexecpath(argv, cp);
 	if (!cp) progpath = strdup2(origpath);
 	else
 #endif
@@ -892,13 +907,14 @@ static VOID NEAR prepareexitfd(VOID_A)
 {
 	char cwd[MAXPATHLEN];
 
-	*cwd = '\0';
+	cwd[0] = '\0';
 	if (origpath && _chdir2(origpath) < 0) {
 		perror2(origpath);
 		if (!Xgetwd(cwd)) *cwd = '\0';
 		rawchdir(_SS_);
 	}
 	free(origpath);
+	free(progname);
 #ifndef	_NODOSDRIVE
 	dosallclose();
 #endif
@@ -916,20 +932,20 @@ static VOID NEAR prepareexitfd(VOID_A)
 		free(orighelpindex);
 	}
 	free(origbindlist);
-#  if	!MSDOS && !defined (_NOKEYMAP)
+#  ifndef	_NOKEYMAP
 	freekeyseq(origkeymaplist);
 #  endif
 #  ifndef	_NOARCHIVE
 	if (origlaunchlist) {
-		freelaunch(origlaunchlist, origmaxlaunch);
+		freelaunchlist(origlaunchlist, origmaxlaunch);
 		free(origlaunchlist);
 	}
 	if (origarchivelist) {
-		freearch(origarchivelist, origmaxarchive);
+		freearchlist(origarchivelist, origmaxarchive);
 		free(origarchivelist);
 	}
 #  endif
-#  if	!MSDOS && !defined (_NODOSDRIVE)
+#  ifdef	_USEDOSEMU
 	if (origfdtype) {
 		freedosdrive(origfdtype);
 		free(origfdtype);
@@ -940,7 +956,7 @@ static VOID NEAR prepareexitfd(VOID_A)
 	freehistory(0);
 	freehistory(1);
 	freedefine();
-# if	!MSDOS
+# ifndef	NOUID
 	freeidlist();
 # endif
 	chdir2(NULL);
@@ -950,6 +966,9 @@ static VOID NEAR prepareexitfd(VOID_A)
 # ifndef	_NOROCKRIDGE
 	detranspath(NULL, NULL);
 # endif
+# if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+	discardunitable();
+# endif
 #endif	/* DEBUG */
 	if (*cwd) rawchdir(cwd);
 }
@@ -965,7 +984,7 @@ char *argv[], *envp[];
 	mtrace();
 #endif
 
-#if	MSDOS && defined (DJGPP)
+#if	MSDOS && (defined (DJGPP) || !defined (BSPATHDELIM))
 	adjustfname(argv[0]);
 #endif
 	setexecname(argv[0]);
@@ -1052,7 +1071,7 @@ char *argv[], *envp[];
 #endif	/* !_NOARCHIVE */
 
 	for (i = 0; i < 10; i++) helpindex[i] = strdup2(helpindex[i]);
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	for (i = 0; fdtype[i].name; i++) {
 		fdtype[i].name = strdup2(fdtype[i].name);
 # ifdef	HDDMOUNT
@@ -1068,13 +1087,6 @@ char *argv[], *envp[];
 	environ[i] = NULL;
 #endif
 
-#ifdef	_NOORIGSHELL
-	inittty(0);
-	getterment(NULL);
-#else
-	setshellvar(envp);
-	prepareterm();
-#endif
 	setexecpath(argv[0], envp);
 #ifndef	_NODOSDRIVE
 # ifdef	DATADIR
@@ -1084,27 +1096,27 @@ char *argv[], *envp[];
 # endif
 #endif
 
-#ifndef	_NOORIGSHELL
-	cp = progname;
-	if (*cp == '-') cp++;
-# if	MSDOS
-	if (*cp == 'r' || *cp == 'R') cp++;
-# else
-	if (*cp == 'r') cp++;
-# endif
+#ifdef	_NOORIGSHELL
+	inittty(0);
+	getterment(NULL);
+	argc = initoption(argc, argv, envp);
+#else	/* !_NOORIGSHELL */
+	cp = getshellname(progname, NULL, NULL);
 	if (!strpathcmp(cp, FDSHELL) || !strpathcmp(cp, "su")) {
 		i = main_shell(argc, argv, envp);
 		prepareexitfd();
 		Xexit2(i);
 	}
+	fdmode = interactive = 1;
+	setshellvar(envp);
+	prepareterm();
+	argc = initoption(argc, argv, envp);
 	if (dumbterm > 1) {
 		errno = 0;
 		error(NTERM_K);
 	}
-#endif
+#endif	/* !_NOORIGSHELL */
 
-	fdmode = 1;
-	argc = initoption(argc, argv, envp);
 	ttyiomode(0);
 	initterm();
 	if ((cp = getwsize(WCOLUMNMIN, WHEADERMAX + WFOOTER + WFILEMIN))) {
@@ -1117,7 +1129,16 @@ char *argv[], *envp[];
 
 	locate(0, n_line - 1);
 	inruncom = 1;
+#if	MSDOS && !defined (BSPATHDELIM)
+	/* DEFRUNCOM is gave from command line, not to convert previously */
+/*NOTDEFINED*/
+	cp = strdup2(DEFRUNCOM);
+	i = loadruncom(adjustpname(cp), 0);
+	free(cp);
+#else
+/*NOTDEFINED*/
 	i = loadruncom(DEFRUNCOM, 0);
+#endif
 	i += loadruncom(RUNCOMFILE, 0);
 	inruncom = 0;
 	sigvecset(1);
@@ -1126,7 +1147,7 @@ char *argv[], *envp[];
 		warning(0, HITKY_K);
 	}
 	i = evaloption(argv);
-#if	!MSDOS && defined (_NOORIGSHELL)
+#ifdef	_NOORIGSHELL
 	adjustpath();
 #endif
 	evalenv();
@@ -1151,7 +1172,7 @@ char *argv[], *envp[];
 
 	stdiomode();
 #ifndef	_NOORIGSHELL
-# if	!MSDOS && !defined (NOJOB)
+# ifndef	NOJOB
 	killjob();
 # endif
 #endif
