@@ -5,11 +5,30 @@
  */
 
 #include "fd.h"
+#if	!MSDOS
+#include <sys/param.h>
+#endif
 
-#ifdef	NOVOID
-#define	VOID
+#ifdef	USESELECTH
+#include <sys/select.h>
+#endif
+
+#if	defined (USESYSCONF) && defined (_SC_OPEN_MAX)
+#define	MAXOPENFILE	sysconf(_SC_OPEN_MAX)
 #else
-#define	VOID	void
+# ifdef	NOFILE
+# define	MAXOPENFILE	NOFILE
+# else
+#  ifdef	OPEN_MAX
+#  define	MAXOPENFILE	OPEN_MAX
+#  else
+#   if	MSDOS
+#   define	MAXOPENFILE	20
+#   else
+#   define	MAXOPENFILE	64
+#   endif
+#  endif
+# endif
 #endif
 
 
@@ -48,6 +67,7 @@ char *argv[];
 	printf("s:	__RENAME__:#	mv:\n");
 	printf("s:	__AOUT2EXE__:#	aout2exe:\n");
 	printf("s:	__REMOVE__:#	rm -f:\n");
+	printf("s:__ECHO__:echo:\n");
 	printf("s:__COPY__:cp:\n");
 	printf("s:__RM__:rm -f:\n");
 
@@ -86,6 +106,12 @@ char *argv[];
 		printf("s:__HOSTCC__:$(CC):\n");
 		printf("s:__HOSTCCOPTIONS__:%s:\n", EXTENDCCOPT);
 	}
+#if	defined (DEFFDSETSIZE) && defined (FD_SETSIZE)
+	if (FD_SETSIZE < MAXOPENFILE)
+		printf("s:__FDSETSIZE__:-DFD_SETSIZE=%d:\n", MAXOPENFILE);
+	else
+#endif
+	printf("s:__FDSETSIZE__::\n");
 	printf("s:__MEM__::\n");
 	printf("s:__SHMEM__::\n");
 	printf("s:__BSHMEM__::\n");
