@@ -5,9 +5,7 @@
  */
 
 #include "fd.h"
-#include "term.h"
 #include "func.h"
-#include "kctype.h"
 
 #ifdef	USEUNAME
 #include <sys/utsname.h>
@@ -143,50 +141,6 @@ int plus;
 	return(cp);
 }
 
-/*
- *	ascnumeric(buf, n, 0, max): same as sprintf(buf, "%d", n)
- *	ascnumeric(buf, n, max + 1, max): same as sprintf(buf, "%0*d", max, n)
- *	ascnumeric(buf, n, max, max): same as sprintf(buf, "%*d", max, n)
- *	ascnumeric(buf, n, -1, max): same as sprintf(buf, "%-*d", max, n)
- *	ascnumeric(buf, n, x, max): like as sprintf(buf, "%*d", max, n)
- *	ascnumeric(buf, n, -x, max): like as sprintf(buf, "%-*d", max, n)
- */
-char *ascnumeric(buf, n, digit, max)
-char *buf;
-off_t n;
-int digit, max;
-{
-	char tmp[MAXLONGWIDTH * 2 + 1];
-	int i, j, d;
-
-	i = j = 0;
-	d = digit;
-	if (digit < 0) digit = -digit;
-	if (n < 0) tmp[i++] = '?';
-	else if (!n) tmp[i++] = '0';
-	else {
-		for (;;) {
-			tmp[i++] = '0' + n % 10;
-			if (!(n /= 10) || i >= max) break;
-			if (digit > 1 && ++j >= digit) {
-				if (i >= max - 1) break;
-				tmp[i++] = ',';
-				j = 0;
-			}
-		}
-		if (n) for (j = 0; j < i; j++) if (tmp[j] != ',') tmp[j] = '9';
-	}
-
-	if (d <= 0) j = 0;
-	else if (d > max) for (j = 0; j < max - i; j++) buf[j] = '0';
-	else for (j = 0; j < max - i; j++) buf[j] = ' ';
-	while (i--) buf[j++] = tmp[i];
-	if (d < 0) for (; j < max; j++) buf[j] = ' ';
-	buf[j] = '\0';
-
-	return(buf);
-}
-
 #ifdef	_NOORIGSHELL
 static char *NEAR strtkbrk(s, c, evaldq)
 char *s, *c;
@@ -303,10 +257,10 @@ char *argv[];
 char *evalcomstr(path, delim)
 char *path, *delim;
 {
-# if	!MSDOS && !defined (_NOKANJIFCONV)
-	char *buf;
+# ifndef	_NOKANJIFCONV
+	char *tmp;
 # endif
-	char *cp, *next, *tmp, *epath;
+	char *cp, *next, *epath;
 	int i, len, size;
 
 	epath = next = NULL;
@@ -322,34 +276,19 @@ char *path, *delim;
 		}
 		next = cp + len;
 		if (len) {
-			tmp = _evalpath(cp, next, 0, 0);
-# if	!MSDOS && !defined (_NOKANJIFCONV)
-			len = strlen(tmp) * 3 + 3;
-			buf = malloc2(len + 1);
-			cp = kanjiconv2(buf, tmp,
-				len, DEFCODE, getkcode(tmp));
-# else
+			cp = _evalpath(cp, next, 0, 0);
+# ifndef	_NOKANJIFCONV
+			tmp = newkanjiconv(cp, DEFCODE, getkcode(cp), L_FNAME);
+			if (cp != tmp) free(cp);
 			cp = tmp;
 # endif
 			len = strlen(cp);
 		}
-# ifdef	FAKEUNINIT
-		else {
-			/* fake for -Wuninitialized */
-#  if	!MSDOS && !defined (_NOKANJIFCONV)
-			buf =
-#  endif
-			tmp = NULL;
-		}
-# endif
 
-		epath = (char *)realloc2(epath, size + len + i + 1);
+		epath = realloc2(epath, size + len + i + 1);
 		if (len) {
 			strcpy(&(epath[size]), cp);
-# if	!MSDOS && !defined (_NOKANJIFCONV)
-			free(buf);
-# endif
-			free(tmp);
+			free(cp);
 			size += len;
 		}
 		if (i) {
@@ -375,7 +314,7 @@ int delim;
 	epath = next = NULL;
 	size = 0;
 	for (cp = paths; cp; cp = next) {
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 		if (_dospath(cp)) next = strchr(&(cp[2]), delim);
 		else
 #endif
@@ -390,7 +329,7 @@ int delim;
 #ifdef	FAKEUNINIT
 		else tmp = NULL;	/* fake for -Wuninitialized */
 #endif
-		epath = (char *)realloc2(epath, size + len + 1 + 1);
+		epath = realloc2(epath, size + len + 1 + 1);
 		if (len) {
 			strcpy(&(epath[size]), cp);
 			free(tmp);
@@ -438,7 +377,7 @@ char *name;
 }
 #endif	/* !MSDOS || !_NOORIGSHELL */
 
-#if	!MSDOS && defined (_NOORIGSHELL)
+#ifdef	_NOORIGSHELL
 VOID adjustpath(VOID_A)
 {
 	char *cp, *path;
@@ -446,14 +385,10 @@ VOID adjustpath(VOID_A)
 	if (!(cp = getconstvar("PATH"))) return;
 
 	path = evalpaths(cp, PATHDELIM);
-	if (strpathcmp(path, cp)) {
-		cp = (char *)malloc2(strlen(path) + 5 + 1);
-		strcpy(strcpy2(cp, "PATH="), path);
-		if (putenv2(cp) < 0) error("PATH");
-	}
+	if (strpathcmp(path, cp)) setenv2("PATH", path, 1);
 	free(path);
 }
-#endif	/* !MSDOS && _NOORIGSHELL */
+#endif	/* _NOORIGSHELL */
 
 char *includepath(path, plist)
 char *path, *plist;
@@ -464,14 +399,14 @@ char *path, *plist;
 	if (!plist || !*plist) return(NULL);
 	next = plist;
 	for (cp = next; cp && *cp; cp = next) {
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 		if (_dospath(cp)) next = strchr(&(cp[2]), PATHDELIM);
 		else
 #endif
 		next = strchr(cp, PATHDELIM);
 		len = (next) ? (next++) - cp : strlen(cp);
 		while (len > 1 && cp[len - 1] == _SC_) len--;
-#if	MSDOS
+#ifdef	BSPATHDELIM
 		if (onkanji1(cp, len - 1)) len++;
 #endif
 		if (len > 0 && !strnpathcmp(path, cp, len)
@@ -520,11 +455,11 @@ u_char *fp, *dp, *wp;
 int evalprompt(bufp, prompt)
 char **bufp, *prompt;
 {
-#if	!MSDOS
+#ifndef	NOUID
 	uidtable *up;
-# ifdef	USEUNAME
+#endif
+#if	!MSDOS && defined (USEUNAME)
 	struct utsname uts;
-# endif
 #endif
 	char *cp, *tmp, line[MAXPATHLEN];
 	ALLOC_T size;
@@ -560,13 +495,16 @@ char **bufp, *prompt;
 				line[1] = '\0';
 				break;
 			case '!':
-				long2str(line, histno[0] + 1, sizeof(line));
+				snprintf2(line, sizeof(line), "%d",
+					(int)(histno[0] + 1));
 				break;
-#if	!MSDOS
+#ifndef	NOUID
 			case 'u':
 				if ((up = finduid(getuid(), NULL)))
 					cp = up -> name;
 				break;
+#endif
+#if	!MSDOS
 			case 'h':
 			case 'H':
 # ifdef	USEUNAME
@@ -591,7 +529,7 @@ char **bufp, *prompt;
 				if (!physical_path || !Xgetwd(line))
 					tmp = fullpath;
 				else tmp = line;
-#if	MSDOS || !defined (_NODOSDRIVE)
+#ifdef	_USEDOSPATH
 				if (_dospath(tmp)) tmp += 2;
 #endif
 				cp = strrdelim(tmp, 0);
@@ -666,28 +604,7 @@ char **bufp, *prompt;
 	return(len);
 }
 
-#if	FD >= 2
-char **duplvar(var, margin)
-char **var;
-int margin;
-{
-	char **dupl;
-	int i, n;
-
-	if (margin < 0) {
-		if (!var) return(NULL);
-		margin = 0;
-	}
-	n = countvar(var);
-	dupl = (char **)malloc2((n + margin + 1) * sizeof(char *));
-	for (i = 0; i < n; i++) dupl[i] = strdup2(var[i]);
-	dupl[i] = NULL;
-	return(dupl);
-}
-#endif	/* FD >= 2 */
-
 #ifndef	_NOARCHIVE
-/*ARGSUSED*/
 char *getext(ext, flagsp)
 char *ext;
 u_char *flagsp;
@@ -695,12 +612,12 @@ u_char *flagsp;
 	char *tmp;
 
 	*flagsp = 0;
+# if	FD >= 2
 	if (*ext == '/') {
 		ext++;
-# if	!MSDOS
 		*flagsp |= LF_IGNORECASE;
-# endif
 	}
+# endif
 
 	if (*ext == '*') tmp = strdup2(ext);
 	else {
@@ -721,7 +638,7 @@ int flags2, strict;
 	if (*ext1 == '*') ext1++;
 	if (*ext2 == '*') ext2++;
 	if (!strict && *ext1 != '.' && *ext2 == '.') ext2++;
-# if	!MSDOS
+# ifndef	PATHNOCASE
 	if ((flags1 & LF_IGNORECASE) || (flags2 & LF_IGNORECASE))
 		return(strcasecmp2(ext1, ext2));
 # endif
@@ -829,9 +746,9 @@ int c, tenkey;
 		}
 		else if (ismsb(c)) {
 			buf[i++] = '\\';
-			buf[i++] = (c / (8 * 8)) + '0';
-			buf[i++] = ((c % (8 * 8)) / 8) + '0';
-			buf[i++] = (c % 8) + '0';
+			buf[i++] = (c & 0700 >> 6) + '0';
+			buf[i++] = (c & 0070 >> 3) + '0';
+			buf[i++] = (c & 0007) + '0';
 		}
 		else buf[i++] = c;
 	}
@@ -841,7 +758,8 @@ int c, tenkey;
 
 char *decodestr(s, lenp, evalhat)
 char *s;
-int *lenp, evalhat;
+u_char *lenp;
+int evalhat;
 {
 	char *cp;
 	int i, j, n;
@@ -879,7 +797,7 @@ int *lenp, evalhat;
 	return(s);
 }
 
-#if	!MSDOS && !defined (_NOKEYMAP)
+#ifndef	_NOKEYMAP
 char *encodestr(s, len)
 char *s;
 int len;
@@ -890,13 +808,13 @@ int len;
 	cp = malloc2(len * 4 + 1);
 	j = 0;
 	if (s) for (i = 0; i < len; i++) {
-#ifdef	CODEEUC
+# ifdef	CODEEUC
 		if (isekana(s, i)) cp[j++] = s[i++];
 		else
-#else
+# else
 		if (iskna(s[i])) /*EMPTY*/;
 		else
-#endif
+# endif
 		if (iskanji1(s, i)) cp[j++] = s[i++];
 		else if (isctl(s[i]) || ismsb(s[i])) {
 			for (n = 0; escapechar[n]; n++)
@@ -925,4 +843,4 @@ int len;
 	cp[j] = '\0';
 	return(cp);
 }
-#endif	/* !MSDOS && !_NOKEYMAP */
+#endif	/* !_NOKEYMAP */
