@@ -11,6 +11,9 @@
 #include <string.h>
 #include <errno.h>
 #include "machine.h"
+#include "printf.h"
+#include "kctype.h"
+#include "pathname.h"
 
 # ifndef	NOUNISTDH
 # include <unistd.h>
@@ -20,16 +23,12 @@
 # include <stdlib.h>
 # endif
 #endif	/* !FD */
-#include <ctype.h>
-
-#if	MSDOS && defined (_NOUSELFN) && !defined (_NODOSDRIVE)
-#define	_NODOSDRIVE
-#endif
 
 #if	MSDOS
 #include <process.h>
 #include "unixemu.h"
 # ifndef	FD
+# include <fcntl.h>
 # include <dos.h>
 # include <io.h>
 # define	DS_IRDONLY	001
@@ -42,13 +41,19 @@
 			| DS_IFDIR | DS_IARCHIVE)
 # endif	/* !FD */
 #define	EXTWIDTH	4
-# if	defined (DJGPP) && DJGPP < 2
+# if	defined (__TURBOC__) || (defined (DJGPP) && DJGPP < 2)
 # include <dir.h>
+# endif
+# if	defined (DJGPP) && DJGPP < 2
 # define	find_t	ffblk
 # define	_dos_findfirst(p, a, f)	findfirst(p, f, a)
 # define	_dos_findnext		findnext
 # define	_ENOENT_		ENMFILE
 # else
+# define	ff_attrib		attrib
+# define	ff_ftime		wr_time
+# define	ff_fdate		wr_date
+# define	ff_fsize		size
 # define	ff_name			name
 # define	_ENOENT_		ENOENT
 # endif
@@ -66,8 +71,6 @@
 # endif
 #endif	/* !MSDOS */
 
-#include "pathname.h"
-
 #ifdef	NOERRNO
 extern int errno;
 #endif
@@ -84,10 +87,9 @@ extern char *strdup2 __P_((char *));
 extern char *strndup2 __P_((char *, int));
 extern char *strcpy2 __P_((char *, char *));
 extern char *strncpy2 __P_((char *, char *, int));
-extern int snprintf2 __P_((char *, int, CONST char *, ...));
-# if	MSDOS || !defined (_NODOSDRIVE)
+# ifdef	_USEDOSPATH
 extern int _dospath __P_((char *));
-#endif
+# endif
 extern DIR *Xopendir __P_((char *));
 extern int Xclosedir __P_((DIR *));
 extern struct dirent *Xreaddir __P_((DIR *));
@@ -101,7 +103,7 @@ extern VOID demacroarg __P_((char **));
 extern char *progpath;
 #else	/* !FD */
 #define	getenv2		(char *)getenv
-static VOID NEAR allocerror __P_((VOID_A));
+VOID error __P_((char *));
 char *malloc2 __P_((ALLOC_T));
 char *realloc2 __P_((VOID_P, ALLOC_T));
 char *c_realloc __P_((char *, ALLOC_T, ALLOC_T *));
@@ -109,16 +111,19 @@ char *strdup2 __P_((char *));
 char *strndup2 __P_((char *, int));
 char *strcpy2 __P_((char *, char *));
 char *strncpy2 __P_((char *, char *, int));
-char *ascnumeric __P_((char *, off_t, int, int));
-#define	_dospath(s)	(isalpha(*(s)) && (s)[1] == ':')
 # if	MSDOS
+# define	_dospath(s)	(isalpha(*(s)) && (s)[1] == ':')
 DIR *Xopendir __P_((char *));
 int Xclosedir __P_((DIR *));
 struct dirent *Xreaddir __P_((DIR *));
+static u_int NEAR getdosmode __P_((u_int));
+static time_t NEAR getdostime __P_((u_int, u_int));
+int Xstat __P_((char *, struct stat *));
 # else
 # define	Xopendir	opendir
 # define	Xclosedir	closedir
 # define	Xreaddir	readdir
+# define	Xstat		stat
 # endif
 # ifdef	DJGPP
 char *Xgetwd __P_((char *));
@@ -129,11 +134,10 @@ char *Xgetwd __P_((char *));
 #  define	Xgetwd(p)	(char *)getcwd(p, MAXPATHLEN)
 #  endif
 # endif
-#define	Xstat(f, s)	(stat(f, s) ? -1 : 0)
 #define	Xaccess		access
 #endif	/* !FD */
 
-static char *NEAR getvar __P_((char *, int));
+static char *NEAR getenvvar __P_((char *, int));
 static int NEAR setvar __P_((char *, char *, int));
 static int NEAR ismeta __P_((char *s, int, int, int));
 #ifdef	_NOORIGGLOB
@@ -142,7 +146,7 @@ static char *NEAR cnvregexp __P_((char *, int));
 static int NEAR _regexp_exec __P_((char **, char *));
 #endif
 static char *NEAR catpath __P_((char *, char *, int *, int, int));
-#if	MSDOS
+#ifdef	NODIRLOOP
 static int NEAR _evalwild __P_((int, char ***, char *, char *, int));
 #else
 static int NEAR _evalwild __P_((int, char ***, char *, char *, int,
@@ -161,7 +165,7 @@ static int NEAR isexecute __P_((char *, int, int));
 static int NEAR extaccess __P_((char *, char *, int, int));
 #endif
 #if	!defined (FDSH) && !defined (_NOCOMPLETE)
-# if	!MSDOS
+# ifndef	NOUID
 static int NEAR completeuser __P_((char *, int, int, char ***));
 # endif
 static int NEAR completefile __P_((char *, int, int, char ***,
@@ -184,44 +188,12 @@ static char *NEAR insertarg __P_((char *, int, char *, int, int));
 static int NEAR evalvar __P_((char **, int, char **, int));
 static char *NEAR replacebackquote __P_((char *, int *, char *, int));
 
-#ifdef	LSI_C
-#include <jctype.h>
-#define	iskna		iskana
-#define	issjis1		iskanji
-#define	issjis2		iskanji2
-#else	/* !LSI_C */
-#define	KC_SJIS1	0001
-#define	KC_SJIS2	0002
-#define	KC_KANA		0004
-#define	KC_EUCJP	0010
-#define	iskna(c)	(kctypetable[(u_char)(c)] & KC_KANA)
-
-# ifndef	issjis1
-# define	issjis1(c)	(kctypetable[(u_char)(c)] & KC_SJIS1)
-# endif
-# ifndef	issjis2
-# define	issjis2(c)	(kctypetable[(u_char)(c)] & KC_SJIS2)
-# endif
-
-# ifndef	iseuc
-# define	iseuc(c)	(kctypetable[(u_char)(c)] & KC_EUCJP)
-# endif
-#endif	/* !LSI_C */
-
-#define	isekana(s, i)	((u_char)((s)[i]) == 0x8e && iskna((s)[(i) + 1]))
-
-#ifdef	CODEEUC
-#define	iskanji1(s, i)	(iseuc((s)[i]) && iseuc((s)[(i) + 1]))
-#else
-#define	iskanji1(s, i)	(issjis1((s)[i]) && issjis2((s)[(i) + 1]))
-#endif
-
 #define	BUFUNIT		32
 #define	b_size(n, type) ((((n) / BUFUNIT) + 1) * BUFUNIT * sizeof(type))
 #define	b_realloc(ptr, n, type) \
 			(((n) % BUFUNIT) ? ((type *)(ptr)) \
 			: (type *)realloc2(ptr, b_size(n, type)))
-#define	getconstvar(s)	(getvar(s, sizeof(s) - 1))
+#define	getconstvar(s)	(getenvvar(s, sizeof(s) - 1))
 
 char **argvar = NULL;
 #ifndef	_NOUSEHASH
@@ -239,136 +211,30 @@ char *(*backquotefunc)__P_((char *)) = NULL;
 #ifndef	MINIMUMSHELL
 char *(*posixsubstfunc)__P_((char *, int *)) = NULL;
 #endif
-#if	!MSDOS
+#ifndef	PATHNOCASE
 int pathignorecase = 0;
-#endif	/* !MSDOS */
-#ifndef	LSI_C
-u_char uppercase[256] = {
-	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,	/* 0x00 */
-	0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
-	0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,	/* 0x10 */
-	0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
-	0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,	/* 0x20 */
-	0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
-	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,	/* 0x30 */
-	0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
-	0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,	/* 0x40 */
-	0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
-	0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57,	/* 0x50 */
-	0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f,
-	0x60, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,	/* 0x60 */
-	0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
-	0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57,	/* 0x70 */
-	0x58, 0x59, 0x5a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,
-	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,	/* 0x80 */
-	0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
-	0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97,	/* 0x90 */
-	0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
-	0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,	/* 0xa0 */
-	0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
-	0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7,	/* 0xb0 */
-	0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
-	0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7,	/* 0xc0 */
-	0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
-	0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7,	/* 0xd0 */
-	0xd8, 0xd9, 0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
-	0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7,	/* 0xe0 */
-	0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
-	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,	/* 0xf0 */
-	0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
-};
-u_char lowercase[256] = {
-	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,	/* 0x00 */
-	0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
-	0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,	/* 0x10 */
-	0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
-	0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,	/* 0x20 */
-	0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
-	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,	/* 0x30 */
-	0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
-	0x40, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,	/* 0x40 */
-	0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
-	0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,	/* 0x50 */
-	0x78, 0x79, 0x7a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f,
-	0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,	/* 0x60 */
-	0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
-	0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,	/* 0x70 */
-	0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,
-	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,	/* 0x80 */
-	0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
-	0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97,	/* 0x90 */
-	0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
-	0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,	/* 0xa0 */
-	0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
-	0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7,	/* 0xb0 */
-	0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
-	0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7,	/* 0xc0 */
-	0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
-	0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7,	/* 0xd0 */
-	0xd8, 0xd9, 0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
-	0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7,	/* 0xe0 */
-	0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
-	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,	/* 0xf0 */
-	0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
-};
-# ifdef	FD
-extern u_char kctypetable[256];
-# else	/* !FD */
-u_char kctypetable[256] = {
-	0200, 0200, 0200, 0200, 0200, 0200, 0200, 0200,	/* 0x00 */
-	0200, 0200, 0200, 0200, 0200, 0200, 0200, 0200,
-	0200, 0200, 0200, 0200, 0200, 0200, 0200, 0200,	/* 0x10 */
-	0200, 0200, 0200, 0200, 0200, 0200, 0200, 0200,
-	   0, 0060, 0060, 0060, 0060, 0060, 0060, 0060,	/* 0x20 */
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,	/* 0x30 */
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,	/* 0x40 */
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,	/* 0x50 */
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,
-	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,	/* 0x60 */
-	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,
-	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,	/* 0x70 */
-	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0200,
-	0002, 0003, 0003, 0003, 0003, 0003, 0003, 0003,	/* 0x80 */
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,	/* 0x90 */
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,
-	0002, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xa0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xb0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xc0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xd0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,	/* 0xe0 */
-	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,
-	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,	/* 0xf0 */
-	0013, 0013, 0013, 0013, 0013, 0010, 0010,    0
-};
-# endif	/* !FD */
-#endif	/* !LSI_C */
+#endif
 
 static int skipdotfile = 0;
 #if	!defined (USEREGCMP) && !defined (USEREGCOMP) && !defined (USERE_COMP)
 static char *wildsymbol1 = "?";
 static char *wildsymbol2 = "*";
 #endif
-#if	!MSDOS
+#if	defined (FD) && !defined (NOUID)
 static uidtable *uidlist = NULL;
 static int maxuid = 0;
 static gidtable *gidlist = NULL;
 static int maxgid = 0;
-#endif	/* !MSDOS */
+#endif	/* FD && !NOUID */
 
 
 #ifndef	FD
-static VOID NEAR allocerror(VOID_A)
+VOID error(s)
+char *s;
 {
-	fputs("fatal error: memory allocation error\n", stderr);
-	fflush(stderr);
+	fputs(s, stderr);
+	fputs(": memory allocation error", stderr);
+	fputnl(stderr);
 	exit(2);
 }
 
@@ -378,7 +244,7 @@ ALLOC_T size;
 	char *tmp;
 
 	if (!size || !(tmp = (char *)malloc(size))) {
-		allocerror();
+		error("malloc()");
 #ifdef	FAKEUNINIT
 		tmp = NULL;	/* fake for -Wuninitialized */
 #endif
@@ -395,7 +261,7 @@ ALLOC_T size;
 	if (!size
 	|| !(tmp = (ptr) ? (char *)realloc(ptr, size) : (char *)malloc(size)))
 	{
-		allocerror();
+		error("realloc()");
 #ifdef	FAKEUNINIT
 		tmp = NULL;	/* fake for -Wuninitialized */
 #endif
@@ -423,7 +289,7 @@ char *s;
 
 	if (!s) return(NULL);
 	n = strlen(s);
-	if (!(tmp = (char *)malloc((ALLOC_T)n + 1))) allocerror();
+	if (!(tmp = (char *)malloc((ALLOC_T)n + 1))) error("malloc()");
 	memcpy(tmp, s, n + 1);
 	return(tmp);
 }
@@ -437,7 +303,7 @@ int n;
 
 	if (!s) return(NULL);
 	for (i = 0; i < n; i++) if (!s[i]) break;
-	if (!(tmp = (char *)malloc((ALLOC_T)i + 1))) allocerror();
+	if (!(tmp = (char *)malloc((ALLOC_T)i + 1))) error("malloc()");
 	memcpy(tmp, s, i);
 	tmp[i] = '\0';
 	return(tmp);
@@ -464,50 +330,6 @@ int n;
 	return(&(s1[i]));
 }
 
-/*
- *	ascnumeric(buf, n, 0, max): same as sprintf(buf, "%d", n)
- *	ascnumeric(buf, n, max + 1, max): same as sprintf(buf, "%0*d", max, n)
- *	ascnumeric(buf, n, max, max): same as sprintf(buf, "%*d", max, n)
- *	ascnumeric(buf, n, -1, max): same as sprintf(buf, "%-*d", max, n)
- *	ascnumeric(buf, n, x, max): like as sprintf(buf, "%*d", max, n)
- *	ascnumeric(buf, n, -x, max): like as sprintf(buf, "%-*d", max, n)
- */
-char *ascnumeric(buf, n, digit, max)
-char *buf;
-off_t n;
-int digit, max;
-{
-	char tmp[MAXLONGWIDTH * 2 + 1];
-	int i, j, d;
-
-	i = j = 0;
-	d = digit;
-	if (digit < 0) digit = -digit;
-	if (n < 0) tmp[i++] = '?';
-	else if (!n) tmp[i++] = '0';
-	else {
-		for (;;) {
-			tmp[i++] = '0' + n % 10;
-			if (!(n /= 10) || i >= max) break;
-			if (digit > 1 && ++j >= digit) {
-				if (i >= max - 1) break;
-				tmp[i++] = ',';
-				j = 0;
-			}
-		}
-		if (n) for (j = 0; j < i; j++) if (tmp[j] != ',') tmp[j] = '9';
-	}
-
-	if (d <= 0) j = 0;
-	else if (d > max) for (j = 0; j < max - i; j++) buf[j] = '0';
-	else for (j = 0; j < max - i; j++) buf[j] = ' ';
-	while (i--) buf[j++] = tmp[i];
-	if (d < 0) for (; j < max; j++) buf[j] = ' ';
-	buf[j] = '\0';
-
-	return(buf);
-}
-
 # if	MSDOS
 DIR *Xopendir(dir)
 char *dir;
@@ -573,6 +395,57 @@ DIR *dirp;
 
 	return(&d);
 }
+
+static u_int NEAR getdosmode(attr)
+u_int attr;
+{
+	u_int mode;
+
+	mode = 0;
+	if (attr & DS_IARCHIVE) mode |= S_ISVTX;
+	if (!(attr & DS_IHIDDEN)) mode |= S_IREAD;
+	if (!(attr & DS_IRDONLY)) mode |= S_IWRITE;
+	if (attr & DS_IFDIR) mode |= (S_IFDIR | S_IEXEC);
+	else if (attr & DS_IFLABEL) mode |= S_IFIFO;
+	else if (attr & DS_IFSYSTEM) mode |= S_IFSOCK;
+	else mode |= S_IFREG;
+
+	return(mode);
+}
+
+static time_t NEAR getdostime(d, t)
+u_int d, t;
+{
+	struct tm tm;
+
+	tm.tm_year = 1980 + ((d >> 9) & 0x7f);
+	tm.tm_year -= 1900;
+	tm.tm_mon = ((d >> 5) & 0x0f) - 1;
+	tm.tm_mday = (d & 0x1f);
+	tm.tm_hour = ((t >> 11) & 0x1f);
+	tm.tm_min = ((t >> 5) & 0x3f);
+	tm.tm_sec = ((t << 1) & 0x3e);
+	tm.tm_isdst = -1;
+
+	return(mktime(&tm));
+}
+
+int Xstat(path, stp)
+char *path;
+struct stat *stp;
+{
+	struct find_t find;
+
+	if (_dos_findfirst(path, SEARCHATTRS, &find) != 0) return(-1);
+	stp -> st_mode = getdosmode(find.ff_attrib);
+	stp -> st_mtime = getdostime(find.ff_fdate, find.ff_ftime);
+	stp -> st_size = find.ff_fsize;
+	stp -> st_ctime = stp -> st_atime = stp -> st_mtime;
+	stp -> st_dev = stp -> st_ino = 0;
+	stp -> st_nlink = 1;
+	stp -> st_uid = stp -> st_gid = -1;
+	return(0);
+}
 # endif	/* MSDOS */
 
 # ifdef	DJGPP
@@ -599,9 +472,9 @@ int d;
 	if (d && _dospath(s)) return(s + 1);
 	for (i = 0; s[i]; i++) {
 		if (s[i] == _SC_) return(&(s[i]));
-#if	MSDOS
+# ifdef	BSPATHDELIM
 		if (iskanji1(s, i)) i++;
-#endif
+# endif
 	}
 	return(NULL);
 }
@@ -617,9 +490,9 @@ int d;
 	else cp = NULL;
 	for (i = 0; s[i]; i++) {
 		if (s[i] == _SC_) cp = &(s[i]);
-#if	MSDOS
+# ifdef	BSPATHDELIM
 		if (iskanji1(s, i)) i++;
-#endif
+# endif
 	}
 	return(cp);
 }
@@ -628,7 +501,7 @@ int d;
 char *strrdelim2(s, eol)
 char *s, *eol;
 {
-#if	MSDOS
+#ifdef	BSPATHDELIM
 	char *cp;
 	int i;
 
@@ -648,12 +521,12 @@ int isdelim(s, ptr)
 char *s;
 int ptr;
 {
-#if	MSDOS
+#ifdef	BSPATHDELIM
 	int i;
 #endif
 
 	if (ptr < 0 || s[ptr] != _SC_) return(0);
-#if	MSDOS
+#ifdef	BSPATHDELIM
 	if (--ptr < 0) return(1);
 	if (!ptr) return(!iskanji1(s, 0));
 
@@ -686,7 +559,7 @@ char *s;
 			continue;
 		}
 		cp = NULL;
-#if	MSDOS
+#ifdef	BSPATHDELIM
 		if (iskanji1(s, i)) i++;
 #endif
 	}
@@ -728,7 +601,7 @@ char *buf, *s1, *s2;
 				continue;
 			}
 			cp = NULL;
-#if	MSDOS
+#ifdef	BSPATHDELIM
 			if (iskanji1(s1, i)) {
 				if (!s1[++i]) break;
 				buf[i] = s1[i];
@@ -757,34 +630,6 @@ char *s1, *s2;
 {
 	for (;;) {
 		if (toupper2(*s1) != toupper2(*s2)) return(*s1 - *s2);
-# ifndef	CODEEUC
-		if (issjis1(*s1)) {
-			s1++;
-			s2++;
-			if (*s1 != *s2) return(*s1 - *s2);
-		}
-# endif
-		if (!*s1) break;
-		s1++;
-		s2++;
-	}
-	return(0);
-}
-
-int strpathcmp2(s1, s2)
-char *s1, *s2;
-{
-#if	!MSDOS
-	if (!pathignorecase) for (;;) {
-		if (*s1 != *s2) return(*s1 - *s2);
-		if (!*s1) break;
-		s1++;
-		s2++;
-	}
-	else
-#endif
-	for (;;) {
-		if (toupper2(*s1) != toupper2(*s2)) return(*s1 - *s2);
 #ifndef	CODEEUC
 		if (issjis1(*s1)) {
 			s1++;
@@ -799,19 +644,10 @@ char *s1, *s2;
 	return(0);
 }
 
-int strnpathcmp2(s1, s2, n)
+int strncasecmp2(s1, s2, n)
 char *s1, *s2;
 int n;
 {
-#if	!MSDOS
-	if (!pathignorecase) while (n-- > 0) {
-		if (*s1 != *s2) return(*s1 - *s2);
-		if (!*s1) break;
-		s1++;
-		s2++;
-	}
-	else
-#endif
 	while (n-- > 0) {
 		if (toupper2(*s1) != toupper2(*s2)) return(*s1 - *s2);
 #ifndef	CODEEUC
@@ -829,7 +665,24 @@ int n;
 	return(0);
 }
 
-static char *NEAR getvar(ident, len)
+#ifndef	PATHNOCASE
+int strpathcmp2(s1, s2)
+char *s1, *s2;
+{
+	if (pathignorecase) return(strcasecmp2(s1, s2));
+	else return(strcmp(s1, s2));
+}
+
+int strnpathcmp2(s1, s2, n)
+char *s1, *s2;
+int n;
+{
+	if (pathignorecase) return(strncasecmp2(s1, s2, n));
+	else return(strncmp(s1, s2, n));
+}
+#endif	/* !PATHNOCASE */
+
+static char *NEAR getenvvar(ident, len)
 char *ident;
 int len;
 {
@@ -895,6 +748,38 @@ char *s;
 	return(s);
 }
 
+char *getbasename(s)
+char *s;
+{
+	char *cp;
+
+	if ((cp = strrdelim(s, 1))) return(cp + 1);
+	return(s);
+}
+
+char *getshellname(s, loginp, restrictedp)
+char *s;
+int *loginp, *restrictedp;
+{
+	if (loginp) *loginp = 0;
+	if (restrictedp) *restrictedp = 0;
+	if (*s == '-') {
+		s++;
+		if (loginp) *loginp = 1;
+	}
+#ifdef	PATHNOCASE
+	if (tolower2(*s) == 'r')
+#else
+	if (*s == 'r')
+#endif
+	{
+		s++;
+		if (restrictedp) *restrictedp = 1;
+	}
+
+	return(s);
+}
+
 static int NEAR ismeta(s, ptr, quote, len)
 char *s;
 int ptr, quote, len;
@@ -911,11 +796,11 @@ int ptr, quote, len;
 		if (!s[ptr + 1]) return(0);
 		if (quote) {
 			if (s[ptr + 1] == quote && !s[ptr + 2]) return(0);
-# if	MSDOS
+# ifdef	BSPATHDELIM
 			if (!strchr(DQ_METACHAR, s[ptr + 1])) return(0);
 # endif
 		}
-# if	MSDOS
+# ifdef	BSPATHDELIM
 		else if (!strchr(METACHAR, s[ptr + 1])) return(0);
 # endif
 	}
@@ -923,19 +808,6 @@ int ptr, quote, len;
 #endif	/* !FAKEMETA */
 }
 
-char *long2str(s, n, size)
-char *s;
-long n;
-int size;
-{
-#ifdef	FD
-	snprintf2(s, size, "%ld", n);
-	return(s);
-#else
-	return(ascnumeric(s, (off_t)n, 0, size - 1));
-#endif
-}
-
 #ifdef	_NOORIGGLOB
 static char *NEAR cnvregexp(s, len)
 char *s;
@@ -997,7 +869,7 @@ int len;
 #ifdef	USERE_COMP
 extern char *re_comp __P_((CONST char *));
 extern int re_exec __P_((CONST char *));
-# if	MSDOS
+# ifdef	PATHNOCASE
 extern int re_ignore_case;
 # endif
 
@@ -1005,7 +877,7 @@ reg_t *regexp_init(s, len)
 char *s;
 int len;
 {
-# if	MSDOS
+# ifdef	PATHNOCASE
 	re_ignore_case = 1;
 # endif
 	skipdotfile = (*s == '*' || *s == '?' || *s == '[');
@@ -1044,7 +916,7 @@ int len;
 	skipdotfile = (*s == '*' || *s == '?' || *s == '[');
 	s = cnvregexp(s, len);
 	re = (reg_t *)malloc2(sizeof(reg_t));
-# if	MSDOS
+# ifdef	PATHNOCASE
 	if (regcomp(re, s, REG_EXTENDED | REG_ICASE)) {
 # else
 	if (regcomp(re, s, REG_EXTENDED | (pathignorecase ? REG_ICASE : 0))) {
@@ -1142,7 +1014,7 @@ int len;
 #ifdef	BASHSTYLE
 	/* bash treats a character quoted by \ in "[]" as a character itself */
 				paren[plen++] = PMETA;
-# if	!MSDOS
+# ifndef	PATHNOCASE
 				if (!pathignorecase) paren[plen++] = s[i];
 				else
 # endif
@@ -1175,7 +1047,7 @@ int len;
 #endif
 			else if (s[i] == '-') {
 				if (i + 1 >= len) j = '\0';
-#if	!MSDOS
+#ifndef	PATHNOCASE
 				else if (!pathignorecase) j = s[i + 1];
 #endif
 				else j = toupper2(s[i + 1]);
@@ -1217,7 +1089,7 @@ int len;
 					paren[plen++] = s[i++];
 					paren[plen++] = s[i];
 				}
-#if	!MSDOS
+#ifndef	PATHNOCASE
 				else if (!pathignorecase) paren[plen++] = s[i];
 #endif
 				else paren[plen++] = toupper2(s[i]);
@@ -1253,7 +1125,7 @@ int len;
 				cp[j++] = s[i++];
 				cp[j++] = s[i];
 			}
-#if	!MSDOS
+#ifndef	PATHNOCASE
 			else if (!pathignorecase) cp[j++] = s[i];
 #endif
 			else cp[j++] = toupper2(s[i]);
@@ -1290,8 +1162,8 @@ char *s;
 		else
 #endif
 		if (iskanji1(s, n2)) c1 = (c1 << 8) + (u_char)(s[++n2]);
-#if	!MSDOS
-		else if (!pathignorecase);
+#ifndef	PATHNOCASE
+		else if (!pathignorecase) /*EMPTY*/;
 #endif
 		else c1 = toupper2(c1);
 
@@ -1406,7 +1278,7 @@ int *plenp, flen, isoverwrite;
 	return(new);
 }
 
-#if	MSDOS
+#ifdef	NODIRLOOP
 static int NEAR _evalwild(argc, argvp, s, fixed, len)
 int argc;
 char ***argvp, *s, *fixed;
@@ -1419,7 +1291,7 @@ int len, nino;
 devino_t *ino;
 #endif
 {
-#if	!MSDOS
+#ifndef	NODIRLOOP
 	devino_t *dupino;
 #endif
 	DIR *dirp;
@@ -1438,7 +1310,7 @@ devino_t *ino;
 				(argc + 2) * sizeof(char *));
 			(*argvp)[argc++] = fixed;
 		}
-#if	!MSDOS
+#ifndef	NODIRLOOP
 		if (ino) free(ino);
 #endif
 		return(argc);
@@ -1450,7 +1322,7 @@ devino_t *ino;
 		fixed[0] = *s;
 		fixed[1] = ':';
 		fixed[2] = '\0';
-# if	MSDOS
+# ifdef	NODIRLOOP
 		return(_evalwild(argc, argvp, &(s[2]), fixed, 2));
 # else
 		return(_evalwild(argc, argvp, &(s[2]), fixed, 2, nino, ino));
@@ -1463,7 +1335,7 @@ devino_t *ino;
 		if (s[i] == _SC_) break;
 		if (s[i] == '*' || s[i] == '?'
 		|| s[i] == '[' || s[i] == ']') n = 1;
-#if	MSDOS
+#ifdef	BSPATHDELIM
 		if (iskanji1(s, i)) i++;
 #endif
 	}
@@ -1472,7 +1344,7 @@ devino_t *ino;
 		fixed = realloc2(fixed, len + 1 + 1);
 		fixed[len++] = _SC_;
 		fixed[len] = '\0';
-#if	MSDOS
+#ifdef	NODIRLOOP
 		return(_evalwild(argc, argvp, &(s[1]), fixed, len));
 #else
 		return(_evalwild(argc, argvp, &(s[1]), fixed, len, nino, ino));
@@ -1485,7 +1357,7 @@ devino_t *ino;
 		else if (s[i]) {
 			if ((st.st_mode & S_IFMT) != S_IFDIR) free(fixed);
 			else {
-#if	MSDOS
+#ifdef	NODIRLOOP
 				return(_evalwild(argc, argvp,
 					&(s[i + 1]), fixed, len));
 #else
@@ -1503,14 +1375,14 @@ devino_t *ino;
 				(argc + 2) * sizeof(char *));
 			(*argvp)[argc++] = fixed;
 		}
-#if	!MSDOS
+#ifndef	NODIRLOOP
 		if (ino) free(ino);
 #endif
 		return(argc);
 	}
 
 	if (i == 2 && s[i] && s[0] == '*' && s[1] == '*') {
-#if	MSDOS
+#ifdef	NODIRLOOP
 		argc = _evalwild(argc, argvp, &(s[3]), strdup2(fixed), len);
 #else
 		if (!ino) dupino = NULL;
@@ -1528,7 +1400,7 @@ devino_t *ino;
 	}
 	else if (!(re = regexp_init(s, i))) {
 		if (fixed) free(fixed);
-#if	!MSDOS
+#ifndef	NODIRLOOP
 		if (ino) free(ino);
 #endif
 		return(argc);
@@ -1536,7 +1408,7 @@ devino_t *ino;
 	if (!(dirp = Xopendir((len) ? fixed : "."))) {
 		regexp_free(re);
 		if (fixed) free(fixed);
-#if	!MSDOS
+#ifndef	NODIRLOOP
 		if (ino) free(ino);
 #endif
 		return(argc);
@@ -1554,7 +1426,7 @@ devino_t *ino;
 				continue;
 			}
 
-#if	!MSDOS
+#ifndef	NODIRLOOP
 			dupino = (devino_t *)malloc2((nino + 1)
 				* sizeof(devino_t));
 			for (n = 0; n < nino; n++) {
@@ -1567,12 +1439,12 @@ devino_t *ino;
 			if (!re) {
 				if (*(dp -> d_name) == '.') {
 					free(cp);
-#if	!MSDOS
+#ifndef	NODIRLOOP
 					free(dupino);
 #endif
 					continue;
 				}
-#if	MSDOS
+#ifdef	NODIRLOOP
 				argc = _evalwild(argc, argvp, s, cp, l);
 #else
 				for (n = 0; n < nino; n++)
@@ -1588,11 +1460,11 @@ devino_t *ino;
 			}
 			else if (!regexp_exec(re, dp -> d_name, 1)) {
 				free(cp);
-#if	!MSDOS
+#ifndef	NODIRLOOP
 				free(dupino);
 #endif
 			}
-#if	MSDOS
+#ifdef	NODIRLOOP
 			else argc = _evalwild(argc, argvp, &(s[i + 1]), cp, l);
 #else
 			else argc = _evalwild(argc, argvp,
@@ -1609,7 +1481,7 @@ devino_t *ino;
 	Xclosedir(dirp);
 	regexp_free(re);
 	if (fixed) free(fixed);
-#if	!MSDOS
+#ifndef	NODIRLOOP
 	if (ino) free(ino);
 #endif
 	return(argc);
@@ -1629,7 +1501,7 @@ char *s;
 	int argc;
 
 	argv = (char **)malloc2(1 * sizeof(char *));
-#if	MSDOS
+#ifdef	NODIRLOOP
 	argc = _evalwild(0, &argv, s, NULL, 0);
 #else
 	argc = _evalwild(0, &argv, s, NULL, 0, 0, NULL);
@@ -1794,7 +1666,7 @@ int len, exe;
 	}
 	return(-1);
 }
-#endif
+#endif	/* MSDOS */
 
 int searchhash(hpp, com, search)
 hashlist **hpp;
@@ -1843,19 +1715,24 @@ char *com, *search;
 	if ((*hpp = findhash(com, n))) {
 		path = (*hpp) -> path;
 		if (((*hpp) -> type & CM_RECALC)
-		&& (!searchcwd || !Xgetwd(buf) || strcmp(searchcwd, buf)));
+		&& (!searchcwd || !Xgetwd(buf) || strcmp(searchcwd, buf)))
+			/*EMPTY*/;
 		else if (isexecute(path, 0, 1) >= 0) return((*hpp) -> type);
 		rmhash(com, n);
 	}
 #endif
 
-#if	MSDOS && !defined (DISMISS_CURPATH)
+#ifdef	CWDINPATH
 	len = strlen(com);
 	path = malloc2(len + 2 + EXTWIDTH + 1);
 	path[0] = '.';
 	path[1] = _SC_;
 	strcpy(&(path[2]), com);
+# if	MSDOS
 	if ((ret = extaccess(path, ext, len + 2, 1)) < 0) free(path);
+# else
+	if ((ret = isexecute(path, 0, 0)) < 0) free(path);
+# endif
 	else {
 # ifdef	_NOUSEHASH
 		*hpp = (hashlist *)path;
@@ -1868,7 +1745,7 @@ char *com, *search;
 # endif
 		return(ret);
 	}
-#endif	/* MSDOS && !DISMISS_CURPATH */
+#endif	/* CWDINPATH */
 
 	recalc = 0;
 	if ((next = (search) ? search : getconstvar("PATH"))) {
@@ -1925,7 +1802,7 @@ char *com, *search;
 	return(CM_NOTFOUND);
 }
 
-char *searchpath(path, search)
+char *searchexecpath(path, search)
 char *path, *search;
 {
 	hashlist *hp;
@@ -1953,7 +1830,7 @@ char **argv;
 	return(NULL);
 }
 
-#if	!MSDOS
+# ifndef	NOUID
 static int NEAR completeuser(name, len, argc, argvp)
 char *name;
 int len, argc;
@@ -1964,7 +1841,7 @@ char ***argvp;
 	int size;
 
 	len = strlen(name);
-# ifdef	DEBUG
+#  ifdef	DEBUG
 	_mtrace_file = "setpwent(start)";
 	setpwent();
 	if (_mtrace_file) _mtrace_file = NULL;
@@ -2002,7 +1879,7 @@ char ***argvp;
 		_mtrace_file = "endpwent(end)";
 		malloc(0);	/* dummy alloc */
 	}
-# else	/* !DEBUG */
+#  else	/* !DEBUG */
 	setpwent();
 	while ((pwd = getpwent())) {
 		if (strnpathcmp(name, pwd -> pw_name, len)) continue;
@@ -2020,10 +1897,10 @@ char ***argvp;
 		(*argvp)[argc++] = new;
 	}
 	endpwent();
-# endif	/* !DEBUG */
+#  endif	/* !DEBUG */
 	return(argc);
 }
-#endif	/* !MSDOS */
+# endif	/* !NOUID */
 
 static int NEAR completefile(file, len, argc, argvp, dir, dlen, exe)
 char *file;
@@ -2075,15 +1952,15 @@ char ***argvp;
 	char *cp, *tmp, *next;
 	int dlen;
 
-#if	MSDOS && !defined (DISMISS_CURPATH)
+# ifdef	CWDINPATH
 	argc = completefile(file, len, argc, argvp, ".", 1, 1);
-#endif
+# endif
 	if (!(next = getconstvar("PATH"))) return(argc);
 	for (cp = next; cp; cp = next) {
-#if	MSDOS || (defined (FD) && !defined (_NODOSDRIVE))
+# if	MSDOS || (defined (FD) && !defined (_NODOSDRIVE))
 		if (_dospath(cp)) next = strchr(&(cp[2]), PATHDELIM);
 		else
-#endif
+# endif
 		next = strchr(cp, PATHDELIM);
 		dlen = (next) ? (next++) - cp : strlen(cp);
 		tmp = _evalpath(cp, cp + dlen, 1, 1);
@@ -2105,22 +1982,22 @@ int exe;
 
 	dir = path;
 	dlen = 0;
-#if	MSDOS || (defined (FD) && !defined (_NODOSDRIVE))
+# if	MSDOS || (defined (FD) && !defined (_NODOSDRIVE))
 	if (_dospath(path)) {
 		dir += 2;
 		dlen = 2;
 	}
-#endif
+# endif
 
 	if ((file = strrdelim(dir, 0))) {
 		dlen += (file == dir) ? 1 : file - dir;
 		return(completefile(file + 1, strlen(file + 1), argc, argvp,
 			path, dlen, exe));
 	}
-#if	!MSDOS
+# ifndef	NOUID
 	else if (*path == '~')
 		return(completeuser(&(path[1]), len - 1, argc, argvp));
-#endif
+# endif
 	else if (exe && dir == path)
 		return(completeexe(path, len, argc, argvp));
 	return(completefile(path, len, argc, argvp, ".", 1, exe));
@@ -2230,6 +2107,24 @@ char **var;
 	errno = duperrno;
 }
 
+char **duplvar(var, margin)
+char **var;
+int margin;
+{
+	char **dupl;
+	int i, n;
+
+	if (margin < 0) {
+		if (!var) return(NULL);
+		margin = 0;
+	}
+	n = countvar(var);
+	dupl = (char **)malloc2((n + margin + 1) * sizeof(char *));
+	for (i = 0; i < n; i++) dupl[i] = strdup2(var[i]);
+	dupl[i] = NULL;
+	return(dupl);
+}
+
 int parsechar(s, len, spc, qflag, qp, pqp)
 char *s;
 int len, spc, qflag, *qp, *pqp;
@@ -2334,11 +2229,7 @@ int *eolp, *ptrp, qed;
 	}
 	else if (strchr("-=?+", i)) mode = i;
 #ifndef	MINIMUMSHELL
-# if	MSDOS
-	else if (i != '\\' && i != '#') return(-1);
-# else
-	else if (i != '%' && i != '#') return(-1);
-# endif
+	else if (i != RMSUFFIX && i != '#') return(-1);
 	else if ((*bufp)[*ptrp] != i) mode = i;
 	else {
 		(*ptrp)++;
@@ -2404,7 +2295,7 @@ int qed, nonl, nest;
 			*ptrp += (cp - s);
 			continue;
 		}
-		else if (pc != PC_NORMAL);
+		else if (pc != PC_NORMAL) /*EMPTY*/;
 		else if (nonl && s[*ptrp] == '\n') return(-1);
 #ifndef	MINIMUMSHELL
 		else if (nest && s[*ptrp] == '(') {
@@ -2586,21 +2477,27 @@ int plen, *modep;
 			break;
 		case '#':
 			if (!argvar) break;
-			cp = long2str(tmp, countvar(argvar + 1), sizeof(tmp));
+			snprintf2(tmp, sizeof(tmp), "%d",
+				countvar(argvar + 1));
+			cp = tmp;
 			break;
 		case '?':
-			i = (getretvalfunc) ? (*getretvalfunc)() : 0;
-			cp = long2str(tmp, i, sizeof(tmp));
+			snprintf2(tmp, sizeof(tmp), "%d",
+				(getretvalfunc) ? (*getretvalfunc)() : 0);
+			cp = tmp;
 			break;
 		case '$':
 			if (!getpidfunc) pid = getpid();
 			else pid = (*getpidfunc)();
-			cp = long2str(tmp, pid, sizeof(tmp));
+			snprintf2(tmp, sizeof(tmp), "%id", pid);
+			cp = tmp;
 			break;
 		case '!':
 			if (getlastpidfunc
-			&& (pid = (*getlastpidfunc)()) >= 0)
-				cp = long2str(tmp, pid, sizeof(tmp));
+			&& (pid = (*getlastpidfunc)()) >= 0) {
+				snprintf2(tmp, sizeof(tmp), "%id", pid);
+				cp = tmp;
+			}
 			break;
 		case '-':
 			if (getflagfunc) cp = (*getflagfunc)();
@@ -2660,14 +2557,11 @@ int s, len, vlen, mode, quoted;
 		demacroarg(cpp);
 #endif
 		for (i = 0; i < len; i++) fputc(arg[i], stderr);
-		fputs(": ", stderr);
-		if (vlen <= 0)
-			fputs("parameter null or not set", stderr);
-		else for (i = 0; (*cpp)[i]; i++) fputc((*cpp)[i], stderr);
+		fprintf2(stderr, ": %k",
+			(vlen > 0) ? *cpp : "parameter null or not set");
+		fputnl(stderr);
 		free(*cpp);
 		*cpp = NULL;
-		fputc('\n', stderr);
-		fflush(stderr);
 		if (exitfunc) (*exitfunc)();
 		return(-2);
 	}
@@ -2743,8 +2637,8 @@ int quoted;
 #endif	/* !MINIMUMSHELL */
 
 	c = '\0';
-	if ((cp = new));
-	else if (isidentchar(*top)) cp = getvar(top, len);
+	if ((cp = new)) /*EMPTY*/;
+	else if (isidentchar(*top)) cp = getenvvar(top, len);
 	else if (isdigit(*top)) {
 		if (len > 1) return(-1);
 		if (!argvar) {
@@ -2777,8 +2671,11 @@ int quoted;
 	if (cp && (nul == 0x80) && !*cp) cp = NULL;
 
 #ifndef	MINIMUMSHELL
-	if (mode == 'n')
-		cp = long2str(tmp, (cp) ? strlen(cp) : 0, sizeof(tmp));
+	if (mode == 'n') {
+		snprintf2(tmp, sizeof(tmp), "%d",
+			(cp) ? strlen(cp) : 0);
+		cp = tmp;
+	}
 	else if (nul == -1) {
 		new = removeword(cp, top + s, vlen, mode);
 		if (new) cp = new;
@@ -2817,7 +2714,7 @@ int quoted;
 	return(ptr);
 }
 
-#if	!MSDOS
+#if	defined (FD) && !defined (NOUID)
 uidtable *finduid(uid, name)
 uid_t uid;
 char *name;
@@ -2908,9 +2805,35 @@ char *name;
 	gidlist = b_realloc(gidlist, maxgid, gidtable);
 	gidlist[maxgid].gid = grp -> gr_gid;
 	gidlist[maxgid].name = strdup2(grp -> gr_name);
+	gidlist[maxgid].gr_mem = duplvar(grp -> gr_mem, -1);
+	gidlist[maxgid].ismem = 0;
 	return(&(gidlist[maxgid++]));
 }
 
+int isgroupmember(gid)
+gid_t gid;
+{
+	uidtable *up;
+	gidtable *gp;
+	int i;
+
+	if (!(gp = findgid(gid, NULL))) return(0);
+	if (!(gp -> ismem)) {
+		gp -> ismem++;
+		if (gp -> gr_mem && (up = finduid(geteuid(), NULL)))
+		for (i = 0; gp -> gr_mem[i]; i++) {
+			if (!strpathcmp(up -> name, gp -> gr_mem[i])) {
+				gp -> ismem++;
+				break;
+			}
+		}
+		freevar(gp -> gr_mem);
+		gp -> gr_mem = NULL;
+	}
+
+	return(gp -> ismem - 1);
+}
+
 # ifdef	DEBUG
 VOID freeidlist(VOID_A)
 {
@@ -2924,7 +2847,10 @@ VOID freeidlist(VOID_A)
 		free(uidlist);
 	}
 	if (gidlist) {
-		for (i = 0; i < maxgid; i++) free(gidlist[i].name);
+		for (i = 0; i < maxgid; i++) {
+			free(gidlist[i].name);
+			freevar(gidlist[i].gr_mem);
+		}
 		free(gidlist);
 	}
 	uidlist = NULL;
@@ -2932,7 +2858,7 @@ VOID freeidlist(VOID_A)
 	maxuid = maxgid = 0;
 }
 # endif
-#endif	/* !MSDOS */
+#endif	/* FD && !NOUID */
 
 static char *NEAR replacebackquote(buf, ptrp, bbuf, rest)
 char *buf;
@@ -2957,18 +2883,38 @@ int rest;
 
 char *gethomedir(VOID_A)
 {
-#if	!MSDOS
+#ifndef	NOUID
+# ifdef	FD
 	uidtable *up;
-#endif
+# else
+	struct passwd *pwd;
+# endif
+#endif	/* !NOUID */
 	char *cp;
 
 	if ((cp = getconstvar("HOME"))) return(cp);
-#if	!MSDOS
+#ifndef	NOUID
+# ifdef	FD
 	if ((up = finduid(getuid(), NULL))) return(up -> home);
-#endif
+# else	/* !FD */
+#  ifdef	DEBUG
+	_mtrace_file = "getpwuid(start)";
+	pwd = getpwuid(getuid());
+	if (_mtrace_file) _mtrace_file = NULL;
+	else {
+		_mtrace_file = "getpwuid(end)";
+		malloc(0);	/* dummy alloc */
+	}
+#  else
+	pwd = getpwuid(getuid());
+#  endif
+	if (pwd) return(pwd -> pw_dir);
+# endif	/* !FD */
+#endif	/* !NOUID */
 	return(NULL);
 }
 
+#ifndef	MINIMUMSHELL
 int evalhome(bufp, ptr, argp)
 char **bufp;
 int ptr;
@@ -2981,21 +2927,40 @@ char **argp;
 
 	len = ((cp = strdelim(top, 0))) ? (cp - top) : strlen(top);
 	if (!len) cp = gethomedir();
-#ifdef	FD
+# ifdef	FD
 	else if (len == sizeof("FD") - 1 && !strnpathcmp(top, "FD", len))
 		cp = progpath;
-#endif
+# endif
 	else {
-#if	!MSDOS
+# ifdef	NOUID
+		cp = NULL;
+# else	/* !NOUID */
+#  ifdef	FD
 		uidtable *up;
 
 		cp = strndup2(top, len);
 		up = finduid(0, cp);
 		free(cp);
-		if (up) cp = up -> home;
-		else
-#endif
-		cp = NULL;
+		cp = (up) ? up -> home : NULL;
+#  else	/* !FD */
+		struct passwd *pwd;
+
+		cp = strndup2(top, len);
+#   ifdef	DEBUG
+		_mtrace_file = "getpwnam(start)";
+		pwd = getpwnam(cp);
+		if (_mtrace_file) _mtrace_file = NULL;
+		else {
+			_mtrace_file = "getpwnam(end)";
+			malloc(0);	/* dummy alloc */
+		}
+#   else
+		pwd = getpwnam(cp);
+#   endif
+		free(cp);
+		cp = (pwd) ? pwd -> pw_dir : NULL;
+#  endif	/* !FD */
+# endif	/* !NOUID */
 	}
 
 	if (!cp) {
@@ -3012,6 +2977,7 @@ char **argp;
 	*argp += len;
 	return(ptr);
 }
+#endif	/* !MINIMUMSHELL */
 
 char *evalarg(arg, stripq, backq, qed)
 char *arg;
@@ -3086,7 +3052,11 @@ int stripq, backq, qed;
 			tmpq = (q) ? q : qed;
 			if (!cp[1]) buf[i++] = *cp;
 #ifdef	FAKEMETA
+# ifdef	MINIMUMSHELL
+			else if (cp[1] == '$') cp++;
+# else
 			else if (cp[1] == '$' || cp[1] == '~') cp++;
+# endif
 #endif
 			else if ((i = evalvar(&buf, i, &cp, tmpq)) < 0) {
 				if (bbuf) free(bbuf);
@@ -3097,9 +3067,10 @@ int stripq, backq, qed;
 		}
 		else if (pc == PC_META) {
 			cp++;
-			if (*cp == '$');
-			else if (backq && *cp == '`');
-			else if (stripq && (*cp == '\'' || *cp == '"'));
+			if (*cp == '$') /*EMPTY*/;
+			else if (backq && *cp == '`') /*EMPTY*/;
+			else if (stripq && (*cp == '\'' || *cp == '"'))
+				/*EMPTY*/;
 			else buf[i++] = PMETA;
 			buf[i++] = *cp;
 		}
@@ -3107,9 +3078,11 @@ int stripq, backq, qed;
 			if (*cp == '`') j = 0;
 			else if (!stripq) buf[i++] = *cp;
 		}
-		else if (pc != PC_NORMAL);
+		else if (pc != PC_NORMAL) /*EMPTY*/;
+#ifndef	MINIMUMSHELL
 		else if (*cp == '~' && (!prev || prev == ':' || prev == '='))
 			i = evalhome(&buf, i, &cp);
+#endif
 		else buf[i++] = *cp;
 	}
 #ifndef	BASHSTYLE
@@ -3136,7 +3109,7 @@ char ***argvp, *ifs;
 			pc = parsechar(&((*argvp)[n][i]), -1,
 				'\0', 0, &quote, NULL);
 			if (pc == PC_WORD || pc == PC_META) i++;
-			else if (pc != PC_NORMAL);
+			else if (pc != PC_NORMAL) /*EMPTY*/;
 			else if (strchr(ifs, (*argvp)[n][i])) {
 				for (j = i + 1; (*argvp)[n][j]; j++)
 					if (!strchr(ifs, (*argvp)[n][j]))
@@ -3196,8 +3169,8 @@ int stripq;
 				&& !strchr(DQ_METACHAR, (*argvp)[n][i]))
 					cp[j++] = PMETA;
 			}
-			else if (pc != PC_NORMAL);
-			else if (!strchr("?*[", (*argvp)[n][i]));
+			else if (pc != PC_NORMAL) /*EMPTY*/;
+			else if (!strchr("?*[", (*argvp)[n][i])) /*EMPTY*/;
 			else if ((wild = evalwild((*argvp)[n]))) {
 				w = countvar(wild);
 				if (w > 1) {
@@ -3320,7 +3293,7 @@ int uniqdelim, evalq;
 #endif
 			if (evalq) continue;
 		}
-		else if (pc != PC_NORMAL);
+		else if (pc != PC_NORMAL) /*EMPTY*/;
 		else if (uniqdelim && cp[i] == _SC_ && c == _SC_) continue;
 		tmp[j++] = cp[i];
 	}
