@@ -4,12 +4,17 @@
  *	XSH command for POSIX
  */
 
+#include <fcntl.h>
+#ifdef	FD
+#include "fd.h"
+#else	/* !FD */
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <fcntl.h>
 #include "machine.h"
+#include "printf.h"
 #include "kctype.h"
+#include "pathname.h"
 
 #ifndef	NOUNISTDH
 #include <unistd.h>
@@ -18,20 +23,16 @@
 #ifndef	NOSTDLIBH
 #include <stdlib.h>
 #endif
+#endif	/* !FD */
 
 #if	MSDOS
 #include <io.h>
 #include "unixemu.h"
-#define	Xexit		exit
-#define	DEFPATH		":"
 #else	/* !MSDOS */
 #include <sys/param.h>
 #include <sys/ioctl.h>
-#define	Xexit		_exit
-#define	DEFPATH		":/bin:/usr/bin"
 #endif	/* !MSDOS */
 
-#include "pathname.h"
 #include "system.h"
 
 #if	!defined (MINIMUMSHELL) \
@@ -42,37 +43,38 @@
 extern int Xstat __P_((char *, struct stat *));
 extern int Xlstat __P_((char *, struct stat *));
 extern int Xaccess __P_((char *, int));
-extern int kanjifputs __P_((char *, FILE *));
 #else	/* !FD */
 extern int ttyio;
 extern FILE *ttyout;
-#define	Xstat(f, s)	(stat(f, s) ? -1 : 0)
 # if	MSDOS
-# define	Xlstat(f, s)	(stat(f, s) ? -1 : 0)
+extern int Xstat __P_((char *, struct stat *));
+# define	Xlstat		Xstat
 # else
+# define	Xstat		stat
 # define	Xlstat		lstat
 # endif
 #define	Xaccess(p, m)	(access(p, m) ? -1 : 0)
-#define	kanjifputs	fputs
 #endif	/* !FD */
 
 #ifndef	EPERM
 #define	EPERM		EACCES
 #endif
 
+extern VOID error __P_((char *));
 extern char *malloc2 __P_((ALLOC_T));
 extern char *realloc2 __P_((VOID_P, ALLOC_T));
 extern char *strdup2 __P_((char *));
 extern char *strndup2 __P_((char *, int));
+extern char *strchr2 __P_((char *, int));
 extern char *strncpy2 __P_((char *, char *, int));
-extern int setenv2 __P_((char *, char *));
+extern int setenv2 __P_((char *, char *, int));
 extern time_t time2 __P_((VOID_A));
 
-#if	!MSDOS && !defined (NOJOB)
+#ifndef	NOJOB
 static char *NEAR headstree __P_((syntaxtree *));
 #endif
 #ifndef	NOALIAS
-static aliastable *NEAR duplalias __P_((aliastable *));
+static shaliastable *NEAR duplalias __P_((shaliastable *));
 static int cmpalias __P_((CONST VOID_P, CONST VOID_P));
 #endif
 static int NEAR _evalexpression __P_((char *, int, long *));
@@ -105,7 +107,7 @@ typedef struct _mailpath_t {
 } mailpath_t;
 #endif
 
-#if	!MSDOS && !defined (NOJOB)
+#ifndef	NOJOB
 int gettermio __P_((p_id_t));
 VOID dispjob __P_((int, FILE *));
 int searchjob __P_((p_id_t, int *));
@@ -114,17 +116,17 @@ int stackjob __P_((p_id_t, int, syntaxtree *));
 int stoppedjob __P_((p_id_t));
 VOID killjob __P_((VOID_A));
 VOID checkjob __P_((int));
-#endif	/* !MSDOS && !NOJOB */
+#endif	/* !NOJOB */
 char *evalposixsubst __P_((char *, int *));
 #if	!MSDOS
 VOID replacemailpath __P_((char *, int));
 VOID checkmail __P_((int));
 #endif
 #ifndef	NOALIAS
-VOID freealias __P_((aliastable *));
+VOID freealias __P_((shaliastable *));
 int checkalias __P_((syntaxtree *, char *, int, int));
 #endif
-#if	!MSDOS && !defined (NOJOB)
+#ifndef	NOJOB
 int posixjobs __P_((syntaxtree *));
 int posixfg __P_((syntaxtree *));
 int posixbg __P_((syntaxtree *));
@@ -140,7 +142,7 @@ int posixcommand __P_((syntaxtree *));
 int posixgetopts __P_((syntaxtree *));
 #endif
 
-#if	!MSDOS && !defined (NOJOB)
+#ifndef	NOJOB
 jobtable *joblist = NULL;
 int maxjobs = 0;
 #endif
@@ -148,7 +150,7 @@ int maxjobs = 0;
 int mailcheck = 0;
 #endif
 #ifndef	NOALIAS
-aliastable *shellalias = NULL;
+shaliastable *shellalias = NULL;
 #endif
 #ifndef	NOPOSIXUTIL
 int posixoptind = 0;
@@ -160,14 +162,15 @@ static int maxmailpath = 0;
 #endif
 
 
-#if	!MSDOS && !defined (NOJOB)
+#ifndef	NOJOB
 int gettermio(pgrp)
 p_id_t pgrp;
 {
 	int ret;
 	sigmask_t mask, omask;
 
-	if (ttypgrp < 0L || pgrp < 0L || pgrp == ttypgrp) return(0);
+	if (ttypgrp < (p_id_t)0 || pgrp < (p_id_t)0 || pgrp == ttypgrp)
+		return(0);
 	else if (!jobok) {
 		ttypgrp = pgrp;
 		return(0);
@@ -189,14 +192,8 @@ p_id_t pgrp;
 	Xsigblock(omask, mask);
 
 #ifdef	JOBVERBOSE
-	fputs("gettermio: ", ttyout);
-	fputlong(mypid, ttyout);
-	fputs(": ", ttyout);
-	fputlong(ttypgrp, ttyout);
-	fputs(" -> ", ttyout);
-	fputlong(pgrp, ttyout);
-	fputc('\n', ttyout);
-	fflush(ttyout);
+	fprintf2(ttyout, "gettermio: %id: %id -> %id", mypid, ttypgrp, pgrp);
+	fputnl(ttyout);
 #endif	/* JOBVERBOSE */
 	if ((ret = settcpgrp(ttyio, pgrp)) >= 0) ttypgrp = pgrp;
 	Xsigsetmask(omask);
@@ -229,20 +226,14 @@ FILE *fp;
 	int i, sig;
 
 	if (n < 0 || n >= maxjobs || !(joblist[n].pids)) return;
-	fputc('[', fp);
-	fputlong(n + 1, fp);
-	fputc(']', fp);
-	if (n == lastjob) fputc('+', fp);
-	else if (n == prevjob) fputc('-', fp);
-	else fputc(' ', fp);
-	fputc(' ', fp);
 	i = joblist[n].npipe;
-	fputlong(joblist[n].pids[i], fp);
-	fputc(' ', fp);
+	fprintf2(fp, "[%d]%c %id ",
+		n + 1, (n == lastjob) ? '+' : ((n == prevjob) ? '-' : ' '),
+		joblist[n].pids[i]);
 	sig = joblist[n].stats[i];
 
-	if (!sig) fputstr("Running", 28, fp);
-	else if (sig < 0) fputstr("Done", 28, fp);
+	if (sig <= 0)
+		fprintf2(fp, "%-28.28s", (sig) ? "Done" : "Running");
 	else {
 		if (sig >= 128) sig -= 128;
 		dispsignal(sig, 28, fp);
@@ -253,8 +244,7 @@ FILE *fp;
 			joblist[n].trp -> type = OP_NONE;
 		printstree(joblist[n].trp, -1, fp);
 	}
-	fputc('\n', fp);
-	fflush(fp);
+	fputnl(fp);
 }
 
 int searchjob(pid, np)
@@ -337,7 +327,7 @@ syntaxtree *trp;
 				break;
 			}
 		}
-		if (i < maxjobs);
+		if (i < maxjobs) /*EMPTY*/;
 		else if (n >= 0) i = n;
 		else {
 			joblist = (jobtable *)realloc2(joblist,
@@ -383,19 +373,10 @@ syntaxtree *trp;
 	}
 
 #ifdef	JOBVERBOSE
-	fputs("stackjob: ", ttyout);
-	fputlong(mypid, ttyout);
-	fputs(": ", ttyout);
-	fputlong(pid, ttyout);
-	fputs(", ", ttyout);
-	fputlong(i, ttyout);
-	fputc(':', ttyout);
-	for (j = 0; j <= joblist[i].npipe; j++) {
-		fputlong(joblist[i].pids[j], ttyout);
-		fputc(' ', ttyout);
-	}
-	fputc('\n', ttyout);
-	fflush(ttyout);
+	fprintf2(ttyout, "stackjob: %id: %id, %d:", mypid, pid, i);
+	for (j = 0; j <= joblist[i].npipe; j++)
+		fprintf2(ttyout, "%id ", joblist[i].pids[j]);
+	fputnl(ttyout);
 #endif	/* JOBVERBOSE */
 	return(i);
 }
@@ -458,18 +439,18 @@ VOID killjob(VOID_A)
 		if (j > 0) Xkillpg(joblist[i].pids[0], SIGCONT);
 	}
 }
-#endif	/* !MSDOS && !NOJOB */
+#endif	/* !NOJOB */
 
 #ifndef	NOALIAS
-static aliastable *NEAR duplalias(alias)
-aliastable *alias;
+static shaliastable *NEAR duplalias(alias)
+shaliastable *alias;
 {
-	aliastable *dupl;
+	shaliastable *dupl;
 	int i, n;
 
 	if (!alias) n = 0;
 	else for (n = 0; alias[n].ident; n++);
-	dupl = (aliastable *)malloc2((n + 1) * sizeof(aliastable));
+	dupl = (shaliastable *)malloc2((n + 1) * sizeof(shaliastable));
 	for (i = 0; i < n; i++) {
 		dupl[i].ident = strdup2(alias[i].ident);
 		dupl[i].comm = strdup2(alias[i].comm);
@@ -479,7 +460,7 @@ aliastable *alias;
 }
 
 VOID freealias(alias)
-aliastable *alias;
+shaliastable *alias;
 {
 	int i;
 
@@ -496,10 +477,10 @@ static int cmpalias(vp1, vp2)
 CONST VOID_P vp1;
 CONST VOID_P vp2;
 {
-	aliastable *ap1, *ap2;
+	shaliastable *ap1, *ap2;
 
-	ap1 = (aliastable *)vp1;
-	ap2 = (aliastable *)vp2;
+	ap1 = (shaliastable *)vp1;
+	ap2 = (shaliastable *)vp2;
 	return(strpathcmp2(ap1 -> ident, ap2 -> ident));
 }
 
@@ -519,7 +500,7 @@ int len, delim;
 		return(-1);
 	for (i = 0; shellalias[i].ident; i++)
 		if (!(shellalias[i].flags & AL_USED)
-		&& !strnpathcmp(ident, shellalias[i].ident, len)
+		&& !strncommcmp(ident, shellalias[i].ident, len)
 		&& !(shellalias[i].ident[len])) break;
 	if (!(shellalias[i].ident)) return(-1);
 
@@ -535,58 +516,64 @@ int ptr;
 long *resultp;
 {
 	long n;
+	int i, c, base;
 
 	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
 	if (!s[ptr]) return(-1);
 
-	if (s[ptr] == '+') ptr = _evalexpression(s, ptr + 1, resultp);
+	if (s[ptr] == '+') {
+		if ((ptr = _evalexpression(s, ptr + 1, resultp)) < 0)
+			return(-1);
+	}
 	else if (s[ptr] == '-') {
-		ptr = _evalexpression(s, ptr + 1, resultp);
+		if ((ptr = _evalexpression(s, ptr + 1, resultp)) < 0)
+			return(-1);
 		*resultp = -(*resultp);
 	}
 	else if (s[ptr] == '~') {
-		ptr = _evalexpression(s, ptr + 1, resultp);
+		if ((ptr = _evalexpression(s, ptr + 1, resultp)) < 0)
+			return(-1);
 		*resultp = ~(*resultp);
 	}
 	else if (s[ptr] == '!' && s[ptr + 1] != '=') {
-		ptr = _evalexpression(s, ptr + 1, resultp);
+		if ((ptr = _evalexpression(s, ptr + 1, resultp)) < 0)
+			return(-1);
 		*resultp = (*resultp) ? 0L : 1L;
 	}
 	else if (s[ptr] == '(') {
-		ptr = evalexpression(s, ptr + 1, resultp, 9);
-		if (ptr >= 0) {
-			while (strchr(IFS_SET, s[ptr])) ptr++;
-			if (s[ptr] != ')') ptr = -1;
-			else ptr++;
-		}
+		if ((ptr = evalexpression(s, ptr + 1, resultp, 9)) < 0)
+			return(-1);
+		while (strchr(IFS_SET, s[ptr])) ptr++;
+		if (s[ptr++] != ')') return(-1);
 	}
-	else if (s[ptr] == '0') {
+	else if (isdigit2(s[ptr])) {
+		if (s[ptr] != '0') base = 10;
+		else if (toupper2(s[++ptr]) != 'X') base = 8;
+		else {
+			base = 16;
+			ptr++;
+		}
+
 		n = 0;
-		if (toupper2(s[++ptr]) == 'X') for (ptr++; s[ptr]; ptr++) {
-			if (isdigit(s[ptr])) n = n * 16L + s[ptr] - '0';
-			else if (s[ptr] >= 'a' && s[ptr] <= 'f')
-				n = n * 16L + s[ptr] - 'a' + 10;
-			else if (s[ptr] >= 'A' && s[ptr] <= 'F')
-				n = n * 16L + s[ptr] - 'A' + 10;
-			else break;
-			if (n < 0L) return(-1);
-		}
-		else for (; s[ptr]; ptr++) {
-			if (s[ptr] < '0' || s[ptr] > '7') break;
-			n = n * 8L + s[ptr] - '0';
-			if (n < 0L) return(-1);
+		for (i = ptr; s[i]; i++) {
+			c = s[i];
+			if (isdigit2(c)) c -= '0';
+			else if (islower2(c)) c -= 'a' - 10;
+			else if (isupper2(c)) c -= 'A' - 10;
+			else c = -1;
+			if (c < 0 || c >= base) break;
+
+			if (n > MAXTYPE(long) / base
+			|| (n == MAXTYPE(long) / base
+			&& c > MAXTYPE(long) % base))
+				return(-1);
+			n = n * base + c;
 		}
+		if (i <= ptr) return(-1);
+		ptr = i;
 		*resultp = n;
 	}
-	else if (isdigit(s[ptr])) {
-		n = (long)(s[ptr++] - '0');
-		while (isdigit(s[ptr])) {
-			n = n * 10L + (s[ptr++] - '0');
-			if (n < 0L) return(-1);
-		}
-		*resultp = n;
-	}
-	else ptr = -1;
+	else return(-1);
 
 	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
 	return(ptr);
@@ -740,7 +727,7 @@ char *evalposixsubst(s, ptrp)
 char *s;
 int *ptrp;
 {
-	char *cp, buf[MAXLONGWIDTH + 1];
+	char *cp;
 	long n;
 	int i, len;
 
@@ -784,14 +771,7 @@ int *ptrp;
 		}
 
 		if (*ptrp < 0) cp = NULL;
-		else if (n >= 0L)
-			cp = strdup2(long2str(buf, n, sizeof(buf)));
-		else {
-			long2str(buf, -n, sizeof(buf));
-			cp = malloc2(strlen(buf) + 2);
-			cp[0] = '-';
-			strcpy(&(cp[1]), buf);
-		}
+		else if (asprintf2(&cp, "%ld", n) < 0) error("malloc()");
 	}
 	free(s);
 
@@ -899,7 +879,7 @@ int reset;
 }
 #endif	/* !MSDOS */
 
-#if	!MSDOS && !defined (NOJOB)
+#ifndef	NOJOB
 /*ARGSUSED*/
 int posixjobs(trp)
 syntaxtree *trp;
@@ -950,12 +930,8 @@ syntaxtree *trp;
 		prepareexit(-1);
 		Xexit(RET_FATALERR);
 	}
-	fputc('[', stderr);
-	fputlong(i + 1, stderr);
-	fputs("] ", stderr);
-	fputlong(joblist[i].pids[n], stderr);
-	fputc('\n', stderr);
-	fflush(stderr);
+	fprintf2(stderr, "[%d] %id", i + 1, joblist[i].pids[n]);
+	fputnl(stderr);
 	if (joblist[i].tty) tioctl(ttyio, REQSETP, joblist[i].tty);
 # ifdef	USESGTTY
 	if (joblist[i].ttyflag) ioctl(ttyio, TIOCLSET, joblist[i].ttyflag);
@@ -1002,13 +978,13 @@ syntaxtree *trp;
 	lastjob = i;
 	return(RET_SUCCESS);
 }
-#endif	/* !MSDOS && !NOJOB */
+#endif	/* !NOJOB */
 
 #ifndef	NOALIAS
 int posixalias(trp)
 syntaxtree *trp;
 {
-	aliastable *alias;
+	shaliastable *alias;
 	char **argv;
 	int i, n, len, set, ret;
 
@@ -1016,16 +992,13 @@ syntaxtree *trp;
 	if ((trp -> comm) -> argc <= 1) {
 		alias = duplalias(shellalias);
 		for (i = 0; alias[i].ident; i++);
-		if (i > 1) qsort(alias, i, sizeof(aliastable), cmpalias);
+		if (i > 1) qsort(alias, i, sizeof(shaliastable), cmpalias);
 		for (i = 0; alias[i].ident; i++) {
-			fputs("alias ", stdout);
-			kanjifputs(alias[i].ident, stdout);
-			fputs("='", stdout);
-			kanjifputs(alias[i].comm, stdout);
-			fputs("'\n", stdout);
+			fprintf2(stdout, "alias %k='%k'",
+				alias[i].ident, alias[i].comm);
+			fputnl(stdout);
 		}
 		freealias(alias);
-		fflush(stdout);
 		return(RET_SUCCESS);
 	}
 
@@ -1037,7 +1010,7 @@ syntaxtree *trp;
 			break;
 		}
 		for (i = 0; shellalias[i].ident; i++)
-			if (!strnpathcmp(shellalias[i].ident, argv[n], len)
+			if (!strncommcmp(shellalias[i].ident, argv[n], len)
 			&& !(shellalias[i].ident[len]))
 				break;
 
@@ -1050,20 +1023,17 @@ syntaxtree *trp;
 				ret = RET_FAIL;
 				ERRBREAK;
 			}
-			fputs("alias ", stdout);
-			kanjifputs(shellalias[i].ident, stdout);
-			fputs("='", stdout);
-			kanjifputs(shellalias[i].comm, stdout);
-			fputs("'\n", stdout);
-			fflush(stdout);
+			fprintf2(stdout, "alias %k='%k'",
+				shellalias[i].ident, shellalias[i].comm);
+			fputnl(stdout);
 		}
 		else if (shellalias[i].ident) {
 			free(shellalias[i].comm);
 			shellalias[i].comm = strdup2(&(argv[n][++len]));
 		}
 		else {
-			shellalias = (aliastable *)realloc2(shellalias,
-				(i + 2) * sizeof(aliastable));
+			shellalias = (shaliastable *)realloc2(shellalias,
+				(i + 2) * sizeof(shaliastable));
 			shellalias[i].ident = strndup2(argv[n], len);
 			shellalias[i].comm = strdup2(&(argv[n][++len]));
 			shellalias[++i].ident = NULL;
@@ -1089,7 +1059,7 @@ syntaxtree *trp;
 				if (!regexp_exec(re, shellalias[i].ident, 0))
 					continue;
 			}
-			else if (strpathcmp(shellalias[i].ident, argv[n]))
+			else if (strcommcmp(shellalias[i].ident, argv[n]))
 				continue;
 			c++;
 			free(shellalias[i].ident);
@@ -1123,8 +1093,8 @@ syntaxtree *trp;
 
 	if ((trp -> comm) -> argc <= 1) {
 		fputs("usage: kill [ -sig ] pid ...\n", stderr);
-		fputs("for a list of signals: kill -l\n", stderr);
-		fflush(stderr);
+		fputs("for a list of signals: kill -l", stderr);
+		fputnl(stderr);
 		return(RET_SYNTAXERR);
 	}
 #ifdef	SIGTERM
@@ -1140,10 +1110,11 @@ syntaxtree *trp;
 				for (i = 0; signallist[i].sig >= 0; i++)
 					if (sig == signallist[i].sig) break;
 				if (signallist[i].sig < 0) continue;
-				fputs(signallist[i].ident, stdout);
-				fputc((++n % 16) ? ' ' : '\n', stdout);
+				fprintf2(stdout, "%s%c",
+					signallist[i].ident,
+					(++n % 16) ? ' ' : '\n');
 			}
-			if (n % 16) fputc('\n', stdout);
+			if (n % 16) fputnl(stdout);
 			fflush(stdout);
 			return(RET_SUCCESS);
 		}
@@ -1164,12 +1135,12 @@ syntaxtree *trp;
 		i = 2;
 	}
 
-#if	!MSDOS && !defined (NOJOB)
+#ifndef	NOJOB
 	checkjob(0);
 #endif
 	for (; i < (trp -> comm) -> argc; i++) {
 		s = (trp -> comm) -> argv[i];
-#if	!MSDOS && !defined (NOJOB)
+#ifndef	NOJOB
 		if (*s == '%') {
 			if ((n = getjob(s)) < 0) {
 				execerror(s, ER_NOSUCHJOB, 0);
@@ -1178,11 +1149,11 @@ syntaxtree *trp;
 			n = Xkillpg(joblist[n].pids[0], sig);
 		}
 		else
-#endif	/* !MSDOS && !NOJOB */
-		if ((pid = isnumeric(s)) < 0L) {
+#endif	/* !NOJOB */
+		if ((pid = isnumeric(s)) < (p_id_t)0) {
 			fputs("usage: kill [ -sig ] pid ...\n", stderr);
-			fputs("for a list of signals: kill -l\n", stderr);
-			fflush(stderr);
+			fputs("for a list of signals: kill -l", stderr);
+			fputnl(stderr);
 			return(RET_SYNTAXERR);
 		}
 #if	MSDOS
@@ -1436,8 +1407,8 @@ syntaxtree *trp;
 	if (argc <= 0) return(RET_FAIL);
 	if (argv[0][0] == '[' && !argv[0][1]
 	&& (--argc <= 0 || argv[argc][0] != ']' || argv[argc][1])) {
-		fputs("] missing\n", stderr);
-		fflush(stderr);
+		fputs("] missing", stderr);
+		fputnl(stderr);
 		return(RET_NOTICE);
 	}
 	if (argc <= 1) return(RET_FAIL);
@@ -1447,18 +1418,17 @@ syntaxtree *trp;
 	if (ret < 0) {
 		switch (ret) {
 			case -1:
-				fputs("argument expected\n", stderr);
+				fputs("argument expected", stderr);
 				break;
 			case -2:
-				fputs("unknown operator ", stderr);
-				fputs(argv[ptr], stderr);
-				fputc('\n', stderr);
+				fprintf2(stderr, "unknown operator %k",
+					argv[ptr]);
 				break;
 			default:
-				fputs("syntax error\n", stderr);
+				fputs("syntax error", stderr);
 				break;
 		}
-		fflush(stderr);
+		fputnl(stderr);
 		return(RET_NOTICE);
 	}
 	return(ret);
@@ -1493,7 +1463,6 @@ syntaxtree *trp;
 # endif
 			}
 		}
-		fflush(stdout);
 		return(ret);
 	}
 	else if (flag == 'v') {
@@ -1520,14 +1489,13 @@ syntaxtree *trp;
 			}
 
 			if (cp) {
-				fputs(cp, stdout);
-				fputc('\n', stdout);
+				kanjifputs(cp, stdout);
+				fputnl(stdout);
 # ifdef	BASHSTYLE
 				ret = RET_SUCCESS;
 # endif
 			}
 		}
-		fflush(stdout);
 		return(ret);
 	}
 
@@ -1537,14 +1505,10 @@ syntaxtree *trp;
 	type = checktype(argv[n], &id, 0, 0);
 #endif
 	if (verboseexec) {
-		fputs("+ ", stderr);
-		kanjifputs(argv[n], stderr);
-		for (i = n + 1; i < argc; i++) {
-			fputc(' ', stderr);
-			kanjifputs(argv[i], stderr);
-		}
-		fputc('\n', stderr);
-		fflush(stderr);
+		fprintf2(stderr, "+ %k", argv[n]);
+		for (i = n + 1; i < argc; i++)
+			fprintf2(stderr, " %k", argv[i]);
+		fputnl(stderr);
 	}
 
 	(trp -> comm) -> argc -= n;
@@ -1557,14 +1521,14 @@ syntaxtree *trp;
 #endif
 	else {
 		path = strdup2(getconstvar("PATH"));
-		setenv2("PATH", DEFPATH);
+		setenv2("PATH", DEFPATH, 1);
 #if	MSDOS
 		ret = exec_simplecom(trp, type, id);
 #else
 		ret = exec_simplecom(trp, type, id, 0);
 #endif
-		setenv2("PATH", path);
-		free(path);
+		setenv2("PATH", path, 1);
+		if (path) free(path);
 	}
 	(trp -> comm) -> argc = argc;
 	(trp -> comm) -> argv = argv;
@@ -1634,7 +1598,7 @@ syntaxtree *trp;
 	if (ret != RET_SUCCESS) n = '?';
 	else {
 		n = argv[posixoptind][ptr++];
-		if (!islower(n & 0xff) || !(cp = strchr(optstr, n))) {
+		if (n == ':' || ismsb(n) || !(cp = strchr2(optstr, n))) {
 			buf[0] = n;
 			buf[1] = '\0';
 			n = '?';
@@ -1662,15 +1626,16 @@ syntaxtree *trp;
 		}
 	}
 
-	if (posixoptarg) setenv2("OPTARG", posixoptarg);
+	if (posixoptarg) setenv2("OPTARG", posixoptarg, 0);
 	else if (ret == RET_SUCCESS) unset("OPTARG", sizeof("OPTARG") - 1);
 
 	buf[0] = n;
 	buf[1] = '\0';
-	setenv2(name, buf);
+	setenv2(name, buf, 0);
 
 	n = posixoptind;
-	setenv2("OPTIND", long2str(buf, n, sizeof(buf)));
+	snprintf2(buf, sizeof(buf), "%d", n);
+	setenv2("OPTIND", buf, 0);
 	posixoptind = n;
 
 	return(ret);
