@@ -5,9 +5,7 @@
  */
 
 #include "fd.h"
-#include "term.h"
 #include "func.h"
-#include "kctype.h"
 #include "kanji.h"
 
 #if	MSDOS && !defined (_NOUSELFN)
@@ -44,7 +42,7 @@ extern int internal_status;
 #endif
 
 static int NEAR checksc __P_((char *, int, char *));
-#if	!MSDOS && !defined (_NOKANJICONV)
+#ifndef	_NOKANJICONV
 static int NEAR extconv __P_((char **, int, int, ALLOC_T *, int));
 #endif
 static int NEAR isneedargs __P_((char *, int, int *));
@@ -52,7 +50,7 @@ static int NEAR setarg __P_((char **, int, ALLOC_T *, char *, char *, int));
 static int NEAR flag2str __P_((char *, int, int));
 static int NEAR skipquote __P_((char *, int *));
 #if	!defined (MACROMETA) || !defined (_NOEXTRAMACRO)
-static char *NEAR restorearg __P_((char *));
+static char *NEAR _restorearg __P_((char *));
 static char *NEAR _demacroarg __P_((char *));
 #endif
 #ifdef	_NOEXTRAMACRO
@@ -95,10 +93,10 @@ char *buf;
 int ptr;
 char *arg;
 {
-	if (!arg || !*arg);
-	else if (*arg == _SC_ || isdotdir(arg));
+	if (!arg || !*arg) /*EMPTY*/;
+	else if (*arg == _SC_ || isdotdir(arg)) /*EMPTY*/;
 #if	MSDOS
-	else if (_dospath(arg));
+	else if (_dospath(arg)) /*EMPTY*/;
 #endif
 	else return(ptr);
 
@@ -111,22 +109,20 @@ char *arg;
 	return(ptr - 2);
 }
 
-#if	!MSDOS && !defined (_NOKANJICONV)
+#ifndef	_NOKANJICONV
 static int NEAR extconv(bufp, ptr, eol, sizep, code)
 char **bufp;
 int ptr, eol;
 ALLOC_T *sizep;
 int code;
 {
-# ifndef	_NOKANJIFCONV
-	char rpath[MAXPATHLEN];
-
-# endif
 	char *cp;
 	int len;
 
 # ifndef	_NOKANJIFCONV
 	if (code < 0) {
+		char rpath[MAXPATHLEN];
+
 		cp = _evalpath(&((*bufp)[ptr]), &((*bufp)[eol]), 0, 0);
 		realpath2(cp, rpath, 1);
 		free(cp);
@@ -134,16 +130,15 @@ int code;
 	}
 # endif
 	(*bufp)[eol] = '\0';
-	len = (eol - ptr) * 3 + 3;
-	cp = malloc2(len + 1);
-	len = kanjiconv(cp, &((*bufp)[ptr]), len, DEFCODE, code, L_FNAME);
+	cp = newkanjiconv(&((*bufp)[ptr]), DEFCODE, code, L_FNAME);
+	len = strlen(cp);
 
 	*bufp = c_realloc(*bufp, ptr + len + 1, sizep);
 	memcpy(&((*bufp)[ptr]), cp, len);
 	free(cp);
 	return(ptr + len);
 }
-#endif	/* !MSDOS && !_NOKANJICONV */
+#endif	/* !_NOKANJICONV */
 
 static int NEAR isneedargs(s, n, flagsp)
 char *s;
@@ -206,7 +201,7 @@ int flags;
 	else {
 		strcatdelim2(path, dir, arg);
 		arg = path;
-#if	MSDOS && !defined (_NOARCHIVE)
+#if	defined (BSPATHDELIM) && !defined (_NOARCHIVE)
 		if (flags & F_ISARCH) for (len = 0; arg[len]; len++) {
 			if (iskanji1(arg, len)) len++;
 			if (arg[len] == _SC_) arg[len] = '/';
@@ -215,8 +210,9 @@ int flags;
 	}
 
 	if (arg != path) arg = convput(conv, arg, 1, 0, NULL, NULL);
-#if	!MSDOS && !defined (_NOKANJIFCONV)
-	else arg = kanjiconv2(conv, arg, MAXPATHLEN - 1, DEFCODE, fnamekcode);
+#ifndef	_NOKANJIFCONV
+	else arg = kanjiconv2(conv, arg, MAXPATHLEN - 1,
+		DEFCODE, fnamekcode, L_FNAME);
 #endif
 	optr = ptr;
 	ptr = checksc(*bufp, ptr, arg);
@@ -273,12 +269,13 @@ int *ptrp;
 }
 
 #if	!defined (MACROMETA) || !defined (_NOEXTRAMACRO)
-static char *NEAR restorearg(s)
+static char *NEAR _restorearg(s)
 char *s;
 {
 	char *buf;
 	int i, j, m, flags;
 
+	if (!s) return(NULL);
 	i = strlen(s);
 # ifdef	MACROMETA
 	buf = malloc2(i * 3 + 1);	/* MACROMETA+flags -> %XSTA (x2.5) */
@@ -313,10 +310,10 @@ char *s;
 static char *NEAR _demacroarg(s)
 char *s;
 {
-	char *buf;
+	char *cp;
 
-	if ((buf = restorearg(s)) != s) free(s);
-	return(buf);
+	if ((cp = _restorearg(s)) != s && s) free(s);
+	return(cp);
 }
 #endif	/* !MACROMETA || !_NOEXTRAMACRO */
 
@@ -440,7 +437,7 @@ int needmark;
 
 # if	MSDOS && !defined (_NOUSELFN)
 	org = arg;
-	*alias = '\0';
+	alias[0] = '\0';
 # endif
 	arg = convput(conv, arg, 1, 0, NULL, NULL);
 # ifdef	FAKEUNINIT
@@ -469,7 +466,7 @@ int needmark;
 			ins = arg;
 # if	MSDOS && !defined (_NOUSELFN)
 			if (flags & F_TOSFN) {
-				if (!*alias) {
+				if (!alias[0]) {
 					if (shortname(org, alias) == alias)
 						org = alias;
 					else strcpy(alias, org);
@@ -522,7 +519,7 @@ char *command, *arg;
 macrostat *stp;
 int ignorelist;
 {
-#if	!MSDOS && !defined (_NOKANJICONV)
+#ifndef	_NOKANJICONV
 	int cnvcode = NOCNV;
 	int tmpcode, cnvptr;
 #endif
@@ -538,7 +535,7 @@ int ignorelist;
 	}
 	stp -> addopt = -1;
 	stp -> needmark = stp -> needburst = 0;
-#if	!MSDOS && !defined (_NOKANJICONV)
+#ifndef	_NOKANJICONV
 	cnvptr = 0;
 #endif
 
@@ -547,7 +544,7 @@ int ignorelist;
 		line = c_realloc(line, j + 1, &size);
 
 		c = '\0';
-		if ((pc = skipquote(command, &i)) == 1);
+		if ((pc = skipquote(command, &i)) == 1) /*EMPTY*/;
 		else if (pc) line[j++] = command[i - 1];
 		else if (command[i] == '%') c = command[++i];
 		else if (flags & (F_NOEXT | F_TOSFN)) c = command[i];
@@ -631,7 +628,7 @@ int ignorelist;
 			case 'K':
 				flags |= F_NOCONFIRM;
 				break;
-#if	!MSDOS && !defined (_NOKANJICONV)
+#ifndef	_NOKANJICONV
 			case 'J':
 				c = command[i + 1];
 				if (c == 'S') tmpcode = SJIS;
@@ -645,6 +642,7 @@ int ignorelist;
 				else if (c == 'H') tmpcode = HEX;
 				else if (c == 'C') tmpcode = CAP;
 				else if (c == 'U') tmpcode = UTF8;
+				else if (c == 'M') tmpcode = M_UTF8;
 #  ifndef	_NOKANJIFCONV
 				else if (c == 'A') tmpcode = -1;
 #  endif
@@ -666,7 +664,7 @@ int ignorelist;
 				cnvcode = tmpcode;
 				cnvptr = j;
 				break;
-#endif	/* !MSDOS && !_NOKANJICONV */
+#endif	/* !_NOKANJICONV */
 			case '%':
 #ifndef	MACROMETA
 				line[j++] = '%';
@@ -693,7 +691,7 @@ int ignorelist;
 	}
 #endif
 
-#if	!MSDOS && !defined (_NOKANJICONV)
+#ifndef	_NOKANJICONV
 	if (cnvcode != NOCNV) j = extconv(&line, cnvptr, j, &size, cnvcode);
 #endif
 
@@ -756,7 +754,7 @@ int ignorelist;
 
 		len = strlen(arg);
 #if	defined (_NOEXTRAMACRO) && defined (MAXCOMMSTR)
-		if (j + len + 1 > MAXCOMMSTR);
+		if (j + len + 1 > MAXCOMMSTR) /*EMPTY*/;
 		else
 #endif
 		{
@@ -1093,42 +1091,24 @@ char **argp;
 }
 #endif	/* !_NOEXTRAMACRO */
 
-#if	!defined (MACROMETA) || defined (_NOEXTRAMACRO)
-int argfputs(s, fp)
+char *restorearg(s)
 char *s;
-FILE *fp;
 {
-	return(kanjifputs(s, fp));
+#if	defined (MACROMETA) && !defined (_NOEXTRAMACRO)
+	return(_restorearg(s));
+#else
+	return(s);
+#endif
 }
 
 /*ARGSUSED*/
 VOID demacroarg(argp)
 char **argp;
 {
-	return;
-}
-
-#else	/* MACROMETA && !_NOEXTRAMACRO */
-
-int argfputs(s, fp)
-char *s;
-FILE *fp;
-{
-	char *buf;
-	int n;
-
-	buf = restorearg(s);
-	n = kanjifputs(buf, fp);
-	if (buf != s) free(buf);
-	return(n);
-}
-
-VOID demacroarg(argp)
-char **argp;
-{
+#if	defined (MACROMETA) && !defined (_NOEXTRAMACRO)
 	*argp = _demacroarg(*argp);
+#endif
 }
-#endif	/* MACROMETA && !_NOEXTRAMACRO */
 
 char *inputshellstr(prompt, ptr, def)
 char *prompt;
@@ -1273,10 +1253,7 @@ int ignorelist;
 	if (!wastty) stdiomode();
 	free(command);
 	if (!cp) {
-		if (!wastty) {
-			fputc('\n', stdout);
-			fflush(stdout);
-		}
+		if (!wastty) fputnl(stdout);
 		return(NULL);
 	}
 	if (!*cp) {
@@ -1463,13 +1440,18 @@ int argset;
 {
 	macrostat st;
 	FILE *fp;
-	char *tmp;
+	char *tmp, *lang;
 
 	internal_status = -2;
 	st.flags = (argset || isinternalcomm(command)) ? F_ARGSET : 0;
 
 	if (!(tmp = evalcommand(command, arg, &st, 1))) return(NULL);
+	if ((lang = strdup2(getenv2("LANG")))) setenv2("LANG", "C", 1);
 	fp = popen2(tmp, "r");
+	if (lang) {
+		setenv2("LANG", lang, 1);
+		free(lang);
+	}
 	free(tmp);
 	return(fp);
 }
@@ -1521,7 +1503,7 @@ int noconf, argset, ignorelist;
 
 	if (!(argc = getargs(command, &argv))) i = maxuserfunc;
 	else for (i = 0; i < maxuserfunc; i++)
-		if (!strpathcmp(argv[0], userfunclist[i].func)) break;
+		if (!strcommcmp(argv[0], userfunclist[i].func)) break;
 	if (i >= maxuserfunc)
 		ret = execmacro(command, arg, noconf, argset, ignorelist);
 	else {
@@ -1551,7 +1533,7 @@ char *command;
 	len = (cp = strpbrk(command, " \t")) ? cp - command : strlen(command);
 
 	for (i = 0; i < maxalias; i++)
-		if (!strnpathcmp(command, aliaslist[i].alias, len)
+		if (!strncommcmp(command, aliaslist[i].alias, len)
 		&& !aliaslist[i].alias[len]) break;
 	if (i >= maxalias) return(NULL);
 
@@ -1591,7 +1573,7 @@ int uniq;
 	if (uniq) {
 		for (i = 0; i <= size; i++) {
 			if (!history[n][i]) continue;
-#if	MSDOS
+#if	defined (PATHNOCASE) && defined (_USEDOSPATH)
 			if (n == 1 && *s != *(history[n][i])) continue;
 #endif
 			if (!strpathcmp(s, history[n][i])) break;
@@ -1743,8 +1725,8 @@ char *command;
 					continue;
 				}
 				command[++i] = '\0';
-				fputs(&(command[len]), stderr);
-				fputs(": Event not found.\n", stderr);
+				fprintf2(stderr, "%k: Event not found.\n",
+					&(command[len]));
 				free(cp);
 				return(NULL);
 			}
@@ -1760,7 +1742,7 @@ char *command;
 	cp[j] = '\0';
 	if (hit) {
 		kanjifputs(cp, stderr);
-		fputc('\n', stderr);
+		fputnl(stderr);
 		free(command);
 		return(cp);
 	}
@@ -1790,7 +1772,7 @@ char ***argvp;
 	int i;
 
 	for (i = 0; i < maxalias; i++) {
-		if (strnpathcmp(com, aliaslist[i].alias, len)
+		if (strncommcmp(com, aliaslist[i].alias, len)
 		|| finddupl(aliaslist[i].alias, argc, *argvp)) continue;
 		*argvp = (char **)realloc2(*argvp,
 			(argc + 1) * sizeof(char *));
@@ -1807,7 +1789,7 @@ char ***argvp;
 	int i;
 
 	for (i = 0; i < maxuserfunc; i++) {
-		if (strnpathcmp(com, userfunclist[i].func, len)
+		if (strncommcmp(com, userfunclist[i].func, len)
 		|| finddupl(userfunclist[i].func, argc, *argvp)) continue;
 		*argvp = (char **)realloc2(*argvp,
 			(argc + 1) * sizeof(char *));
