@@ -18,6 +18,7 @@
 #if	MSDOS
 #include <dos.h>
 #include <io.h>
+#define	FR_CARRY	00001
 # ifdef	PC98
 # define	NOTUSEBIOS
 # else
@@ -125,18 +126,11 @@ typedef struct sgttyb	termioctl_t;
 extern int errno;
 #endif
 
+#include "printf.h"
 #include "term.h"
 
-#define	MAXLONGWIDTH	20		/* log10(2^64) = 19.266 */
-#define	BUFUNIT		16
 #define	GETSIZE		"\033[6n"
 #define	SIZEFMT		"\033[%d;%dR"
-#define	VF_PLUS		0001
-#define	VF_MINUS	0002
-#define	VF_SPACE	0004
-#define	VF_ZERO		0010
-#define	VF_LONG		0020
-#define	VF_UNSIGNED	0040
 #define	WINTERMNAME	"iris"
 #ifdef	USESTRERROR
 #define	strerror2		strerror
@@ -153,10 +147,14 @@ extern char *_mtrace_file;
 #endif
 
 #if	!MSDOS
+# if	defined (USETERMINFO) && defined (SVR4)
+typedef char	tputs_t;
+# else
+typedef int	tputs_t;
+# endif
 # ifdef	USETERMINFO
 # include <curses.h>
 # include <term.h>
-typedef char	tputs_t;
 # define	tgetnum2(s)		(s)
 # define	tgetflag2(s)		(s)
 # define	tgoto2(s, p1, p2)	tparm(s, p2, p1, \
@@ -262,7 +260,6 @@ typedef char	tputs_t;
 # define	TERM_at1		key_beg
 # define	TERM_at7		key_end
 # else	/* !USETERMINFO */
-typedef int	tputs_t;
 extern int tgetent __P_((char *, char *));
 extern int tgetnum __P_((char *));
 extern int tgetflag __P_((char *));
@@ -420,15 +417,17 @@ extern int tputs __P_((char *, int, int (*)__P_((tputs_t))));
 # define	FREAD	(O_RDONLY + 1)
 # endif
 #endif
+#endif	/* !MSDOS */
 
 #ifndef	FD_SET
 typedef struct fd_set {
-	int fds_bits[1];
+	u_int fds_bits[1];
 } fd_set;
 # define	FD_ZERO(p)	(((p) -> fds_bits[0]) = 0)
-# define	FD_SET(n, p)	(((p) -> fds_bits[0]) |= (1 << (n)))
-#endif
-#endif	/* !MSDOS */
+# define	FD_SET(n, p)	(((p) -> fds_bits[0]) |= ((u_int)1 << (n)))
+#endif	/* !FD_SET */
+
+#define	MAXFDSET	256
 
 #ifndef	LSI_C
 #define	safe_dup	dup
@@ -446,11 +445,6 @@ static char *NEAR tstrdup __P_((char *));
 #endif
 static int NEAR defaultterm __P_((VOID_A));
 static int NEAR maxlocate __P_((int *, int *));
-static int NEAR getnum __P_((CONST char *, int *));
-static char *NEAR setchar __P_((int, char *, int *, int *));
-static char *NEAR setint __P_((long, int,
-		char *, int *, int *, int, int, int));
-static char *NEAR setstr __P_((char *, char *, int *, int *, int, int, int));
 #if	MSDOS
 # ifdef	USEVIDEOBIOS
 static int NEAR bioslocate __P_((int, int));
@@ -787,6 +781,7 @@ int oldd, newd;
 {
 	int fd;
 
+	if (oldd == newd) return(newd);
 	if ((fd = dup2(oldd, newd)) < 0) return(-1);
 	if (newd >= 0 && newd < SYS_OPEN && oldd >= 0 && oldd < SYS_OPEN)
 		_openfile[newd] = _openfile[oldd];
@@ -796,7 +791,11 @@ int oldd, newd;
 
 int opentty(VOID_A)
 {
+#ifdef	SELECTRWONLY
+	if (ttyio < 0 && (ttyio = open(TTYNAME, O_RDONLY, 0600)) < 0
+#else
 	if (ttyio < 0 && (ttyio = open(TTYNAME, O_RDWR, 0600)) < 0
+#endif
 	&& (ttyio = safe_dup(STDERR_FILENO)) < 0)
 		err2("dup()");
 	return(ttyio);
@@ -806,14 +805,19 @@ int opentty(VOID_A)
 static int NEAR newdup(fd)
 int fd;
 {
-	struct stat st;
+	__dpmi_regs reg;
 	int n;
 
 	if (fd < 0
 	|| fd == STDIN_FILENO || fd == STDOUT_FILENO || fd == STDERR_FILENO)
 		return(fd);
 
-	for (n = 20 - 1; n > fd; n--) if (fstat(n, &st) != 0) break;
+	for (n = 20 - 1; n > fd; n--) {
+		reg.x.ax = 0x4400;
+		reg.x.bx = (u_int)n;
+		intdos2(&reg);
+		if (reg.x.flags & FR_CARRY) break;
+	}
 	if (n <= fd || safe_dup2(fd, n) < 0) return(fd);
 	close(fd);
 	return(n);
@@ -1263,8 +1267,11 @@ int no;
 # if	!MSDOS
 	freeterment();
 # endif
-	if (ttyout && ttyout != stderr) fclose(ttyout);
-	else if (ttyio >= 0) close(ttyio);
+	if (ttyout && ttyout != stderr) {
+		if (fileno(ttyout) == ttyio) ttyio = -1;
+		fclose(ttyout);
+	}
+	if (ttyio >= 0) close(ttyio);
 	muntrace();
 #endif
 	exit(no);
@@ -1287,14 +1294,11 @@ char *mes;
 		tabs();
 	}
 	if (dumbterm <= 2) fputc('\007', stderr);
-	fputc('\n', stderr);
+	fputnl(stderr);
 	fputs(mes, stderr);
 
-	if (duperrno) {
-		fputs(": ", stderr);
-		fputs(strerror2(duperrno), stderr);
-	}
-	fputc('\n', stderr);
+	if (duperrno) fprintf2(stderr, ": %s", strerror2(duperrno));
+	fputnl(stderr);
 	inittty(1);
 	exit(2);
 /*NOTREACHED*/
@@ -1355,7 +1359,7 @@ static int NEAR defaultterm(VOID_A)
 #if	!MSDOS
 	for (i = 0; i <= K_MAX - K_MIN; i++) keyseq[i].code = K_MIN + i;
 	for (i = 21; i <= 30; i++)
-		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) & 01000;
+		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | 01000;
 	for (i = 31; K_F(i) < K_DL; i++)
 		if (defkeyseq[K_F(i) - K_MIN]) keyseq[K_F(i) - K_MIN].code = i;
 	keyseq[K_F('?') - K_MIN].code = K_CR;
@@ -1472,284 +1476,6 @@ int *yp, *xp;
 }
 #endif	/* !MSDOS || !USEVIDEOBIOS */
 
-static int NEAR getnum(s, ptrp)
-CONST char *s;
-int *ptrp;
-{
-	int i, n;
-
-	i = *ptrp;
-	for (n = 0; isdigit(s[i]); i++) n = n * 10 + (s[i] - '0');
-	if (i <= *ptrp) n = -1;
-	*ptrp = i;
-	return(n);
-}
-
-static char *NEAR setchar(n, buf, ptrp, sizep)
-int n;
-char *buf;
-int *ptrp, *sizep;
-{
-	char *tmp;
-
-	if (*ptrp + 1 >= *sizep) {
-		*sizep += BUFUNIT;
-		if ((tmp = (char *)realloc(buf, *sizep))) buf = tmp;
-		else {
-			free(buf);
-			return(NULL);
-		}
-	}
-	buf[(*ptrp)++] = n;
-	return(buf);
-}
-
-static char *NEAR setint(n, base, buf, ptrp, sizep, flags, width, prec)
-long n;
-int base;
-char *buf;
-int *ptrp, *sizep, flags, width, prec;
-{
-	char num[MAXLONGWIDTH + 1];
-	u_long ul;
-	int i, len, sign, cap, bit;
-
-	cap = 0;
-	if (base >= 256) {
-		cap++;
-		base -= 256;
-	}
-
-	ul = (u_long)n;
-	bit = 0;
-	if (base != 10) {
-		sign = 0;
-		flags &= ~(VF_PLUS | VF_SPACE);
-		i = 1;
-		for (i = 1; i < base; i <<= 1) bit++;
-		base--;
-	}
-	else if (n >= 0 || (flags & VF_UNSIGNED))
-		sign = (flags & VF_PLUS) ? 1 : 0;
-	else {
-		sign = -1;
-		n = -n;
-	}
-	if (sign || (flags & VF_SPACE)) width--;
-	if ((flags & VF_ZERO) && prec < 0) prec = width;
-	len = 0;
-	while (len < sizeof(num) / sizeof(char)) {
-		if (!bit) i = (((flags & VF_UNSIGNED) ? ul : n) % base) + '0';
-		else if ((i = (ul & base)) < 10) i += '0';
-		else if (cap) i += 'A' - 10;
-		else i += 'a' - 10;
-
-		num[len++] = i;
-		if (bit) {
-			ul >>= bit;
-			if (!ul) break;
-		}
-		else if (flags & VF_UNSIGNED) {
-			ul /= base;
-			if (!ul) break;
-		}
-		else {
-			n /= base;
-			if (!n) break;
-		}
-	}
-	if (!(flags & VF_MINUS) && width >= 0 && len < width) {
-		while (len < width--) {
-			if (prec >= 0 && prec > width) break;
-			if (!(buf = setchar(' ', buf, ptrp, sizep)))
-				return(NULL);
-		}
-	}
-
-	i = '\0';
-	if (sign) i = (sign > 0) ? '+' : '-';
-	else if (flags & VF_SPACE) i = ' ';
-	if (i && !(buf = setchar(i, buf, ptrp, sizep))) return(NULL);
-
-	if (prec >= 0 && len < prec) {
-		while (len < prec--) {
-			width--;
-			if (!(buf = setchar('0', buf, ptrp, sizep)))
-				return(NULL);
-		}
-	}
-	for (i = 0; i < len; i++) {
-		if (!(buf = setchar(num[len - i - 1], buf, ptrp, sizep)))
-			return(NULL);
-	}
-
-	if (width >= 0) for (; i < width; i++) {
-		if (!(buf = setchar(' ', buf, ptrp, sizep))) return(NULL);
-	}
-
-	return(buf);
-}
-
-static char *NEAR setstr(s, buf, ptrp, sizep, flags, width, prec)
-char *s, *buf;
-int *ptrp, *sizep, flags, width, prec;
-{
-	int i, len;
-
-	if (s) {
-		len = strlen(s);
-		if (prec >= 0 && len > prec) len = prec;
-	}
-	else {
-		s = "(null)";
-		len = strlen(s);
-		if (prec >= 0 && len > prec) len = 0;
-	}
-
-	if (!(flags & VF_MINUS) && width >= 0 && len < width) {
-		while (len < width--) {
-			if (!(buf = setchar(' ', buf, ptrp, sizep)))
-				return(NULL);
-		}
-	}
-	for (i = 0; i < len; i++) {
-		if (!(buf = setchar(s[i], buf, ptrp, sizep)))
-			return(NULL);
-	}
-
-	if (width >= 0) for (; i < width; i++) {
-		if (!(buf = setchar(' ', buf, ptrp, sizep)))
-			return(NULL);
-	}
-
-	return(buf);
-}
-
-int vasprintf2(sp, fmt, args)
-char **sp;
-CONST char *fmt;
-va_list args;
-{
-	char *buf;
-	long l;
-	int i, j, base, size, flags, width, prec;
-
-	if (!(buf = (char *)malloc(size = BUFUNIT))) return(-1);
-	for (i = j = 0; fmt[i]; i++) {
-		if (fmt[i] != '%') {
-			if (!(buf = setchar(fmt[i], buf, &j, &size)))
-				return(-1);
-			continue;
-		}
-
-		i++;
-		flags = 0;
-		width = prec = -1;
-		for (;;) {
-			if (fmt[i] == '+') flags |= VF_PLUS;
-			else if (fmt[i] == '-') flags |= VF_MINUS;
-			else if (fmt[i] == ' ') flags |= VF_SPACE;
-			else if (fmt[i] == '0') flags |= VF_ZERO;
-			else break;
-
-			i++;
-		}
-
-		if (fmt[i] != '*') width = getnum(fmt, &i);
-		else {
-			i++;
-			width = (int)va_arg(args, int);
-		}
-		if (fmt[i] == '.') {
-			i++;
-			if (fmt[i] != '*') prec = getnum(fmt, &i);
-			else {
-				i++;
-				prec = va_arg(args, int);
-			}
-		}
-		if (fmt[i] == 'l') {
-			i++;
-			flags |= VF_LONG;
-		}
-
-		base = 0;
-		switch (fmt[i]) {
-			case 'd':
-				base = 10;
-				break;
-			case 's':
-				buf = setstr(va_arg(args, char *),
-					buf, &j, &size, flags, width, prec);
-				break;
-			case 'c':
-				buf = setchar(va_arg(args, int),
-					buf, &j, &size);
-				break;
-			case 'p':
-				flags |= VF_LONG;
-/*FALLTHRU*/
-			case 'x':
-				base = 16;
-				break;
-			case 'X':
-				base = 16 + 256;
-				break;
-			case 'o':
-				base = 8;
-				break;
-			case 'u':
-				flags |= VF_UNSIGNED;
-				base = 10;
-				break;
-			default:
-				buf = setchar(fmt[i], buf, &j, &size);
-				break;
-		}
-
-		if (base) {
-			if (flags & VF_LONG) l = va_arg(args, long);
-			else l = (long)va_arg(args, int);
-			buf = setint(l, base,
-				buf, &j, &size, flags, width, prec);
-		}
-
-		if (!buf) return(-1);
-	}
-	va_end(args);
-
-	buf[j] = '\0';
-	if (sp) *sp = buf;
-	else free(buf);
-	return(j);
-}
-
-#ifdef	USESTDARGH
-/*VARARGS2*/
-int asprintf2(char **sp, CONST char *fmt, ...)
-#else
-/*VARARGS2*/
-int asprintf2(sp, fmt, va_alist)
-char **sp;
-CONST char *fmt;
-va_dcl
-#endif
-{
-	va_list args;
-	int n;
-
-#ifdef	USESTDARGH
-	va_start(args, fmt);
-#else
-	va_start(args);
-#endif
-
-	n = vasprintf2(sp, fmt, args);
-	va_end(args);
-	if (n < 0) err2("malloc()");
-	return(n);
-}
-
 #if	MSDOS
 char *tparamstr(s, arg1, arg2)
 char *s;
@@ -1757,7 +1483,7 @@ int arg1, arg2;
 {
 	char *cp;
 
-	asprintf2(&cp, s, arg1, arg2);
+	if (asprintf2(&cp, s, arg1, arg2) < 0) return(NULL);
 	return(cp);
 }
 
@@ -1780,7 +1506,6 @@ char *tparamstr(s, arg1, arg2)
 char *s;
 int arg1, arg2;
 {
-	char *buf;
 # ifdef	USETERMINFO
 #  ifdef	DEBUG
 	if (!s) return(NULL);
@@ -1796,57 +1521,55 @@ int arg1, arg2;
 	if (!s || !(s = tparm(s, arg1, arg2, 0, 0, 0, 0, 0, 0, 0)))
 		return(NULL);
 #  endif
-	buf = tstrdup(s);
+	return(tstrdup(s));
 # else	/* !USETERMINFO */
+	printbuf_t pbuf;
 	char *tmp;
-	int i, j, n, len, size, args[2];
+	int i, j, n, err, args[2];
 
-	if (!s) return(NULL);
-	if (!(buf = (char *)malloc(size = BUFUNIT))) err2("malloc()");
+	if (!s || !(pbuf.buf = (char *)malloc(1))) return(NULL);
+	pbuf.ptr = pbuf.size = 0;
+	pbuf.flags = VF_NEW;
 	args[0] = arg1;
 	args[1] = arg2;
 
-	for (i = j = n = 0; s[i]; i++) {
+	for (i = n = 0; s[i]; i++) {
 		tmp = NULL;
+		err = 0;
 		if (s[i] != '%' || s[++i] == '%') {
-			if (!(buf = setchar(s[i], buf, &j, &size)))
-				err2("realloc()");
-		}
-		else if (n >= 2) {
-			free(buf);
-			return(NULL);
+			if (setchar(s[i], &pbuf) < 0) return(NULL);
 		}
+		else if (n >= 2) err++;
 		else switch (s[i]) {
 			case 'd':
-				asprintf2(&tmp, "%d", args[n++]);
+				if (asprintf2(&tmp, "%d", args[n++]) < 0)
+					err++;
 				break;
 			case '2':
-				asprintf2(&tmp, "%02d", args[n++]);
+				if (asprintf2(&tmp, "%02d", args[n++]) < 0)
+					err++;
 				break;
 			case '3':
-				asprintf2(&tmp, "%03d", args[n++]);
+				if (asprintf2(&tmp, "%03d", args[n++]) < 0)
+					err++;
 				break;
 			case '.':
-				asprintf2(&tmp, "%c", args[n++]);
+				if (asprintf2(&tmp, "%c", args[n++]) < 0)
+					err++;
 				break;
 			case '+':
-				if (!s[++i]) {
-					free(buf);
-					return(NULL);
-				}
-				asprintf2(&tmp, "%c", args[n++] + s[i]);
+				if (!s[++i]
+				|| asprintf2(&tmp, "%c", args[n++] + s[i]) < 0)
+					err++;
 				break;
 			case '>':
-				if (!s[++i] || !s[i + 1]) {
-					free(buf);
-					return(NULL);
-				}
-				if (args[n] > s[i++]) args[n] += s[i];
+				if (!s[++i] || !s[i + 1]) err++;
+				else if (args[n] > s[i++]) args[n] += s[i];
 				break;
 			case 'r':
-				len = args[0];
+				j = args[0];
 				args[0] = args[1];
-				args[1] = len;
+				args[1] = j;
 				break;
 			case 'i':
 				args[0]++;
@@ -1864,28 +1587,24 @@ int arg1, arg2;
 				args[n] -= 2 * (args[n] % 16);
 				break;
 			default:
-				free(buf);
-				return(NULL);
-/*NOTREACHED*/
+				err++;
 				break;
 		}
 
+		if (err) {
+			free(pbuf.buf);
+			return(NULL);
+		}
 		if (tmp) {
-			len = strlen(tmp);
-			if (j + len + 1 >= size) {
-				size += BUFUNIT;
-				if (!(buf = (char *)realloc(buf, size)))
-					err2("realloc()");
-			}
-			memcpy(&(buf[j]), tmp, len);
+			for (j = 0; tmp[j]; j++)
+				if (setchar(tmp[j], &pbuf) < 0) return(NULL);
 			free(tmp);
-			j += len;
 		}
 	}
 
-	buf[j] = '\0';
+	pbuf.buf[pbuf.ptr] = '\0';
+	return(pbuf.buf);
 # endif	/* !USETERMINFO */
-	return(buf);
 }
 
 static char *NEAR tgetstr2(term, s)
@@ -1998,9 +1717,16 @@ static int NEAR cmpkeyseq(vp1, vp2)
 CONST VOID_P vp1;
 CONST VOID_P vp2;
 {
-	if (!((keyseq_t *)vp1) -> str) return(-1);
-	if (!((keyseq_t *)vp2) -> str) return(1);
-	return(strcmp(((keyseq_t *)vp1) -> str, ((keyseq_t *)vp2) -> str));
+	keyseq_t *kp1, *kp2;
+	int n;
+
+	kp1 = (keyseq_t *)vp1;
+	kp2 = (keyseq_t *)vp2;
+	if (!(kp1 -> str)) return(-1);
+	if (!(kp2 -> str)) return(1);
+	n = (kp1 -> len < kp2 -> len) ? kp1 -> len : kp2 -> len;
+	if ((n = memcmp(kp1 -> str, kp2 -> str, n))) return(n);
+	return((int)(kp1 -> len) - (int)(kp2 -> len));
 }
 
 static int NEAR sortkeyseq(VOID_A)
@@ -2045,7 +1771,10 @@ char *s;
 	int i, j, dumb, dupdumbterm;
 
 	if (termflags & F_TERMENT) return(-1);
-	if (!ttyout && !(ttyout = fdopen(ttyio, "w+"))) ttyout = stderr;
+	if (!ttyout) {
+		if (!(ttyout = fdopen(ttyio, "w+"))
+		&& !(ttyout = fopen(TTYNAME, "w+"))) ttyout = stderr;
+	}
 	dupdumbterm = dumbterm;
 	dumbterm = dumb = 0;
 	term = (s) ? s : (char *)getenv("TERM");
@@ -2305,7 +2034,10 @@ int len;
 		keyseq[i].len = len;
 		break;
 	}
-	if (i > K_MAX - K_MIN) return(-1);
+	if (i > K_MAX - K_MIN) {
+		free(str);
+		return(-1);
+	}
 
 	if (str) {
 		for (i = 0; i <= K_MAX - K_MIN; i++) {
@@ -2325,17 +2057,20 @@ int len;
 	return(0);
 }
 
-char *getkeyseq(n, lenp)
-int n, *lenp;
+int getkeyseq(kp)
+keyseq_t *kp;
 {
 	int i;
 
-	for (i = 0; i <= K_MAX - K_MIN; i++) if (keyseq[i].code == n) {
-		if (lenp) *lenp = keyseq[i].len;
-		return((keyseq[i].str) ? keyseq[i].str : "");
+	for (i = 0; i <= K_MAX - K_MIN; i++) {
+		if (keyseq[i].code == kp -> code) {
+			memcpy((char *)kp, (char *)&(keyseq[i]),
+				sizeof(keyseq_t));
+			return(0);
+		}
 	}
-	if (lenp) *lenp = 0;
-	return(NULL);
+	kp -> len = 0;
+	return(-1);
 }
 
 keyseq_t *copykeyseq(list)
@@ -2810,7 +2545,7 @@ u_long usec;
 # ifndef	PC98
 	fd_set readfds;
 	struct timeval tv;
-	int n;
+	int n, fd;
 # endif
 #endif
 
@@ -2831,15 +2566,17 @@ u_long usec;
 	return(reg.h.bh != 0);
 # else	/* !PC98 */
 #  if	defined (DJGPP) && (DJGPP >= 2)
+	fd = (ttyio < MAXFDSET || (n = safe_dup(ttyio)) < 0) ? ttyio : n;
 	tv.tv_sec = usec / 1000000L;
 	tv.tv_usec = usec % 1000000L;
 	FD_ZERO(&readfds);
-	FD_SET(STDIN_FILENO, &readfds);
+	FD_SET(fd, &readfds);
 
 	do {
-		n = select(STDIN_FILENO + 1, &readfds, NULL, NULL, &tv);
+		n = select(fd + 1, &readfds, NULL, NULL, &tv);
 	} while (n < 0 && errno == EINTR);
-	if (n < 0) err2(TTYNAME);
+	if (fd != ttyio) close(fd);
+	if (n < 0) err2("select()");
 	return(n);
 #  else	/* !DJGPP || DJGPP < 2 */
 	reg.h.ah = 0x01;
@@ -2957,6 +2694,7 @@ int sig;
 			raise(sig);
 		}
 # endif
+		if (keywaitfunc) (*keywaitfunc)();
 	} while (!i);
 #else	/* !DJGPP || NOTUSEBIOS || PC98 */
 	if (tbuf1[0] == 0xff) dosgettime(tbuf1);
@@ -2968,6 +2706,7 @@ int sig;
 # endif
 			memcpy((char *)tbuf1, (char *)tbuf2, sizeof(tbuf1));
 		}
+		if (keywaitfunc) (*keywaitfunc)();
 	}
 #endif	/* !DJGPP || NOTUSEBIOS || PC98 */
 	ch = getch2();
@@ -3090,17 +2829,19 @@ u_long usec;
 # else
 	fd_set readfds;
 	struct timeval tv;
-	int n;
+	int n, fd;
 
+	fd = (ttyio < MAXFDSET || (n = safe_dup(ttyio)) < 0) ? ttyio : n;
 	tv.tv_sec = usec / 1000000L;
 	tv.tv_usec = usec % 1000000L;
 	FD_ZERO(&readfds);
-	FD_SET(STDIN_FILENO, &readfds);
+	FD_SET(fd, &readfds);
 
 	do {
-		n = select(STDIN_FILENO + 1, &readfds, NULL, NULL, &tv);
+		n = select(fd + 1, &readfds, NULL, NULL, &tv);
 	} while (n < 0 && errno == EINTR);
-	if (n < 0) err2(TTYNAME);
+	if (fd != ttyio) close(fd);
+	if (n < 0) err2("select()");
 	return(n);
 # endif
 }
@@ -3362,6 +3103,19 @@ va_dcl
 	return(n);
 }
 
+int cputnl(VOID_A)
+{
+	cputs2("\r\n");
+	tflush();
+	return(0);
+}
+
+int kanjiputs(s)
+char *s;
+{
+	return(cprintf2("%k", s));
+}
+
 int chgcolor(color, reverse)
 int color, reverse;
 {
