@@ -6,7 +6,6 @@
 
 #include <ctype.h>
 #include "fd.h"
-#include "term.h"
 #include "func.h"
 #include "kanji.h"
 
@@ -23,6 +22,7 @@ extern int win_y;
 #ifndef	_NODOSDRIVE
 extern int lastdrv;
 #endif
+extern int tradlayout;
 extern int sizeinfo;
 
 #define	DIRFIELD	3
@@ -107,7 +107,7 @@ int disp;
 		}
 		else if (disp) {
 			locate(x + TREEFIELD + 4, y);
-			kanjiputs2(dp -> d_name, w, 0);
+			cprintf2("%-*.*k", w, w, dp -> d_name);
 			i++;
 			if (++y >= LFILEBOTTOM) {
 				y = LFILETOP + 1;
@@ -118,7 +118,7 @@ int disp;
 	}
 	if (disp && !i) {
 		locate(x + TREEFIELD + 4, LFILETOP + 1);
-		kanjiputs2("[No Files]", w, 0);
+		cprintf2("%-*.*s", w, w, "[No Files]");
 	}
 	Xclosedir(dirp);
 	return(i);
@@ -134,7 +134,7 @@ int level, *maxp;
 	struct dirent *dp;
 	struct stat st;
 	char *cp, *dir, *subdir, cwd[MAXPATHLEN];
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	char tmp[MAXPATHLEN];
 #endif
 	int i, len;
@@ -160,7 +160,7 @@ int level, *maxp;
 		subdir = path + 1;
 		if (!*subdir) subdir = NULL;
 	}
-#if	!MSDOS && !defined (_NODOSDRIVE)
+#ifdef	_USEDOSEMU
 	else if (_dospath(path)) {
 		dir = strndup2(path, 3);
 		subdir = path + 3;
@@ -191,7 +191,7 @@ int level, *maxp;
 			list[*maxp].name = strdup2(dp -> d_name);
 			list[*maxp].sub = NULL;
 			list[*maxp].max = 0;
-#if	!MSDOS
+#ifndef	NODIRLOOP
 			list[*maxp].dev = st.st_dev;
 			list[*maxp].ino = st.st_ino;
 			list[*maxp].parent = parent;
@@ -203,29 +203,41 @@ int level, *maxp;
 			list[0].name = strdup2(dp -> d_name);
 			list[0].sub = &(list[0]);
 			list[0].max = 0;
-#if	!MSDOS
+#ifndef	NODIRLOOP
 			list[0].dev = st.st_dev;
 			list[0].ino = st.st_ino;
 			list[0].parent = parent;
 #endif
 			if (++(*maxp) >= 2) break;
 		}
-		else if (!i) {
-			i++;
-			list[1].name = NULL;
-			list[1].sub = NULL;
-			list[1].max = -1;
-#if	!MSDOS
-			list[1].dev = 0;
-			list[1].ino = 0;
-			list[1].parent = NULL;
+		else {
+			if (!(*maxp)) {
+				list[0].name = NULL;
+				list[0].sub = NULL;
+				list[0].max = -1;
+#ifndef	NODIRLOOP
+				list[0].dev = 0;
+				list[0].ino = 0;
+				list[0].parent = NULL;
 #endif
-			if (++(*maxp) >= 2) break;
+			}
+			if (!i) {
+				i++;
+				list[1].name = NULL;
+				list[1].sub = NULL;
+				list[1].max = -1;
+#ifndef	NODIRLOOP
+				list[1].dev = 0;
+				list[1].ino = 0;
+				list[1].parent = NULL;
+#endif
+				if (++(*maxp) >= 2) break;
+			}
 		}
 	}
 	Xclosedir(dirp);
 	if (list) for (i = 0; i < *maxp; i++) {
-#if	!MSDOS
+#ifndef	NODIRLOOP
 		treelist *lp;
 
 		if (!(list[i].name)) lp = NULL;
@@ -242,7 +254,7 @@ int level, *maxp;
 			list[i].sub = maketree(subdir, NULL, &(list[i]),
 				level + 1, &(list[i].max));
 		else {
-			if (list[i].max >= 0
+			if (list[i].max >= 0 && list[i].name
 			&& evaldir(nodospath(tmp, list[i].name), 0))
 				list[i].max = -1;
 		}
@@ -251,14 +263,13 @@ int level, *maxp;
 
 #if	MSDOS
 	if (_dospath(path)) path += 2;
-	if (*path == _SC_) {
-#else	/* !MSDOS */
-	if (*path == _SC_
-# ifndef	_NODOSDRIVE
-	|| _dospath(path)
-# endif
-	) {
-#endif	/* !MSDOS */
+#endif
+#if	MSDOS || defined (_NODOSDRIVE)
+	if (*path == _SC_)
+#else
+	if (*path == _SC_ || _dospath(path))
+#endif
+	{
 		if (_chdir2(fullpath) < 0) error(fullpath);
 	}
 	else if (strcmp(path, ".") && _chdir2(cwd) < 0) error("..");
@@ -321,7 +332,7 @@ static VOID NEAR showtree(VOID_A)
 	for (i = LFILETOP + 1; i < LFILEBOTTOM; i++) {
 		locate(1, i);
 		if (i == tr_line) putterm(t_standout);
-		kanjiputs2(bufptr(i), TREEFIELD, 0);
+		cprintf2("%-*.*k", TREEFIELD, TREEFIELD, bufptr(i));
 		if (i == tr_line) putterm(end_standout);
 	}
 	evaldir(treepath, 1);
@@ -331,8 +342,7 @@ static VOID NEAR showtree(VOID_A)
 static VOID NEAR treebar(VOID_A)
 {
 	locate(1, LFILETOP);
-	cputs2("Tree=");
-	kanjiputs2(treepath, n_column - 6, 0);
+	cprintf2("Tree=%-*.*k", n_column - 6, n_column - 6, treepath);
 	locate(0, L_MESLINE);
 	putterm(l_clear);
 }
@@ -351,7 +361,7 @@ treelist *list;
 int max, nest;
 {
 	treelist *lp, *tmplp;
-	int i, j, w, len;
+	int i, w, len;
 
 	lp = NULL;
 	w = TREEFIELD - (nest * DIRFIELD) - 1;
@@ -372,17 +382,18 @@ int max, nest;
 #if	MSDOS
 				if (!_dospath(list[i].name)
 				|| list[i].name[2] != _SC_) len++;
-				for (j = strlen(treepath); j >= 0; j--)
-					treepath[j + len] = treepath[j];
+				memmove(&(treepath[len]), treepath,
+					strlen(treepath) + 1);
 				memcpy(treepath, list[i].name, len - 1);
 #else	/* !MSDOS */
+# ifdef	_NODOSDRIVE
+				if (*list[i].name != _SC_) len++;
+# else
 				if (*list[i].name != _SC_
-# ifndef	_NODOSDRIVE
-				&& !_dospath(list[i].name)
+				&& !_dospath(list[i].name)) len++;
 # endif
-				) len++;
-				for (j = strlen(treepath); j >= 0; j--)
-					treepath[j + len] = treepath[j];
+				memmove(&(treepath[len]), treepath,
+					strlen(treepath) + 1);
 				if (*list[i].name != _SC_)
 					memcpy(treepath, list[i].name,
 						len - 1);
@@ -413,7 +424,10 @@ treelist *list;
 		memcpy((char *)lp, (char *)&(list -> sub[0]),
 			sizeof(treelist));
 		for (i = 1; i < list -> max; i++)
-			if ((list -> sub[i]).name) free((list -> sub[i]).name);
+			if ((list -> sub[i]).name) {
+				free((list -> sub[i]).name);
+				(list -> sub[i]).name = NULL;
+			}
 	}
 
 	if (_chdir2(treepath) < 0) {
@@ -435,8 +449,10 @@ treelist *list;
 		return(i);
 	}
 	if (list -> sub) {
-		for (i = 0; i < list -> max; i++)
-			if (!strpathcmp(lp -> name, lptmp[i].name)) break;
+		if (!(lp -> name)) i = list -> max;
+		else for (i = 0; i < list -> max; i++)
+			if (lptmp[i].name
+			&& !strpathcmp(lp -> name, lptmp[i].name)) break;
 		if (i < list -> max) {
 			free(lptmp[i].name);
 			for (; i > 0; i--) memcpy((char *)&(lptmp[i]),
@@ -682,7 +698,7 @@ static int NEAR _tree_input(VOID_A)
 
 static char *NEAR _tree(VOID_A)
 {
-#if	!MSDOS
+#ifndef	NODIRLOOP
 	struct stat st;
 # ifndef	_NODOSDRIVE
 	char tmp[MAXPATHLEN];
@@ -697,7 +713,7 @@ static char *NEAR _tree(VOID_A)
 	tr_root -> name = NULL;
 	tr_root -> max = 1;
 	tr_root -> sub = tr_cur = (treelist *)malloc2(sizeof(treelist));
-#if	!MSDOS
+#ifndef	NODIRLOOP
 	tr_root -> ino = 0;
 	tr_root -> parent = NULL;
 #endif
@@ -717,7 +733,9 @@ static char *NEAR _tree(VOID_A)
 		tr_cur[0].name = strdup2(_SS_);
 	}
 # endif
+#endif	/* !MSDOS */
 
+#ifndef	NODIRLOOP
 	if (Xstat(nodospath(tmp, tr_cur[0].name), &st) < 0)
 		tr_cur[0].dev = tr_cur[0].ino = 0;
 	else {
@@ -725,7 +743,7 @@ static char *NEAR _tree(VOID_A)
 		tr_cur[0].ino = st.st_ino;
 	}
 	tr_cur[0].parent = NULL;
-#endif	/* !MSDOS */
+#endif
 	tr_cur[0].sub = maketree(path, NULL, &(tr_cur[0]),
 		0, &(tr_cur[0].max));
 
@@ -738,7 +756,7 @@ static char *NEAR _tree(VOID_A)
 	if (_dospath(cwd)) cwd += 2;
 # endif
 #endif	/* !MSDOS */
-	if (!strcmp(cwd, _SS_));
+	if (!strcmp(cwd, _SS_)) /*EMPTY*/;
 	else for (cp = cwd; (cp = strdelim(cp, 0)); cp++, tr_line++)
 		if ((tr_line + 1) * DIRFIELD + 2 > TREEFIELD
 		|| !(tr_cur = &(tr_cur -> sub[0]))) break;
@@ -771,11 +789,13 @@ static char *NEAR _tree(VOID_A)
 		else if (oy != tr_line) {
 			locate(1, tr_line);
 			putterm(t_standout);
-			kanjiputs2(bufptr(tr_line), TREEFIELD, 0);
+			cprintf2("%-*.*k",
+				TREEFIELD, TREEFIELD, bufptr(tr_line));
 			putterm(end_standout);
 			locate(1, oy);
 			if (stable_standout) putterm(end_standout);
-			else kanjiputs2(bufptr(oy), TREEFIELD, 0);
+			else cprintf2("%-*.*k",
+				TREEFIELD, TREEFIELD, bufptr(oy));
 			evaldir(path, 1);
 		}
 	} while (ch != K_ESC && ch != K_CR);
