@@ -9,12 +9,8 @@
 #include <io.h>
 #include "machine.h"
 
-#ifdef	NOVOID
-#define	VOID
-#define	VOID_T	int
-#else
-#define	VOID	void
-#define	VOID_T	void
+#ifndef	NOUNISTDH
+#include <unistd.h>
 #endif
 
 #if	MSDOS && defined (_NOUSELFN) && !defined (_NODOSDRIVE)
@@ -23,37 +19,22 @@
 
 #include "unixdisk.h"
 #include "dosdisk.h"
+#include "kctype.h"
+
+#ifndef	ENODEV
+#define	ENODEV	EACCES
+#endif
+#ifndef	EIO
+#define	EIO	ENODEV
+#endif
+#ifndef	O_ACCMODE
+#define	O_ACCMODE	(O_RDONLY | O_WRONLY | O_RDWR)
+#endif
 
-#ifdef	LSI_C
-#include <jctype.h>
-#define	toupper2	toupper
-#define	tolower2	tolower
-#define	issjis1		iskanji
-#define	issjis2		iskanji2
-#else	/* !LSI_C */
-#define	toupper2(c)	(uppercase[(u_char)(c)])
-#define	tolower2(c)	(lowercase[(u_char)(c)])
 #define	KC_SJIS1	0001
 #define	KC_SJIS2	0002
 #define	KC_EUCJP	0010
-
-# ifndef	issjis1
-# define	issjis1(c)	(kctypetable[(u_char)(c)] & KC_SJIS1)
-# endif
-# ifndef	issjis2
-# define	issjis2(c)	(kctypetable[(u_char)(c)] & KC_SJIS2)
-# endif
-
-# ifndef	iseuc
-# define	iseuc(c)	(kctypetable[(u_char)(c)] & KC_EUCJP)
-# endif
-#endif	/* !LSI_C */
-
-#ifdef	CODEEUC
-#define	iskanji1(s, i)	(iseuc((s)[i]) && iseuc((s)[(i) + 1]))
-#else
-#define	iskanji1(s, i)	(issjis1((s)[i]) && issjis2((s)[(i) + 1]))
-#endif
+#define	int21call(reg, sreg)	intcall(0x21, reg, sreg)
 
 #ifdef	FD
 extern int _dospath __P_((char *));
@@ -73,13 +54,11 @@ static char *NEAR strcatdelim2 __P_((char *, char *, char *));
 static int NEAR isdotdir __P_((char *));
 #endif
 
-static int NEAR seterrno __P_((u_short));
+static int NEAR seterrno __P_((u_int));
 #ifdef	DJGPP
 static int NEAR dos_putpath __P_((char *, int));
 #endif
 static char *NEAR duplpath __P_((char *));
-static long NEAR intcall __P_((int, __dpmi_regs *, struct SREGS *));
-#define	int21call(reg, sreg)	intcall(0x21, reg, sreg)
 #ifndef	_NODOSDRIVE
 static char *NEAR regpath __P_((char *, char *));
 static VOID NEAR biosreset __P_((int));
@@ -96,15 +75,18 @@ static int NEAR lockdrive __P_((int));
 static int NEAR unlockdrive __P_((int, int));
 static int NEAR dosrawdiskio __P_((int, u_long, u_char *, int, int, int));
 #endif
-static int NEAR dos_findfirst __P_((char *, u_short, struct dosfind_t *));
+static int NEAR dos_findfirst __P_((char *, u_int, struct dosfind_t *));
 static int NEAR dos_findnext __P_((struct dosfind_t *));
 #ifndef	_NOUSELFN
-static long NEAR lfn_findfirst __P_((char *, u_short, struct lfnfind_t *));
-static int NEAR lfn_findnext __P_((u_short, struct lfnfind_t *));
-static int NEAR lfn_findclose __P_((u_short));
+static int NEAR lfn_findfirst __P_((u_int *, char *,
+		u_int, struct lfnfind_t *));
+static int NEAR lfn_findnext __P_((u_int, struct lfnfind_t *));
+static int NEAR lfn_findclose __P_((u_int));
 #endif
+#ifndef	_NOUSELFN
 static int NEAR gendosname __P_((char *));
 static int NEAR unixrenamedir __P_((char *, char *));
+#endif
 static u_int NEAR getdosmode __P_((u_int));
 static u_int NEAR putdosmode __P_((u_int));
 static time_t NEAR getdostime __P_((u_int, u_int));
@@ -168,117 +150,6 @@ static u_char int25bin[] = {
 static int (far *doint25)__P_((VOID_A)) = NULL;
 #endif
 
-#ifndef	LSI_C
-# ifdef	FD
-extern u_char uppercase[256];
-extern u_char lowercase[256];
-extern u_char kctypetable[256];
-# else	/* !FD */
-static u_char uppercase[256] = {
-	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,	/* 0x00 */
-	0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
-	0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,	/* 0x10 */
-	0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
-	0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,	/* 0x20 */
-	0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
-	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,	/* 0x30 */
-	0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
-	0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,	/* 0x40 */
-	0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
-	0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57,	/* 0x50 */
-	0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f,
-	0x60, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,	/* 0x60 */
-	0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
-	0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57,	/* 0x70 */
-	0x58, 0x59, 0x5a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,
-	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,	/* 0x80 */
-	0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
-	0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97,	/* 0x90 */
-	0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
-	0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,	/* 0xa0 */
-	0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
-	0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7,	/* 0xb0 */
-	0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
-	0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7,	/* 0xc0 */
-	0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
-	0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7,	/* 0xd0 */
-	0xd8, 0xd9, 0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
-	0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7,	/* 0xe0 */
-	0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
-	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,	/* 0xf0 */
-	0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
-};
-static u_char lowercase[256] = {
-	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,	/* 0x00 */
-	0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
-	0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,	/* 0x10 */
-	0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
-	0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,	/* 0x20 */
-	0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
-	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,	/* 0x30 */
-	0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
-	0x40, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,	/* 0x40 */
-	0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
-	0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,	/* 0x50 */
-	0x78, 0x79, 0x7a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f,
-	0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,	/* 0x60 */
-	0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
-	0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,	/* 0x70 */
-	0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,
-	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,	/* 0x80 */
-	0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
-	0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97,	/* 0x90 */
-	0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
-	0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,	/* 0xa0 */
-	0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
-	0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7,	/* 0xb0 */
-	0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
-	0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7,	/* 0xc0 */
-	0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
-	0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7,	/* 0xd0 */
-	0xd8, 0xd9, 0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
-	0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7,	/* 0xe0 */
-	0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
-	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,	/* 0xf0 */
-	0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
-};
-static u_char kctypetable[256] = {
-	0200, 0200, 0200, 0200, 0200, 0200, 0200, 0200,	/* 0x00 */
-	0200, 0200, 0200, 0200, 0200, 0200, 0200, 0200,
-	0200, 0200, 0200, 0200, 0200, 0200, 0200, 0200,	/* 0x10 */
-	0200, 0200, 0200, 0200, 0200, 0200, 0200, 0200,
-	   0, 0060, 0060, 0060, 0060, 0060, 0060, 0060,	/* 0x20 */
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,	/* 0x30 */
-	0060, 0060, 0060, 0060, 0060, 0060, 0060, 0060,
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,	/* 0x40 */
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,	/* 0x50 */
-	0062, 0062, 0062, 0062, 0062, 0062, 0062, 0062,
-	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,	/* 0x60 */
-	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,
-	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0022,	/* 0x70 */
-	0022, 0022, 0022, 0022, 0022, 0022, 0022, 0200,
-	0002, 0003, 0003, 0003, 0003, 0003, 0003, 0003,	/* 0x80 */
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,	/* 0x90 */
-	0003, 0003, 0003, 0003, 0003, 0003, 0003, 0003,
-	0002, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xa0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xb0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xc0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,	/* 0xd0 */
-	0016, 0016, 0016, 0016, 0016, 0016, 0016, 0016,
-	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,	/* 0xe0 */
-	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,
-	0013, 0013, 0013, 0013, 0013, 0013, 0013, 0013,	/* 0xf0 */
-	0013, 0013, 0013, 0013, 0013, 0010, 0010,    0
-};
-# endif	/* !FD */
-#endif	/* !LSI_C */
-
 
 #ifndef	FD
 static int NEAR _dospath(path)
@@ -296,7 +167,9 @@ int d;
 	if (d && _dospath(s)) return(s + 1);
 	for (i = 0; s[i]; i++) {
 		if (s[i] == _SC_) return(&(s[i]));
+# ifdef	BSPATHDELIM
 		if (iskanji1(s, i)) i++;
+# endif
 	}
 	return(NULL);
 }
@@ -312,7 +185,9 @@ int d;
 	else cp = NULL;
 	for (i = 0; s[i]; i++) {
 		if (s[i] == _SC_) cp = &(s[i]);
+# ifdef	BSPATHDELIM
 		if (iskanji1(s, i)) i++;
+# endif
 	}
 	return(cp);
 }
@@ -321,15 +196,21 @@ static int NEAR isdelim(s, ptr)
 char *s;
 int ptr;
 {
+# ifdef	BSPATHDELIM
 	int i;
+# endif
 
 	if (ptr < 0 || s[ptr] != _SC_) return(0);
+# ifdef	BSPATHDELIM
 	if (--ptr < 0) return(1);
 	if (!ptr) return(!iskanji1(s, 0));
 
 	for (i = 0; s[i] && i < ptr; i++) if (iskanji1(s, i)) i++;
 	if (!s[i] || i > ptr) return(1);
 	return(!iskanji1(s, i));
+# else
+	return(1);
+# endif
 }
 
 static char *NEAR strcatdelim(s)
@@ -350,7 +231,9 @@ char *s;
 			continue;
 		}
 		cp = NULL;
+# ifdef	BSPATHDELIM
 		if (iskanji1(s, i)) i++;
+# endif
 	}
 	if (!cp) {
 		cp = &(s[i]);
@@ -376,7 +259,7 @@ char *buf, *s1, *s2;
 	if (!s1[i]) cp = &(buf[i]);
 	else if (s1[i] == _SC_ && !s1[i + 1]) {
 		cp = &(buf[i]);
-		*cp = _SC_;
+		*(cp++) = _SC_;
 	}
 	else {
 		cp = NULL;
@@ -387,10 +270,12 @@ char *buf, *s1, *s2;
 				continue;
 			}
 			cp = NULL;
+# ifdef	BSPATHDELIM
 			if (iskanji1(s1, i)) {
 				if (!s1[++i]) break;
 				buf[i] = s1[i];
 			}
+# endif
 		}
 		if (!cp) {
 			cp = &(buf[i]);
@@ -418,7 +303,7 @@ char *s;
 #endif	/* !FD */
 
 static int NEAR seterrno(doserr)
-u_short doserr;
+u_int doserr;
 {
 	int i;
 
@@ -468,7 +353,7 @@ char *path;
 	return(buf);
 }
 
-static long NEAR intcall(vect, regp, sregp)
+int intcall(vect, regp, sregp)
 int vect;
 __dpmi_regs *regp;
 struct SREGS *sregp;
@@ -511,9 +396,9 @@ struct SREGS *sregp;
 	int86x(vect, regp, regp, sregp);
 # endif
 #endif	/* !__TURBOC__ */
-	if (!((*regp).x.flags & FR_CARRY)) return((long)((*regp).x.ax));
+	if (!((*regp).x.flags & FR_CARRY)) return(0);
 	seterrno((*regp).x.ax);
-	return(-1L);
+	return(-1);
 }
 
 int getcurdrv(VOID_A)
@@ -644,7 +529,7 @@ char *path;
 	if (drv[0] >= 'A' && drv[0] <= 'Z') return(0);
 
 	drv[1] = ':';
-	drv[2] = _SC_;
+	drv[2] = '\\';
 	drv[3] = '\0';
 
 	reg.x.ax = 0x71a0;
@@ -690,6 +575,9 @@ int drive;
 	if (int21call(&reg, &sreg) < 0) return(NULL);
 #ifdef	DJGPP
 	dosmemget(__tb, MAXPATHLEN - 3, pathname);
+#endif
+#ifndef	BSPATHDELIM
+	adjustpname(pathname);
 #endif
 	return(pathname);
 }
@@ -751,6 +639,9 @@ char *path, *alias;
 # ifdef	DJGPP
 	dosmemget(__tb + i, MAXPATHLEN, alias);
 # endif
+# ifndef	BSPATHDELIM
+	adjustpname(alias);
+# endif
 
 	if (!path || !(i = _dospath(path))) i = getcurdrv();
 	if (i >= 'a' && i <= 'z') *alias = tolower2(*alias);
@@ -813,6 +704,10 @@ char *path, *resolved;
 #ifdef	DJGPP
 	dosmemget(__tb + i, MAXPATHLEN, resolved);
 #endif
+#ifndef	BSPATHDELIM
+	adjustpname(resolved);
+#endif
+
 	if (!path || !(i = _dospath(path))) i = getcurdrv();
 	else path += 2;
 	if (!_dospath(resolved) || resolved[2] != _SC_) {
@@ -843,6 +738,46 @@ char *path, *resolved;
 	return(resolved);
 }
 
+#ifndef	BSPATHDELIM
+char *adjustpname(path)
+char *path;
+{
+	int i;
+
+	for (i = 0; path[i]; i++) {
+		if (path[i] == '\\') path[i] = _SC_;
+		else if (iskanji1(path, i)) i++;
+	}
+	return(path);
+}
+#endif	/* !BSPATHDELIM */
+
+#if	defined (DJGPP) || !defined (BSPATHDELIM)
+char *adjustfname(path)
+char *path;
+{
+# ifndef	_NOUSELFN
+	char tmp[MAXPATHLEN];
+# endif
+	int i;
+
+# ifndef	_NOUSELFN
+	if (supportLFN(path) > 0 && unixrealpath(path, tmp)) strcpy(path, tmp);
+	else
+# endif
+	for (i = (_dospath(path)) ? 2 : 0; path[i]; i++) {
+# ifdef	BSPATHDELIM
+		if (path[i] == '/') path[i] = _SC_;
+# else
+		if (path[i] == '\\') path[i] = _SC_;
+		else if (iskanji1(path, i)) i++;
+# endif
+		else path[i] = toupper2(path[i]);
+	}
+	return(path);
+}
+#endif	/* DJGPP || !BSPATHDELIM */
+
 #ifndef	_NOUSELFN
 char *preparefile(path, alias)
 char *path, *alias;
@@ -864,23 +799,6 @@ char *path, *alias;
 	return(NULL);
 }
 
-#ifdef	DJGPP
-char *adjustfname(path)
-char *path;
-{
-	char tmp[MAXPATHLEN];
-	int i;
-
-	if (supportLFN(path) > 0 && unixrealpath(path, tmp)) strcpy(path, tmp);
-	else
-	for (i = (_dospath(path)) ? 2 : 0; path[i]; i++) {
-		if (path[i] == '/') path[i] = _SC_;
-		else path[i] = toupper2(path[i]);
-	}
-	return(path);
-}
-#endif	/* DJGPP */
-
 #ifndef	_NODOSDRIVE
 static VOID NEAR biosreset(drive)
 int drive;
@@ -1645,7 +1563,7 @@ int n, size, iswrite;
 
 static int NEAR dos_findfirst(path, attr, result)
 char *path;
-u_short attr;
+u_int attr;
 struct dosfind_t *result;
 {
 	struct SREGS sreg;
@@ -1694,14 +1612,14 @@ struct dosfind_t *result;
 }
 
 #ifndef	_NOUSELFN
-static long NEAR lfn_findfirst(path, attr, result)
+static int NEAR lfn_findfirst(fdp, path, attr, result)
+u_int *fdp;
 char *path;
-u_short attr;
+u_int attr;
 struct lfnfind_t *result;
 {
 	struct SREGS sreg;
 	__dpmi_regs reg;
-	long fd;
 
 	path = duplpath(path);
 	reg.x.ax = 0x714e;
@@ -1714,15 +1632,16 @@ struct lfnfind_t *result;
 	reg.x.dx = PTR_OFF(path, sizeof(struct lfnfind_t));
 	sreg.es = PTR_SEG(result);
 	reg.x.di = PTR_OFF(result, 0);
-	if ((fd = int21call(&reg, &sreg)) < 0) return(-1L);
+	if (int21call(&reg, &sreg) < 0) return(-1);
+	*fdp = reg.x.ax;
 #ifdef	DJGPP
 	dosmemget(__tb, sizeof(struct lfnfind_t), result);
 #endif
-	return(fd);
+	return(0);
 }
 
 static int NEAR lfn_findnext(fd, result)
-u_short fd;
+u_int fd;
 struct lfnfind_t *result;
 {
 	struct SREGS sreg;
@@ -1741,14 +1660,14 @@ struct lfnfind_t *result;
 }
 
 static int NEAR lfn_findclose(fd)
-u_short fd;
+u_int fd;
 {
 	struct SREGS sreg;
 	__dpmi_regs reg;
 
 	reg.x.ax = 0x71a1;
 	reg.x.bx = fd;
-	return((int21call(&reg, &sreg) < 0) ? -1 : 0);
+	return(int21call(&reg, &sreg));
 }
 #endif	/* !_NOUSELFN */
 
@@ -1787,18 +1706,12 @@ char *dir;
 			(struct dosfind_t *)(dirp -> dd_buf));
 		if (i >= 0) dirp -> dd_id |= DID_IFLABEL;
 		else {
-			long fd;
+			u_int fd;
 
-			fd = lfn_findfirst(path, SEARCHATTRS,
+			i = lfn_findfirst(&fd, path, SEARCHATTRS,
 				(struct lfnfind_t *)(dirp -> dd_buf));
-			if (fd >= 0) {
-				dirp -> dd_fd = (u_short)fd;
-				i = 0;
-			}
-			else {
-				dirp -> dd_id &= ~DID_IFLFN;
-				i = -1;
-			}
+			if (i >= 0) dirp -> dd_fd = fd;
+			else dirp -> dd_id &= ~DID_IFLFN;
 		}
 	}
 	else
@@ -1847,7 +1760,7 @@ int unixclosedir(dirp)
 DIR *dirp;
 {
 #ifndef	_NODOSDRIVE
-	if (dirp -> dd_id < 0) return(dosclosedir(dirp));
+	if (dirp -> dd_id == DID_IFDOSDRIVE) return(dosclosedir(dirp));
 #endif
 #ifndef	_NOUSELFN
 	if (dirp -> dd_id & DID_IFLFN) lfn_findclose(dirp -> dd_fd);
@@ -1866,7 +1779,7 @@ DIR *dirp;
 	int i;
 
 #ifndef	_NODOSDRIVE
-	if (dirp -> dd_id < 0) return(dosreaddir(dirp));
+	if (dirp -> dd_id == DID_IFDOSDRIVE) return(dosreaddir(dirp));
 #endif
 	if (dirp -> dd_off < 0) return(NULL);
 	d.d_off = dirp -> dd_off;
@@ -1903,19 +1816,13 @@ DIR *dirp;
 
 #ifndef	_NOUSELFN
 		if (dirp -> dd_id & DID_IFLFN) {
-			long fd;
+			u_int fd;
 
 			strcpy(cp, "*");
-			fd = lfn_findfirst(path, SEARCHATTRS,
+			i = lfn_findfirst(&fd, path, SEARCHATTRS,
 				(struct lfnfind_t *)(dirp -> dd_buf));
-			if (fd >= 0) {
-				dirp -> dd_fd = (u_short)fd;
-				i = 0;
-			}
-			else {
-				dirp -> dd_id &= ~DID_IFLFN;
-				i = -1;
-			}
+			if (i >= 0) dirp -> dd_fd = fd;
+			else dirp -> dd_id &= ~DID_IFLFN;
 		}
 		else
 #endif	/* !_NOUSELFN */
@@ -1944,7 +1851,7 @@ DIR *dirp;
 	char *cp;
 
 #ifndef	_NODOSDRIVE
-	if (dirp -> dd_id < 0) return(dosrewinddir(dirp));
+	if (dirp -> dd_id == DID_IFDOSDRIVE) return(dosrewinddir(dirp));
 #endif
 #ifndef	_NOUSELFN
 	if (dirp -> dd_id & DID_IFLFN) lfn_findclose(dirp -> dd_fd);
@@ -1961,7 +1868,23 @@ DIR *dirp;
 	return(0);
 }
 
-#ifndef	_NOUSELFN
+#ifdef	_NOUSELFN
+# ifndef	DJGPP
+/*ARGSUSED*/
+int unixmkdir(path, mode)
+char * path;
+int mode;
+{
+	struct stat st;
+
+	if (unixstat(path, &st) >= 0) {
+		errno = EEXIST;
+		return(-1);
+	}
+	return(mkdir(path) ? -1 : 0);
+}
+# endif
+#else	/* !_NOUSELFN */
 int unixunlink(path)
 char *path;
 {
@@ -1989,7 +1912,7 @@ char *path;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
-	return((int21call(&reg, &sreg) < 0) ? -1 : 0);
+	return(int21call(&reg, &sreg));
 }
 
 static int NEAR gendosname(s)
@@ -2186,7 +2109,7 @@ char *path;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
-	return((int21call(&reg, &sreg) < 0) ? -1 : 0);
+	return(int21call(&reg, &sreg));
 }
 
 int unixchdir(path)
@@ -2212,13 +2135,13 @@ char *path;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
-	return((int21call(&reg, &sreg) < 0) ? -1 : 0);
+	return(int21call(&reg, &sreg));
 }
 #endif	/* !_NOUSELFN */
 
-char *unixgetcwd(pathname, size, pseudo)
+char *unixgetcwd(pathname, size)
 char *pathname;
-int size, pseudo;
+int size;
 {
 #ifdef	DJGPP
 	char tmp[MAXPATHLEN];
@@ -2227,7 +2150,7 @@ int size, pseudo;
 
 	drive = getcurdrv();
 #ifndef	_NODOSDRIVE
-	if (pseudo && checkdrive(toupper2(drive) - 'A') > 0) {
+	if (checkdrive(toupper2(drive) - 'A') > 0) {
 		if (!dosgetcwd(pathname, size)) return(NULL);
 	}
 	else
@@ -2348,7 +2271,6 @@ statfs_t *buf;
 		buf -> f_blocks = *((u_long *)&(tmp[sizeof(long) * 1]));
 		buf -> f_bfree =
 		buf -> f_bavail = *((u_long *)&(tmp[sizeof(long) * 2]));
-		buf -> f_files = -1;
 	}
 	else
 # endif
@@ -2363,6 +2285,7 @@ statfs_t *buf;
 		sreg.ds = PTR_SEG(path);
 		reg.x.dx = PTR_OFF(path, sizeof(fsbuf));
 		if (int21call(&reg, &sreg) < 0) return(-1);
+
 # ifdef	DJGPP
 		dosmemget(__tb, sizeof(fsbuf), &fsbuf);
 # endif
@@ -2371,7 +2294,6 @@ statfs_t *buf;
 		buf -> f_blocks = (u_long)fsbuf.f_blocks;
 		buf -> f_bfree =
 		buf -> f_bavail = (u_long)fsbuf.f_bavail;
-		buf -> f_files = -1;
 	}
 	else
 #endif	/* !_NOUSELFN */
@@ -2384,19 +2306,19 @@ statfs_t *buf;
 			return(-1);
 		}
 
-		if (!reg.x.ax || !reg.x.cx || !reg.x.dx || reg.x.dx < reg.x.bx)
-			buf -> f_bsize = buf -> f_blocks =
-			buf -> f_bfree = buf -> f_bavail = -1L;
-		else {
-			buf -> f_bsize = (u_long)(reg.x.ax)
-				* (u_long)(reg.x.cx);
-			buf -> f_blocks = (u_long)(reg.x.dx);
-			buf -> f_bfree =
-			buf -> f_bavail = (u_long)(reg.x.bx);
-		}
-		buf -> f_files = -1;
+		buf -> f_bsize = (u_long)(reg.x.ax) * (u_long)(reg.x.cx);
+		buf -> f_blocks = (u_long)(reg.x.dx);
+		buf -> f_bfree =
+		buf -> f_bavail = (u_long)(reg.x.bx);
 	}
 
+	if (!(buf -> f_blocks) || !(buf -> f_bsize)
+	|| buf -> f_blocks < buf -> f_bavail) {
+		errno = EIO;
+		return(-1);
+	}
+	buf -> f_files = -1L;
+
 	return(0);
 }
 
@@ -2406,7 +2328,7 @@ struct stat *stp;
 {
 #ifndef	_NOUSELFN
 	struct lfnfind_t lbuf;
-	long fd;
+	u_int fd;
 #endif
 	struct dosfind_t dbuf;
 	int i;
@@ -2430,7 +2352,7 @@ struct stat *stp;
 	}
 # endif	/* !_NODOSDRIVE */
 	if (i <= 0) {
-		fd = -1;
+		fd = (u_int)-1;
 		if ((i = dos_findfirst(path, SEARCHATTRS, &dbuf)) < 0) {
 			if (errno && errno != ENOENT) return(-1);
 			if (!strcmp(path, ".."))
@@ -2439,16 +2361,12 @@ struct stat *stp;
 		}
 	}
 	else {
-		if ((fd = lfn_findfirst(path, SEARCHATTRS, &lbuf)) < 0) {
+		if ((i = lfn_findfirst(&fd, path, SEARCHATTRS, &lbuf)) < 0) {
 			if (errno && errno != ENOENT) return(-1);
-			if (!strcmp(path, "..")) {
-				fd = lfn_findfirst(".", SEARCHATTRS, &lbuf);
-				i = (fd >= 0) ? 0 : -1;
-			}
-			else {
-				fd = -1;
-				i = dos_findfirst(path, DS_IFLABEL, &dbuf);
-			}
+			if (!strcmp(path, ".."))
+				i = lfn_findfirst(&fd, ".",
+					SEARCHATTRS, &lbuf);
+			else i = dos_findfirst(path, DS_IFLABEL, &dbuf);
 		}
 	}
 #endif	/* !_NOUSELFN */
@@ -2458,7 +2376,7 @@ struct stat *stp;
 	}
 
 #ifndef	_NOUSELFN
-	if (fd >= 0) {
+	if (fd != (u_int)-1) {
 		stp -> st_mode = getdosmode(lbuf.attr);
 		stp -> st_mtime = getdostime(lbuf.wrdate, lbuf.wrtime);
 		stp -> st_size = lbuf.size_l;
@@ -2512,7 +2430,7 @@ int mode;
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
-	return((int21call(&reg, &sreg) < 0) ? -1 : 0);
+	return(int21call(&reg, &sreg));
 }
 
 #ifndef	_NOUSELFN
@@ -2521,21 +2439,20 @@ int unixutime(path, times)
 char *path;
 struct utimbuf *times;
 {
-	time_t t = times -> modtime;
+	time_t t;
 	__dpmi_regs reg;
 	struct SREGS sreg;
 	char buf[MAXPATHLEN];
-	int i;
+	int i, fd;
 
+	t = times -> modtime;
 	path = duplpath(path);
 	i = supportLFN(path);
-	if (i <= -3) return(utime(path, times));
-	else if (i < 0) {
+	if (i < 0 && i > -3) {
 # ifndef	_NODOSDRIVE
 		return(dosutime(regpath(path, buf), times));
 # else
 		if (!(path = preparefile(path, buf))) return(-1);
-		return(utime(path, times));
 # endif
 	}
 #else	/* !USEUTIME */
@@ -2543,24 +2460,32 @@ int unixutimes(path, tvp)
 char *path;
 struct timeval tvp[2];
 {
-	time_t t = tvp[1].tv_sec;
+	time_t t;
 	__dpmi_regs reg;
 	struct SREGS sreg;
 	char buf[MAXPATHLEN];
-	int i;
+	int i, fd;
 
+	t = tvp[1].tv_sec;
 	path = duplpath(path);
 	i = supportLFN(path);
-	if (i <= -3) return(utimes(path, tvp));
-	else if (i < 0) {
+	if (i < 0 && i > -3) {
 # ifndef	_NODOSDRIVE
 		return(dosutimes(regpath(path, buf), tvp));
 # else
 		if (!(path = preparefile(path, buf))) return(-1);
-		return(utimes(path, tvp));
 # endif
 	}
 #endif	/* !USEUTIME */
+	if (i <= 0) {
+		if ((fd = open(path, O_RDONLY, 0666)) < 0) return(-1);
+		reg.x.ax = 0x5701;
+		reg.x.bx = (u_short)fd;
+		putdostime(&(reg.x.dx), &(reg.x.cx), t);
+		i = int21call(&reg, &sreg);
+		close(fd);
+		return(i);
+	}
 
 	reg.x.ax = 0x7143;
 	reg.h.bl = 0x03;
@@ -2570,7 +2495,8 @@ struct timeval tvp[2];
 #endif
 	sreg.ds = PTR_SEG(path);
 	reg.x.dx = PTR_OFF(path, 0);
-	return((int21call(&reg, &sreg) < 0) ? -1 : 0);
+
+	return(int21call(&reg, &sreg));
 }
 
 int unixopen(path, flags, mode)
@@ -2596,8 +2522,8 @@ int flags, mode;
 
 	reg.x.ax = 0x716c;
 	reg.x.bx = 0x0110;	/* SH_DENYRW | NO_BUFFER */
-	if (flags & O_WRONLY) reg.x.bx |= 0x0001;
-	else if (flags & O_RDWR) reg.x.bx |= 0x0002;
+	if ((flags & O_ACCMODE) == O_WRONLY) reg.x.bx |= 0x0001;
+	else if ((flags & O_ACCMODE) == O_RDWR) reg.x.bx |= 0x0002;
 	reg.x.cx = (u_short)putdosmode(mode);
 	if (flags & O_CREAT) {
 		if (flags & O_EXCL) {
